void Plot()
{
   //====================================================================
   // ---------------
   // Constants
   // ---------------
   const Double_t kMassW   =   80.4; // W mass by s.y
   const Double_t kSigmaMw =    4.0; // W mass resolution
   const Double_t kMassZ   = 91.187; // Z mass
   const Double_t kSigmaMz =    4.0; // Z mass resolution
   const Double_t kMassH   =  120.0; // H mass
   const Double_t kSigmaMh =    4.0; // H mass resolution
   // ---------------
   // Selection Cuts
   // ---------------
   const Double_t xEvis    = 220.0; // Evis cut [GeV}
   const Double_t xPt      = 999.0; // Pt   cut [GeV}
   const Double_t xPl      = 999.0; // Pl   cut [GeV]
   const Double_t xM2j     =  40.0; // mass cut [GeV]
#if 1
   const Double_t xCosj    =   0.8; // cos(theta_J) cut
   const Int_t    xNpsj    =    10; // cut on #particles/jet
#else
   const Double_t xCosj    =  0.95; // cos(theta_J) cut
   const Int_t    xNpsj    =     0; // cut on #particles/jet
#endif
   //====================================================================
   //------------------------------------------------------
   // Prepare a canvas
   //------------------------------------------------------
   TCanvas *c1p = new TCanvas("Jetanal","Jet Analysis",20,20,920,620);
   TCanvas  &c1  = *c1p;
   c1.Divide(3,2);
   Int_t id = 1;

   //------------------------------------------------------
   // Get TNtuple
   //------------------------------------------------------
   // TNtupleD("hjet","QQAnalysis",
   //          "evis:pt:pl:cosj1:cosj2:ej1:ej2:npj1:npj2:mmass:zmass");
   //
   TNtupleD &hjet = *static_cast<TNtupleD *>(gROOT->FindObject("hjet"));

   stringstream scuts;
#if 0
   scuts.str("");
   scuts.clear(stringstream::goodbit);
#endif
   //------------------------------------------------------
   // Evis cut
   //------------------------------------------------------
   scuts << "evis<" << xEvis;

   c1.cd(id++); hjet.Draw("evis",scuts.str().data());

   //------------------------------------------------------
   // Momentum balance cuts
   //------------------------------------------------------
   scuts << "&&pt<" << xPt << "&&abs(pl)<" << xPl;

   c1.cd(id++); hjet.Draw("pl:pt",scuts.str().data());
   c1.cd(id++); hjet.Draw("ej2:ej1",scuts.str().data());
   c1.cd(id++); hjet.Draw("cosj2:cosj1",scuts.str().data());
   c1.cd(id++); hjet.Draw("npj2:npj1",scuts.str().data());

   //------------------------------------------------------
   // Jet quality cuts
   //------------------------------------------------------
   scuts << "&&abs(cosj1)<" << xCosj << "&&abs(cosj2)<" << xCosj
         << "&&npj1>" << xNpsj << "&&npj2>" << xNpsj << ends;

   c1.cd(id++); hjet.Draw("mmass:zmass",scuts.str().data());
}
