//*LastUpdate : v.01.01 undefined by undefined
//*-- Author  : undefined undefined

///////////////////////////////////////////////////////////////////
//
//  MyPFOAnalysis
//
//  Sample PFO analysis program
//
//$Id: 
//  
//////////////////////////////////////////////////////////////////

#include <iostream>

#include "JSFSteer.h"
#include "MyPFOAnalysis.h"
#include "JSFGenerator.h"
#include "S4PFObject.h"
#include "S4PFOMakerBuf.h"


ClassImp(MyPFOAnalysis)

//_____________________________________________________________________________
  MyPFOAnalysis::MyPFOAnalysis(const char *name, const char *title)
       : JSFModule(name,title)
{
}

//_____________________________________________________________________________
MyPFOAnalysis::~MyPFOAnalysis()
{
}

//_____________________________________________________________________________
Bool_t MyPFOAnalysis::Initialize()
{
  // Initialize
  fNtuple=new TNtuple("fNtuple","PFO Events Ntuple","npfo:nch:neu:nflv:csjmax:ecm:e:px:py:pz:m:mm");

  return kTRUE;
}

//_____________________________________________________________________________
Bool_t MyPFOAnalysis::Process(Int_t nev)
{
// Acess Generator Information

  JSFGenerator *gen=(JSFGenerator*)gJSF->FindModule("JSFGenerator");
  JSFGeneratorBuf *genbuf=(JSFGeneratorBuf*)gen->EventBuf();
  Double_t ecm = gen != 0  ? genbuf->GetEcm() : 0.0 ;

  TObjArray *buf=genbuf->GetParticles();
  JSFGeneratorParticle *p=0;
  JSFGeneratorParticle *pm=0;
  TIter next(buf);
  Int_t nflv=0;
  Float_t csj=-10.0;
  Float_t csjmax=0.0;
  while( (p=(JSFGeneratorParticle*)next()) ) {
   Int_t idm=-p->GetMother();
   if( idm <= 0 ) continue;
   pm=(JSFGeneratorParticle*)buf->UncheckedAt(idm-1);
   if( pm->GetID() == 23 ) {
      Double_t pabs=TMath::Sqrt(p->GetPx()*p->GetPx()+
	         p->GetPy()*p->GetPy()+p->GetPz()*p->GetPz());
      csj=TMath::Abs(p->GetPz()/pabs);
      if( csj > csjmax ) csjmax=csj; 
      Int_t aid=TMath::Abs(p->GetID()); 
      if ( aid <=3  ) { 
        nflv += 1;  
        } 
      else if ( aid == 4 ) {  nflv += 10; }
      else if ( aid == 5 ) { nflv += 100; }
   }
  }

  //  if ( nflv < 2 ) return kTRUE;

  S4PFOMaker *pfom=(S4PFOMaker*)gJSF->FindModule("S4PFOMaker");
  S4PFOMakerBuf *pfombuf=(S4PFOMakerBuf*)pfom->EventBuf();
  pfombuf->UpdatePFOs();  // Call UpdatePFOs to use latest information
  Int_t npfo=pfombuf->GetEntries();
  TObjArray *pfos=pfombuf->GetOutputBufPtr();

   TIter nextpfo(pfos);
   S4PFObject *pfo;
   ANL4DVector psum(0.0,0.0,0.0,0.0);
   Int_t nch=0;
   Int_t nneu=0;
   
   
   while ((pfo = (S4PFObject *) (nextpfo()))) {
     psum += *pfo;
     Double_t charge=pfo->GetCharge();
     if ( TMath::Abs(charge) < 0.1 ) { nneu += 1; }
     else { nch += 1; }
   }
   ANL4DVector pcm(ecm,0.0,0.0,0.0);
   ANL4DVector pmis=pcm-psum;
   fNtuple->Fill((Float_t)npfo,(Float_t)nch,(Float_t)nneu,
	(Float_t)nflv,(Float_t)csj,(Float_t)ecm,
       (Float_t)psum.E(),(Float_t)psum.Px(),(Float_t)psum.Py(),
	(Float_t)psum.Pz(),(Float_t)psum.GetMass(), (Float_t)pmis.GetMass() );
   return kTRUE;
}

