///////////////////////////////////////////////////////////////////
//
//  Run Manager of Geant4 simulation for JSFJ4
//
//
//$Id: JSFJ4RunManager.cxx,v 1.3 2008/03/25 15:15:51 fujiik Exp $
//
//////////////////////////////////////////////////////////////////


#include "G4Timer.hh"
#include "G4ApplicationState.hh"
#include "G4StateManager.hh"
#include "G4UImanager.hh"
#include "G4Event.hh"

using namespace std;

#include "J4PrimaryGeneratorAction.hh"
#include "JSFJ4RunManager.h"
#include "JSFJ4PrimaryGeneratorAction.h"
#include "JSFSteer.h"
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif


//_____________________________________________________________________________
JSFJ4RunManager::JSFJ4RunManager() :
   fNSelectEvent(0)
{
}

//_____________________________________________________________________________
JSFJ4RunManager::~JSFJ4RunManager()
{
}

//_____________________________________________________________________________
void JSFJ4RunManager::RunInitialize()
{
  //
  bool cond = ConfirmBeamOnCondition();
  if( !cond ) {
    cout << "Jupiter run manager is not ready for run " ;
    cout << " at JSFG4::Initialize() " << endl;
    exit(0);
  }
 G4RunManager::RunInitialization();

 if( verboseLevel > 0 ) { timer->Start() ; }


}

//_____________________________________________________________________________
bool JSFJ4RunManager::SimulateOneEvent(int ievent)
{
  //

  G4int ie=ievent-1;
  fCurrentEvent = GenerateEvent(ie);
  currentEvent = fCurrentEvent;
  eventManager->ProcessOneEvent(fCurrentEvent);
  
  AnalyzeEvent(fCurrentEvent);

  ((JSFJ4PrimaryGeneratorAction*)GetUserPrimaryGeneratorAction())->SavePrimaries(fCurrentEvent);

  StackPreviousEvent(fCurrentEvent);

//  if(runAborted) {
  if( runAborted || 
      J4PrimaryGeneratorAction::GetPrimaryGeneratorAction()->AbortRun() ) { 
    return false;
  } 

  return true;

}

//_____________________________________________________________________________
void JSFJ4RunManager::AbortEvent()
{
  G4RunManager::AbortEvent();
  gJSF->SetReturnCode(JSFSteer::kJSFSkipRestModules|JSFSteer::kJSFNoOutput);

}
//_____________________________________________________________________________
void JSFJ4RunManager::RunTermination()
{

  if( verboseLevel > 0 ) {
    timer->Stop();
    G4cout << "JSFJ4 Run terminated." << G4endl;
    G4cout << "JSFJ4 Run Summary" << G4endl;
    //  if(runAborted)
    //    { G4cout << "  Run Aborted after " << i_event << " events processed." << G4endl; }
    //    else
    //    { G4cout << "  Number of events processed : " << n_event << G4endl; }
    G4cout << "  "  << *timer << G4endl;
  }

  G4RunManager::RunTermination();
}

