//*************************************************************************
//* --------------------
//* S4BabyExactHit
//* --------------------
//* (Description)
//*	Baby Hit class.
//*	Mother class : TObject
//*     
//* (Update Record)
//*	2001/03/15  K.Hoshina	Original version.
//*************************************************************************

#include <string.h>
#include <fstream>
#include "TMath.h"

#include "S4BabyExactHit.h"

ClassImp(S4BabyExactHit)

Int_t S4BabyExactHit::fBabyExactHitSortBy = 0; 

//=====================================================================
//* constructor -------------------------------------------------------

S4BabyExactHit::S4BabyExactHit(	Int_t evtNumber,
		                        Int_t layerNumber,
					Int_t wireNumber, 
   	      		     		Int_t cellNumber, 
   	      		     		Int_t trackNumber,
   	      		     		Int_t mothertrackN,
   	      		     		Int_t hitnumber,
    	      		     		Int_t pcode,
    	      		     		Int_t charge, 
    	      		     		Double_t wireX, 
    	      		     		Double_t wireY, 
    	      		     		Double_t wireZ, 
    	      		     		Double_t driftlen, 
    	      		     		const TVector3 &pre, 
    	      		     		const TVector3 &post, 
    	      		     		const TVector3 &hit, 
    	      		     		const TVector3 &momentum,
    	      		     		Double_t totE, 
    	      		     		Double_t edep, 
    	      		     		Double_t tof ) 
{
	SetEventID	     (evtNumber);
	SetTrackID	     (trackNumber);
	SetMotherTrackID (mothertrackN);
	SetPDGCode	(pcode);
	SetCharge 	(charge);
	SetPrePosition	(pre);
	SetPostPosition	(post);
	SetPosition	(hit);
	SetMomentum	(momentum);
	SetEnergyDeposit(edep);
	SetTotalEnergy	(totE);
	SetTOF		(tof);
	SetHitID	(hitnumber);
		
	fLayerNumber    = layerNumber;
	fWireNumber 	= wireNumber;
	fCellNumber  	= cellNumber;
	fWireX		= wireX;
	fWireY		= wireY;
	fWireZ		= wireZ;
	fDriftLen	= driftlen;
	fDriftRegionNumber = fWireNumber + fLayerNumber*__BabyNWIRESPERCELL__;	
}


//=====================================================================
//* destructor --------------------------------------------------------

S4BabyExactHit::~S4BabyExactHit()
{
}

//=====================================================================
//* SetSortBy --------------------------------------------------------

void S4BabyExactHit::SetSortBy(Char_t *opt )
{
	switch (opt[2]) {
		case 'a':	// sort by "TrackNumber"
			fBabyExactHitSortBy = 0;
			break;
		case 'm': 	// sort by "Time"
			fBabyExactHitSortBy = 1;
			break;
		case 'e': 	// sort by "Energy"
			fBabyExactHitSortBy = 2;
			break;
		case 'i': 	// sort by "DriftLength"
			fBabyExactHitSortBy = 3;
			break;
		case 'l': 	// sort by "CellNumber"
			fBabyExactHitSortBy = 4;
			break;
		case 'r': 	// sort by "WireLayer"
			fBabyExactHitSortBy = 5;
			break;
		case 't': 	// sort by "HitNumber"
			fBabyExactHitSortBy = 6;
			break;
		default: 	// sort by "WireLayer"
			fBabyExactHitSortBy = 5;
	}		
}

//=====================================================================
//* IsSoatable --------------------------------------------------------

Bool_t S4BabyExactHit::IsSortable() const { return kTRUE; }
	
//=====================================================================
//* Compare -----------------------------------------------------------
	
Int_t S4BabyExactHit::Compare(const TObject *obj) const
{
  switch (fBabyExactHitSortBy) {
     case 0: { 
   	if (GetTrackID() < ((S4BabyExactHit *)obj)->GetTrackID()) return -1;
   	else if (GetTrackID() > ((S4BabyExactHit *)obj)->GetTrackID()) return 1;
  	else return 0;
  	break; }
     case 1: {
   	if (GetTOF() < ((S4BabyExactHit *)obj)->GetTOF()) return -1;
   	else if (GetTOF() > ((S4BabyExactHit *)obj)->GetTOF()) return 1;
  	else return 0;
  	break; }
     case 2: {
   	if (GetTotalEnergy() < ((S4BabyExactHit *)obj)->GetTotalEnergy()) return -1;
   	else if (GetTotalEnergy() > ((S4BabyExactHit *)obj)->GetTotalEnergy()) return 1;
  	else return 0; }
  	break;
     case 3: {
     	Double_t driftlen  = TMath::Abs(fDriftLen);
     	Double_t driftleno = TMath::Abs(((S4BabyExactHit *)obj)->fDriftLen);    	
   	if (driftlen < driftleno) return -1;
   	else if (driftlen > driftleno) return 1;
  	else return 0;
  	break; }
     case 4: {
     	Int_t cellno  = fCellNumber;
     	Int_t cellnoo = ((S4BabyExactHit *)obj)->fCellNumber;
   	if (cellno < cellnoo ) return -1;
   	else if (cellno > cellnoo) return 1;
  	else return 0;
  	break; }
     case 5: {
        Int_t drNumber    = fDriftRegionNumber;
        Int_t objdrNumber = ((S4BabyExactHit *)obj)->fDriftRegionNumber;
   	if (drNumber < objdrNumber) return -1;
   	else if (drNumber > objdrNumber) return 1;
  	else return 0;
  	break;}
     case 6: {
        Int_t hitNumber    = GetHitID();
        Int_t objhitNumber = ((S4BabyExactHit *)obj)->GetHitID();
   	if (hitNumber < objhitNumber) return -1;
   	else if (hitNumber > objhitNumber) return 1;
  	else return 0;
  	break;}
     default: {
        Int_t drNumber    = fDriftRegionNumber;
        Int_t objdrNumber = ((S4BabyExactHit *)obj)->fDriftRegionNumber;
   	if (drNumber < objdrNumber) return -1;
   	else if (drNumber > objdrNumber) return 1;
  	else return 0;
  	break;}
  }
}










