#ifndef __S4BabyEXACTHIT__
#define __S4BabyEXACTHIT__
//*************************************************************************
//* --------------------
//* S4BabyExactHit
//* --------------------
//* (Description)
//*	Baby Hit class.
//*	Mother class : TObject
//*     
//* (Update Record)
//*	2002/08/27  R.Kuboshima	  
//*************************************************************************

#include <iostream>
#include <iomanip>

#include "TObject.h"
#include "TVector3.h"
#include "TString.h"

#include "S4VExactHit.h"

#define __BabyNWIRESPERCELL__  5

class S4BabyExactHit : public S4VExactHit {

public:
    
  S4BabyExactHit(Int_t evtNumber, 
		Int_t layerNumber, 
  		Int_t wireNumber, 
   	        Int_t cellNumber,
   	        Int_t trackNumber,
   	        Int_t mothertrackN,
   	        Int_t hitnumber,
    	        Int_t pcode, 
    	        Int_t charge,
		Double_t wirex,
		Double_t wirey,
		Double_t wirez,
    	        Double_t driftlen, 
    	        const TVector3 &pre, 
    	        const TVector3 &post, 
    	        const TVector3 &hit, 
    	        const TVector3 &momentum,
    	        Double_t totE, 
    	        Double_t edep, 
    	        Double_t tof    );  
    	        
  inline       S4BabyExactHit (const S4BabyExactHit& right);    
  inline const S4BabyExactHit& operator=(const S4BabyExactHit& right);
  virtual ~S4BabyExactHit();
  
  
  virtual void Print(){};
  
  static void	SetSortBy(Char_t *opt = "WireLayer");
  Bool_t	IsSortable() const;
  Int_t         Compare(const TObject *obj) const;  
  
  
  //Get method -----------//
  
  inline virtual Int_t    GetLayerNumber()        const ;
  inline virtual Int_t 	  GetWireNumber ()	  const ;
  inline virtual Int_t 	  GetCellNumber ()   	  const ;
  inline virtual Double_t GetDriftLen   () 	  const ; 
  inline virtual Double_t GetWireX      ()        const ;
  inline virtual Double_t GetWireY      ()        const ;
  inline virtual Double_t GetWireZ      ()        const ;
  inline virtual Int_t 	  GetDriftRegionNumber () const ;  
  inline virtual Bool_t   GetFlag       ()        const { return fFlag       ; }
  inline virtual const    TVector3 & GetPrePosition() const
                                           { return fPrePosition      ; }
  inline virtual const    TVector3 & GetPostPosition() const
                                           { return fPostPosition     ; }


						
  
  //Set method -----------//
  
  inline virtual void SetLayerNumber(Int_t layerno) ;
  inline virtual void SetWireNumber (Int_t wireno)  ;
  inline virtual void SetCellNumber (Int_t cellno)  ;
  inline virtual void SetDriftLen   (Double_t drlen);
  inline virtual void SetWireX      (Double_t wx)   ;
  inline virtual void SetWireY      (Double_t wy)   ;
  inline virtual void SetWireZ      (Double_t wz)   ;
  inline virtual void SetFlag       (Bool_t f)       { fFlag = f; }
  inline virtual void SetPrePosition (const TVector3 &x) { fPrePosition  = x; }
  inline virtual void SetPostPosition(const TVector3 &x) { fPostPosition = x; }

  
private:
 
  Int_t         fLayerNumber;
  Int_t		fWireNumber;
  Int_t		fCellNumber;
  Int_t 	fDriftRegionNumber;
  Double_t	fWireX;
  Double_t	fWireY;
  Double_t	fWireZ;
  Double_t	fDriftLen;
  Bool_t        fFlag             ; //
  TVector3      fPrePosition      ; //
  TVector3      fPostPosition     ; //
  static Int_t	fBabyExactHitSortBy;  
  
  	
 ClassDef(S4BabyExactHit, 1) // Datum of Baby Hits
};




//=====================================================================
//---------------------------------
// inline function for S4VExactHit
//---------------------------------

inline S4BabyExactHit::S4BabyExactHit(const S4BabyExactHit& right)
{

  fLayerNumber  = right.fLayerNumber    ;
  fWireNumber	= right.fWireNumber	;
  fCellNumber   = right.fCellNumber	; 
  fWireX        = right.fWireX     	;
  fWireY        = right.fWireY     	;
  fWireZ        = right.fWireZ     	;
  fDriftLen     = right.fDriftLen	;
  fDriftRegionNumber = right.fDriftRegionNumber ; 
            
}

inline const S4BabyExactHit&
        S4BabyExactHit::operator= (const S4BabyExactHit& right)
{

  fLayerNumber  = right.fLayerNumber    ;
  fWireNumber	= right.fWireNumber	;
  fCellNumber   = right.fCellNumber     ; 
  fWireX        = right.fWireX     	;
  fWireY        = right.fWireY     	;
  fWireZ        = right.fWireZ     	;
  fDriftLen     = right.fDriftLen	;
  fDriftRegionNumber = right.fDriftRegionNumber ; 
            
  return *this;
}

//===========================================================
// Get method -----------------------------------------------

inline  Int_t S4BabyExactHit::GetLayerNumber () const
{
        return fLayerNumber;
}

inline  Int_t S4BabyExactHit::GetWireNumber () const
{
	return fWireNumber;
}

inline  Int_t S4BabyExactHit::GetCellNumber () const
{
	return fCellNumber;
}

inline  Double_t S4BabyExactHit::GetDriftLen () const
{
	return fDriftLen;
}

inline  Double_t S4BabyExactHit::GetWireX () const
{
	return fWireX;
}

inline  Double_t S4BabyExactHit::GetWireY () const
{
	return fWireY;
}

inline  Double_t S4BabyExactHit::GetWireZ () const
{
	return fWireZ;
}

inline  Int_t S4BabyExactHit::GetDriftRegionNumber () const
{
	return fDriftRegionNumber;
}

//===========================================================
// Set method -----------------------------------------------

inline  void S4BabyExactHit::SetLayerNumber (Int_t layerno) 
{
        fLayerNumber = layerno;
        fDriftRegionNumber 
           = fLayerNumber*__BabyNWIRESPERCELL__ + fWireNumber;
}

inline  void S4BabyExactHit::SetWireNumber (Int_t wireno)
{
	fWireNumber = wireno;
	fDriftRegionNumber 
	   = fLayerNumber*__BabyNWIRESPERCELL__ + fWireNumber;
}

inline  void S4BabyExactHit::SetCellNumber (Int_t cellno) 
{
	fCellNumber = cellno;
}

inline  void S4BabyExactHit::SetWireX (Double_t wx) 
{
	fWireX    = wx;
}

inline  void S4BabyExactHit::SetWireY (Double_t wy) 
{
	fWireY    = wy;
}

inline  void S4BabyExactHit::SetWireZ (Double_t wz) 
{
	fWireZ    = wz;
}

inline  void S4BabyExactHit::SetDriftLen (Double_t drlen) 
{
	fDriftLen = drlen;
}





#endif
