//*************************************************************************
//* --------------------
//* S4CDCExactHit
//* --------------------
//* (Description)
//*	CDC Hit class.
//*	Mother class : TObject
//*     
//* (Update Record)
//*	2001/03/15  K.Hoshina	Original version.
//*	2004/08/26  K.Fujii	Removed temporary treatment.
//*************************************************************************

#include <string.h>
#include <fstream>
#include "TMath.h"

#include "S4CDCExactHit.h"
#ifndef __HasJSFFromLCIO__
#include "J4VCDCDriftRegion.hh"
#include "J4VCDCDriftRegionHit.hh"
#endif

//#define __DEBUG__
#ifdef __DEBUG__
#include "U4CDCMeasLayer.h"
#include "U4CDCKalDetector.h"
#endif

#define __MMTOCM__   0.100
#define __MEVTOGEV__ 0.001

ClassImp(S4CDCExactHit)

Int_t S4CDCExactHit::fCDCExactHitSortBy = 0; 

//=====================================================================
//* constructor -------------------------------------------------------
#ifndef __HasJSFFromLCIO__
S4CDCExactHit::S4CDCExactHit(const J4VCDCDriftRegionHit *hitp)
{
    J4VCDCDriftRegion *cpt = (J4VCDCDriftRegion *)hitp->GetComponent();
    TVector3 pre  (hitp->GetPrePosition().x(),
                   hitp->GetPrePosition().y(),
                   hitp->GetPrePosition().z());
    TVector3 post (hitp->GetPostPosition().x(),
                   hitp->GetPostPosition().y(),
                   hitp->GetPostPosition().z());
    TVector3 mom  (hitp->GetMomentum().x(),
                   hitp->GetMomentum().y(),
                   hitp->GetMomentum().z());
    TVector3 wendm(hitp->GetWireEnd(0).x(),
                   hitp->GetWireEnd(0).y(),
                   hitp->GetWireEnd(0).z());
    TVector3 wendp(hitp->GetWireEnd(1).x(),
                   hitp->GetWireEnd(1).y(),
                   hitp->GetWireEnd(1).z());

   *this = S4CDCExactHit(0,                           // event No.
                         cpt->GetGlobalLayerNumber(), // layer No.
                         cpt->GetMyID(),              // wire No.
                         hitp->GetCloneID(),          // cell No.
                         hitp->GetTrackID(),          // track ID
                         hitp->GetMotherTrackID(),    // mother track ID
                         hitp->GetHitNumber(),        // hit No.
                         hitp->GetPDGEncoding(),      // PID
                         (Int_t)hitp->GetCharge(),    // charge
                         cpt->GetDeltaPhi(cpt),       // cell dphi
                         -0.5*cpt->GetDeltaPhi(cpt),  // cell start phi
              __MMTOCM__*pre,                         // pre position
              __MMTOCM__*post,                        // post position
            __MEVTOGEV__*mom,                         // momentum
            __MEVTOGEV__*hitp->GetTotalEnergy(),      // total energy
            __MEVTOGEV__*hitp->GetEnergyDeposit(),    // energy deposit
                         hitp->GetTof(),              // TOF
              __MMTOCM__*wendm,                       // -ve z end of wire
              __MMTOCM__*wendp,                       // +ve z end of wire
                         hitp->GetTanStereo(),        // tan(stereo)
              __MMTOCM__*hitp->GetRwaist());          // r_waist
}
#endif

S4CDCExactHit::S4CDCExactHit(     Int_t     evtNumber   ,
                                  Int_t     layerNumber , 
                                  Int_t     wireNumber  , 
                                  Int_t     cellNumber  ,
                                  Int_t     trackNumber ,
                                  Int_t     mothertrackN,
                                  Int_t     hitnumber   ,
                                  Int_t     pcode       , 
                                  Int_t     charge      ,
                                  Double_t  cellDphi    , 
                                  Double_t  cellSphi    , 
                                  Double_t  wirePhi     , 
                                  Double_t  wireR       ,
                                  Double_t  driftlen    , 
                            const TVector3 &pre         , 
                            const TVector3 &post        , 
                            const TVector3 &hit         , 
                            const TVector3 &momentum    ,
                                  Double_t  totE        , 
                                  Double_t  edep        , 
                                  Double_t  tof         ,
                            const TVector3 &wireEndm    , 
                            const TVector3 &wireEndp    ,
                                  Double_t  tanStereo   ,
                                  Double_t  rwaist )  
{
   *this = S4CDCExactHit(evtNumber, layerNumber, wireNumber,
                         cellNumber, trackNumber, mothertrackN,
                         hitnumber, pcode, charge, cellDphi, cellSphi,
                         pre, post, momentum, totE, edep, tof,
                         wireEndm, wireEndp, tanStereo, rwaist);
}

S4CDCExactHit::S4CDCExactHit(     Int_t     evtNumber   ,
                                  Int_t     layerNumber , 
                                  Int_t     wireNumber  , 
                                  Int_t     cellNumber  ,
                                  Int_t     trackNumber ,
                                  Int_t     mothertrackN,
                                  Int_t     hitnumber   ,
                                  Int_t     pcode       , 
                                  Int_t     charge      ,
                                  Double_t  cellDphi    , 
                                  Double_t  cellSphi    , 
                            const TVector3 &pre         , 
                            const TVector3 &post        , 
                            const TVector3 &momentum    ,
                                  Double_t  totE        , 
                                  Double_t  edep        , 
                                  Double_t  tof         ,
                            const TVector3 &wireEndm    , 
                            const TVector3 &wireEndp    ,
                                  Double_t  tanStereo ,
                                  Double_t  rwaist )  
{
   SetEventID       (evtNumber);
   SetTrackID       (trackNumber);
   SetMotherTrackID (mothertrackN);
   SetPDGCode           (pcode);
   SetCharge            (charge);
   SetPrePosition       (pre);
   SetPostPosition      (post);
   SetMomentum          (momentum);
   SetEnergyDeposit     (edep);
   SetTotalEnergy       (totE);
   SetTOF               (tof);
   SetHitID         (hitnumber);

   TVector3 hitpos     = 0.5 * (pre + post);
   SetPosition      (hitpos);
	
   fLayerNumber         = layerNumber;
   fWireNumber          = wireNumber;
   fCellNumber          = cellNumber;
   fCellID              = fCellNumber + fLayerNumber*__CDCCELLIDORDER__;
   fCellDphi            = cellDphi;
   fCellSphi            = cellSphi;
   fWireEndAtMinusZ     = wireEndm;
   fWireEndAtPlusZ      = wireEndp;
   fTanStereo           = tanStereo;
   fRwaist              = rwaist;
   fDriftRegionNumber   = fWireNumber + fLayerNumber*__CDCNWIRESPERCELL__;

   TVector3 xx;
   TVector3 d   = fWireEndAtPlusZ - fWireEndAtMinusZ;
   fDriftLen    = DistanceToLine(hitpos, fWireEndAtMinusZ, d, xx);
   fWireR       = wireEndp.Perp();
   fWirePhi     = xx.Phi();

   //left right
   Double_t hitposphi = hitpos.Phi();
   Double_t dphi      = hitposphi - fWirePhi;
   while (dphi >=  TMath::Pi()) dphi -= 2*TMath::Pi();
   while (dphi <  -TMath::Pi()) dphi += 2*TMath::Pi();
   if ((dphi) < 0) fDriftLen = -fDriftLen;

#ifdef __DEBUG__
   Int_t           lno   = layerNumber * 5 + wireNumber;
   U4CDCMeasLayer *msPtr = U4CDCKalDetector::GetInstance()->GetMeasLayerPtr(lno);
   U4CDCMeasLayer &ms    = *msPtr;

   Int_t lr     = 0;
   Int_t cellno = -999999999;

   TKalMatrix h    = ms.XvToMv(hitpos, lr, cellno);
   Double_t   dlen = h(0,0);

   Double_t s = (hitpos.X() + fRwaist) * (hitpos.X() - fRwaist) +
                (hitpos.Y() + hitpos.Z() * fTanStereo) * (hitpos.Y() - hitpos.Z() * fTanStereo);
                
   Double_t endrad = ms.GetWireEnd().Perp();
   Double_t wireR  = xx.Perp();
   Double_t stereoAngle = ms.GetWireDir().Perp()/ms.GetWireDir().Z();
   stereoAngle = (ms.GetWireEnd(cellno).Cross(ms.GetWireDir(cellno)).Z() > 0. ? stereoAngle : -stereoAngle);
   Double_t cdclen   = wireEndp.Z();
   Double_t wirelen  = 2 * cdclen * TMath::Sqrt(1+stereoAngle*stereoAngle);
   TVector3 wireendm = ms.GetWireEnd(cellno);
   TVector3 wireendp = ms.GetWireEnd(cellno) + wirelen * ms.GetWireDir(cellno).Unit();

   rwaist            = TMath::Sqrt(TMath::Max(0.0,(endrad - cdclen * fTanStereo)*
                                           (endrad + cdclen * fTanStereo)));

   std::cerr << " S4CDCExactHit --------------------------- " << std::endl;
   std::cerr << "    This pointer  : "  << this << std::endl;
   std::cerr << "    DriftRegionNo : "  << fDriftRegionNumber << std::endl;
   std::cerr << "    PDGcode       : "  << pcode << std::endl;
   std::cerr << "    Momentum      : "  << momentum.Mag() << std::endl;
   std::cerr << "    TrackNo       : "  << trackNumber << std::endl;

   std::cerr << "    given : replaced " << std::endl;
   std::cerr << "    wire rad   : " << wireR << " : " << fWireR  << std::endl;
   std::cerr << "    TanStereo  : " << stereoAngle << " : " << fTanStereo  << std::endl;

   std::cerr << "    WireTop    : " << wireendp.X() << " " << wireendp.Y() << " " << wireendp.Z()
                                    << " : " << fWireEndAtPlusZ.X() << " " 
                                    << fWireEndAtPlusZ.Y() << " " << fWireEndAtPlusZ.Z() << std::endl;
   std::cerr << "    WireBtm    : " << wireendm.X() << " " << wireendm.Y() << " " << wireendm.Z()
                                    << " : " << fWireEndAtMinusZ.X() << " " 
                                    << fWireEndAtMinusZ.Y() << " " << fWireEndAtMinusZ.Z() << std::endl;
   std::cerr << "    RWaist     : " << rwaist << " : " << fRwaist  << std::endl;
   std::cerr << "    DriftLen   : " << dlen << " : " << fDriftLen  << std::endl;
   std::cerr << "    wireendrad : " << wireendm.Perp() << " : " << fWireEndAtMinusZ.Perp() << std::endl;
   std::cerr << "    exactpre   : " << pre.X() << " " << pre.Y() << " " << pre.Z()<< std::endl;
   std::cerr << "    pre.perp   : " << pre.Pt() << std::endl;
   std::cerr << "    S          : " << s << std::endl;
   // std::cerr << "    Hit Position is replaced by pre step point. " << std::endl;
#endif 
}


//=====================================================================
//* destructor --------------------------------------------------------

S4CDCExactHit::~S4CDCExactHit()
{
}

//=====================================================================
//* SetSortBy --------------------------------------------------------

void S4CDCExactHit::SetSortBy(Char_t *opt )
{
	switch (opt[2]) {
		case 'a':	// sort by "TrackNumber"
			fCDCExactHitSortBy = 0;
			break;
		case 'm': 	// sort by "Time"
			fCDCExactHitSortBy = 1;
			break;
		case 'e': 	// sort by "Energy"
			fCDCExactHitSortBy = 2;
			break;
		case 'i': 	// sort by "DriftLength"
			fCDCExactHitSortBy = 3;
			break;
		case 'l': 	// sort by "CellID"
			fCDCExactHitSortBy = 4;
			break;
		case 'r': 	// sort by "WireLayer"
			fCDCExactHitSortBy = 5;
			break;
		case 't': 	// sort by "HitNumber"
			fCDCExactHitSortBy = 6;
			break;
		default: 	// sort by "WireLayer"
			fCDCExactHitSortBy = 5;
	}		
}

//=====================================================================
//* IsSoatable --------------------------------------------------------

Bool_t S4CDCExactHit::IsSortable() const { return kTRUE; }
	
//=====================================================================
//* Compare -----------------------------------------------------------
	
Int_t S4CDCExactHit::Compare(const TObject *obj) const
{
  switch (fCDCExactHitSortBy) {
     case 0: { 
   	if (GetTrackID() < ((S4CDCExactHit *)obj)->GetTrackID()) return -1;
   	else if (GetTrackID() > ((S4CDCExactHit *)obj)->GetTrackID()) return 1;
  	else return 0;
  	break; }
     case 1: {
   	if (GetTOF() < ((S4CDCExactHit *)obj)->GetTOF()) return -1;
   	else if (GetTOF() > ((S4CDCExactHit *)obj)->GetTOF()) return 1;
  	else return 0;
  	break; }
     case 2: {
   	if (GetTotalEnergy() < ((S4CDCExactHit *)obj)->GetTotalEnergy()) return -1;
   	else if (GetTotalEnergy() > ((S4CDCExactHit *)obj)->GetTotalEnergy()) return 1;
  	else return 0; }
  	break;
     case 3: {
     	Double_t driftlen  = TMath::Abs(fDriftLen);
     	Double_t driftleno = TMath::Abs(((S4CDCExactHit *)obj)->fDriftLen);    	
   	if (driftlen < driftleno) return -1;
   	else if (driftlen > driftleno) return 1;
  	else return 0;
  	break; }
     case 4: {
     	Int_t cellno  = fCellID;
     	Int_t cellnoo = ((S4CDCExactHit *)obj)->fCellID;
   	if (cellno < cellnoo ) return -1;
   	else if (cellno > cellnoo) return 1;
  	else return 0;
  	break; }
     case 5: {
        Int_t drNumber    = fDriftRegionNumber;
        Int_t objdrNumber = ((S4CDCExactHit *)obj)->fDriftRegionNumber;
   	if (drNumber < objdrNumber) return -1;
   	else if (drNumber > objdrNumber) return 1;
  	else return 0;
  	break;}
     case 6: {
        Int_t hitNumber    = GetHitID();
        Int_t objhitNumber = ((S4CDCExactHit *)obj)->GetHitID();
   	if (hitNumber < objhitNumber) return -1;
   	else if (hitNumber > objhitNumber) return 1;
  	else return 0;
  	break;}
     default: {
        Int_t drNumber    = fDriftRegionNumber;
        Int_t objdrNumber = ((S4CDCExactHit *)obj)->fDriftRegionNumber;
   	if (drNumber < objdrNumber) return -1;
   	else if (drNumber > objdrNumber) return 1;
  	else return 0;
  	break;}
  }
}
