//*************************************************************************
//* ----------------
//* S4CDCMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4CDCMCTruthBuf		: MCTruth data buffer
//* (Update Record)
//*	2001/08/20  K.Hoshina	Original version.
//*     2004/08/10  K.Fujii     Modified ReadFile.
//*************************************************************************

#include <string>
#include <sstream>
#include <iomanip>
#include <iostream>
#include <fstream>
#include "S4CDCMCTruthBuf.h"
#include "S4CDCExactHit.h"
#include "JSFSteer.h"


#define __MMTOCM__   0.100
#define __MEVTOGEV__ 0.001

//#define __DEBUG__

//#define __DONTREMOVEDELTARAY__

#ifdef __KUBORIE__
const Int_t S4CDCMCTruthBuf::kNh = 5;
#endif

ClassImp(S4CDCMCTruthBuf)

//___________________________________________________________________________
// ---------------
// MCTruth Data Buffer
// ---------------
//
//
S4CDCMCTruthBuf::S4CDCMCTruthBuf(const char *name, const char *title, 
				 S4CDCMCTruth *module, Int_t bufsize) 
				 : S4VMCTruthBuf(name, title, module, bufsize)
				 , fEnergyDepositCut(0.0004)
                                 , fRangeCut(0.2)
                                 , fInputBufPtr(0)
#ifdef __KUBORIE__
                                 , hLdef(0), hNEdef(0), hNKdef(0), hXYdef(0)
#endif

{
#ifdef __KUBORIE__
	InitHists();
#endif
}

//___________________________________________________________________________
S4CDCMCTruthBuf::S4CDCMCTruthBuf(S4CDCMCTruth *module, const char *name, 
				 const char *title, Int_t bufsize) 
				 : S4VMCTruthBuf(name, title, module, bufsize)
                                 , fEnergyDepositCut(0.0004)
                                 , fRangeCut(0.2)
                                 , fInputBufPtr(0)
#ifdef __KUBORIE__
                                 , hLdef(0), hNEdef(0), hNKdef(0), hXYdef(0)
#endif
{
#ifdef __KUBORIE__
	InitHists();
#endif
}

//___________________________________________________________________________
S4CDCMCTruthBuf::~S4CDCMCTruthBuf()
{
   fTmpExactHits.Delete();
}

//___________________________________________________________________________
Bool_t S4CDCMCTruthBuf::ReadFile()
{
 // Set Ownership Flag -------//
 
  GetExactHitsPtr()->SetOwner();

 // Start reading data -------//

  char     dummy[1024];
  std::ifstream & in = *S4VMCTruth::GetInputStreamPtr();

 // Seek CDC header -------//

  Int_t eventno = Seek("*****_EXPName:CDC_output_start_*****");
  if (eventno < 0)  return kTRUE;  // -ve means faiure to find one



 // Found a CDC tag. CDC data should follow now. -------//
  
  Int_t    layerNumber, wireNumber, cellNumber, trackNumber, 
           mothertrackN, hitNumber, pcode, charge;
  Double_t cellDphi, cellSphi;
  Double_t prex, prey, prez, postx, posty, postz; 
  Double_t mx, my, mz, totE, edep, tof;
  Double_t wireEndmx, wireEndmy, wireEndmz, wireEndpx, wireEndpy, wireEndpz;
  Double_t tanStereo, rwaist;
  TVector3 pre, post, hit, momentum, wireEndm, wireEndp;

  in >> layerNumber;
  
  while ((in >> wireNumber >> cellNumber  >> trackNumber 
             >> mothertrackN >> hitNumber >> pcode
             >> charge >> cellDphi >> cellSphi 
             >> prex >> prey >> prez 
             >> postx >> posty >> postz 
             >> mx >> my >> mz 
             >> totE >> edep >> tof
             >> wireEndmx >> wireEndmy >> wireEndmz
             >> wireEndpx >> wireEndpy >> wireEndpz
             >> tanStereo >> rwaist) && in.good())
  {
    prex      *= __MMTOCM__;
    prey      *= __MMTOCM__;
    prez      *= __MMTOCM__;
    postx     *= __MMTOCM__;
    posty     *= __MMTOCM__;
    postz     *= __MMTOCM__;
    rwaist    *= __MMTOCM__;
    mx        *= __MEVTOGEV__;
    my        *= __MEVTOGEV__;
    mz        *= __MEVTOGEV__;
    totE      *= __MEVTOGEV__;
    edep      *= __MEVTOGEV__;
    wireEndmx *= __MMTOCM__;
    wireEndmy *= __MMTOCM__;
    wireEndmz *= __MMTOCM__;
    wireEndpx *= __MMTOCM__;
    wireEndpy *= __MMTOCM__;
    wireEndpz *= __MMTOCM__;
   
    pre.SetX(prex);
    pre.SetY(prey);
    pre.SetZ(prez);
    
    post.SetX(postx);
    post.SetY(posty);
    post.SetZ(postz);
    
    momentum.SetX(mx);
    momentum.SetY(my);
    momentum.SetZ(mz);

    wireEndm.SetX(wireEndmx);
    wireEndm.SetY(wireEndmy);
    wireEndm.SetZ(wireEndmz);
 
    wireEndp.SetX(wireEndpx);
    wireEndp.SetY(wireEndpy);
    wireEndp.SetZ(wireEndpz);

    S4CDCExactHit *datum 
	  = new S4CDCExactHit( eventno, layerNumber, wireNumber, cellNumber,
	  		       trackNumber, mothertrackN, hitNumber, pcode, 
			       charge, cellDphi, cellSphi, 
			       pre, post, momentum,
			       totE, edep, tof, wireEndm, wireEndp, tanStereo, rwaist);  
	  
    GetExactHitsPtr()->Add(datum);	

    S4CDCExactHit *tmpdatum = new S4CDCExactHit(*datum);
    fTmpExactHits.Add(tmpdatum);

#if 0	     	     	   
    std::cerr << layerNumber << " " << wireNumber << " " 
              << cellNumber  << " " << cellNumber << " " 
              << trackNumber << " " << mothertrackN << " "
              << hitNumber << " "  << pcode << " " << charge << " "
      	      << cellDphi << " " << cellSphi << " " 
      	      << prex << " " << prey << " " << prez << " " 
      	      << postx << " " << posty << " " << postz << " " 
      	      << mx << " " << my << " " << mz << " " << totE << " " 
      	      << edep << " " << tof << std::endl;
#endif      	

    in >> dummy;
    if (! strncmp(dummy,"*****_EXPName:CDC_output_end_*****" ,34) ) break;

    std::istringstream str(dummy);
    str    >> layerNumber;
  }
  return kTRUE;
}

//___________________________________________________________________________
void S4CDCMCTruthBuf::RemoveDeltaRayHits()
{
#ifdef __KUBORIE__
    TDirectory *last=gDirectory;
    gFile->cd("/");
#endif
    if (GetExactHitsPtr()) {
	S4CDCExactHit::SetSortBy("TrackNumber");
	TObjArray* exacthitsptr = GetExactHitsPtr();
	exacthitsptr->UnSort();
	exacthitsptr->Sort();
	
	Int_t    trackno     = 0;
	Int_t    lasttrackno = -9999;
	Int_t    count       = 0;
	TVector3 firstPos;
#ifndef __DONTREMOVEDELTARAY__
	Int_t    nremoved    = 0;
#endif
#ifdef __KUBORIE__
	Double_t dedx        = 0.;
	static const Double_t Mass[] = { 0.00051099896, // 11   (e-)
		                         0.9382720,     // 2212 (p)
					 0.13957018,    // 211  (pi+)
					 0.105658357,   // 13   (mu-)
					 0.493677	// 321  (K+)
	                               };

#endif
        Int_t nexacthits = exacthitsptr->GetEntries() ;
        Int_t i;
	for (i=0; i<nexacthits; i++) {
	   S4CDCExactHit* exacthitp
	  	   = (S4CDCExactHit *)exacthitsptr->UncheckedAt(i);
           trackno = exacthitp->GetTrackID();
	   if (trackno != lasttrackno && count) {
	      S4CDCExactHit* preexacthitp
			= (S4CDCExactHit *)exacthitsptr->UncheckedAt(i-1);
	      Int_t    pdgcode = preexacthitp->GetPDGCode();
              TVector3 lastPos = preexacthitp->GetPostPosition();
              Double_t       x = (lastPos - firstPos).Mag();
#ifndef __DONTREMOVEDELTARAY__
	      if (count == 1) {
	       	 Double_t energydep = preexacthitp->GetEnergyDeposit();
	         if (pdgcode == 11 && 
		       (energydep > fEnergyDepositCut || x < fRangeCut)) {
                    exacthitsptr->RemoveAt(i-1);
                    delete exacthitsptr->UncheckedAt(i-1);
                    nremoved++;
#ifdef __DEBUG__
                    std::cerr << " S4CDCMCTruthBuf::RemoveDeltaRayHits: "
	  	       	      << " DeltaRayHit is removed" << std::endl;
#endif
		 }
	      }
#endif
#ifdef __KUBORIE__
              dedx   /= count;
#if 0
	      // Angle correction is meaningless for delta rays?
              dedx   *= TMath::Sin((preexacthitp->GetMomentum()).Theta());
#endif
	      // Monitor single hit tracks
	      //
	      Int_t type;
	      switch (pdgcode) {
                 case    11: type =  0; break;
	 	 case  2212: 
		 case -2212: type =  1; break;
                 case   211:
                 case  -211: type =  2; break;
                 case    13:
                 case   -13: type =  3; break;
		 case   321: 
		 case  -321: type =  4; break;
                 default:    type = -1; break;
	      }
	      if (type >= 0) {
	         if (count == 1) {
		    hXY[type] -> Fill(firstPos.x(),firstPos.y(),1.0);
		 }
		 Double_t   wireR = preexacthitp->GetWireR();
		 Int_t    layerNo = preexacthitp->GetLayerNumber();
	         Double_t    totE = preexacthitp->GetTotalEnergy();
	         Double_t    kinE = totE - Mass[type];
		 hNHitE[type]->Fill(count,dedx,1.0);
		 hNHitK[type]->Fill(count,kinE,1.0);

		 if (type == 0) {
		     hLength[0]-> Fill(count,1.0);
		     hLength[1]-> Fill(x,1.0);
		 }
		 if (type == 1) {
		     hLength[2]-> Fill(layerNo,1.0);
		     hLength[3]-> Fill(layerNo,1./wireR);
		 }
	      }
	      dedx  = 0.;
#endif
	      count = 0;
	   }
#ifdef __DEBUG__
	   std::cerr << " S4CDCMCTruthBuf::RemoveDeltaRayHits: "
	             << " TrackNumber = " << lasttrackno 
	             << " Nhits = " << count << std::endl;
#endif
	   count++;
	   lasttrackno = trackno;
#ifdef __KUBORIE__
	   dedx += exacthitp->GetEnergyDeposit();
#endif
	   if (count == 1) {
              firstPos = exacthitp->GetPrePosition();
	   }
	}

        S4CDCExactHit* exacthitp
              = (S4CDCExactHit *)exacthitsptr->UncheckedAt(nexacthits-1);
        Int_t    pdgcode = exacthitp->GetPDGCode();
        TVector3 lastPos = exacthitp->GetPostPosition();
        Double_t       x = (lastPos - firstPos).Mag();
	
#ifndef __DONTREMOVEDELTARAY__
	if (count == 1) {
           Double_t energydep = exacthitp->GetEnergyDeposit();
           if (pdgcode == 11 && 
	         (energydep > fEnergyDepositCut || x < fRangeCut)) {
	      exacthitsptr->RemoveAt(nexacthits-1);
	      delete exacthitsptr->UncheckedAt(nexacthits-1);
              nremoved++;
#ifdef __DEBUG__
                    std::cerr << " S4CDCMCTruthBuf::RemoveDeltaRayHits: "
	  	       	      << " DeltaRayHit is removed" << std::endl;
#endif
           }
	}
#endif
#ifdef __KUBORIE__
#if 0
        // Angle correction is meaningless for delta rays?
        dedx   *= TMath::Sin((preexacthitp->GetMomentum()).Theta());
#endif

        // Monitor single hit tracks
        //
        Int_t type;
           switch (pdgcode) {
              case    11: type =  0; break;
              case  2212: 
	      case -2212: type =  1; break;
              case   211:
              case  -211: type =  2; break;
              case    13:
              case   -13: type =  3; break;
	      case   321: 
	      case  -321: type =  4; break;
              default:    type = -1; break;
	   }
        if (type >= 0) {
	   if (count == 1) {
	   hXY[type] -> Fill(firstPos.x(),firstPos.y(),1.0);
           }
//           dedx   /= count;
           Double_t   wireR = exacthitp->GetWireR();
	   Int_t    layerNo = exacthitp->GetLayerNumber();
	   Double_t    totE = exacthitp->GetTotalEnergy();
	   Double_t    kinE = totE - Mass[type];
	   hNHitE[type]->Fill(count,dedx,1.0);
	   hNHitK[type]->Fill(count,kinE,1.0);

	   if (type == 0) {
	      hLength[0]-> Fill(count,1.0);
	      hLength[1]-> Fill(x,1.0);
	   }
	   if (type == 1) {
	      hLength[2]-> Fill(layerNo,1.0);
	      hLength[3]-> Fill(layerNo,1./wireR);
	   }
	}
	dedx  = 0.;
#endif
        exacthitsptr->Compress();
#ifdef __DEBUG__
        std::cerr << " S4CDCMCTruthBuf::RemoveDeltaRayHits:"
		  << " number of removed hits = " << nremoved 
		  << " original number of hits = " << i << std::endl;
#endif
		
   }
#ifdef __KUBORIE__
   last->cd();
#endif
}

//___________________________________________________________________________
void S4CDCMCTruthBuf::JoinSeparatedHits()
{
    if (GetExactHitsPtr()) {
	S4CDCExactHit::SetSortBy("TrackNumber");
	TObjArray* exacthitsptr = GetExactHitsPtr();
	exacthitsptr->UnSort();
	exacthitsptr->Sort();
		
	TObjArray tracktmp(2000);	
	
	Int_t tracklast = -9999;
	TIter next(exacthitsptr);
	S4CDCExactHit *exacthitp;
	while ((exacthitp = (S4CDCExactHit *)next())) {
		Int_t trackcur = exacthitp->GetTrackID();
		if ( trackcur != tracklast ) {
			if ( tracktmp.GetEntries() > 0 ) {
				AppendTrack( &tracktmp );
				tracktmp.Clear();
			}
			tracklast = trackcur;
		} 
		tracktmp.Add ( exacthitp );
	}
		
	if ( tracktmp.GetEntries() > 0 ) {
		AppendTrack( &tracktmp );
		tracktmp.Clear();
	}
	
	Int_t nhits = GetExactHitsPtr()->GetEntries();
	for (Int_t i=0; i<nhits; i++) {
		S4CDCExactHit* hitp
			= (S4CDCExactHit *)GetExactHitsPtr()->UncheckedAt(i);			
#if 0
		std::cerr << " TrackNo = " << hitp->GetTrackID() 
		     << " CellID = " << hitp->GetCellID()
		     << " WireNo = " << hitp->GetWireNumber()
		     << " prex   = " << (hitp->GetPrePosition()).x()
		     << " postx  = " << (hitp->GetPostPosition()).x()
		     << " flag   = " << hitp->GetFlag() 
		     << std::endl;
#endif
		if (hitp->GetFlag() == kTRUE) {
			GetExactHitsPtr()->RemoveAt(i);
		} 
	}
		
	GetExactHitsPtr()->Compress();		
	
	std::cerr << " S4CDCMCTruthBuf::JoinSeparatedHits: NEntries = " 
	     << GetExactHitsPtr()->GetEntries() 
	     << std::endl;
    }
}

//=====================================================================
//* AppendTrack -------------------------------------------------------

void S4CDCMCTruthBuf::AppendTrack(TObjArray* tracktmpp)
{
	
	TObjArray    celltmp(1000);
	
	S4CDCExactHit::SetSortBy("CellID");
	tracktmpp->UnSort();
	tracktmpp->Sort();
	
	Int_t celllast = -9999;
		
	TIter next(tracktmpp);
	S4CDCExactHit *exacthitp;
	
	while ((exacthitp = (S4CDCExactHit *)next())) {
	
		Int_t cellcur = exacthitp->GetCellID();
		if ( cellcur != celllast ) {
			if ( celltmp.GetEntries() > 0 ) {
				AppendCell( &celltmp );
				celltmp.Clear();
			}
			celllast = cellcur;
		} 
		celltmp.Add ( exacthitp );
	}
		
	if ( celltmp.GetEntries() > 0 ) {
		AppendCell( &celltmp );
		celltmp.Clear();
	}
		
}

//=====================================================================
//* AppendCell -------------------------------------------------------

void S4CDCMCTruthBuf::AppendCell(TObjArray* celltmpp)
{
	
	TObjArray    wiretmp(5);
	
	S4CDCExactHit::SetSortBy("WireNumber");
	celltmpp->UnSort();
	celltmpp->Sort();
	
	Int_t wirelast = -9999;
		
	TIter next(celltmpp);
	S4CDCExactHit *exacthitp;
	
	while ((exacthitp = (S4CDCExactHit *)next())) {
	
		Int_t wirecur = exacthitp->GetWireNumber();
		if ( wirecur != wirelast ) {
			if ( wiretmp.GetEntries() > 0 ) {
				JoinHits( &wiretmp );
				wiretmp.Clear();
			}
			wirelast = wirecur;
		} 
		wiretmp.Add ( exacthitp );
	}
		
	if ( wiretmp.GetEntries() > 0 ) {
		JoinHits( &wiretmp );
		wiretmp.Clear();
	}
		
}

//=====================================================================
//* JoinHits ----------------------------------------------------------

void S4CDCMCTruthBuf::JoinHits( TObjArray* wiretmpp )
{
	S4CDCExactHit::SetSortBy("Time");
	wiretmpp->UnSort();
	wiretmpp->Sort();
		
	TIter next(wiretmpp);
	S4CDCExactHit* hitp = (S4CDCExactHit *)next();
	while (hitp) {
#if 0			
		std::cerr 	<< "* select a hit TrackNo = " 
				<< hitp->GetTrackID() 
				<< " HitNo= "  << hitp->GetHitID()
				<< " CellID= " << hitp->GetCellID()
				<< " wireNo= " << hitp->GetWireNumber()
				<< " HitNo= "  << hitp->GetHitID() 
				<< " TOF= "    << hitp->GetTOF() 
				<< " prex= " << (hitp->GetPrePosition()).x()
				<< " prey= " << (hitp->GetPrePosition()).y()
				<< " prez= " << (hitp->GetPrePosition()).z()
				<< " postx= " << (hitp->GetPostPosition()).x()
				<< " posty= " << (hitp->GetPostPosition()).y()
				<< " postz= " << (hitp->GetPostPosition()).z()
				<< std::endl;
#endif	
			
		S4CDCExactHit * nexthitp;			
		while((nexthitp = (S4CDCExactHit *)next())) {
			TVector3 post1st ( hitp->GetPostPosition()    );
			TVector3 pre2nd  ( nexthitp->GetPrePosition() );
         Int_t    postpdg = hitp->GetPDGCode();
         Int_t    prepdg  = nexthitp->GetPDGCode();
         
						
			if (post1st.x() != pre2nd.x() || 
			    post1st.y() != pre2nd.y() || 
			    post1st.z() != pre2nd.z() ||
             postpdg     != prepdg   )  break;
			      
			TVector3 pre1st  ( hitp    ->GetPrePosition() );
			TVector3 post2nd ( nexthitp->GetPostPosition());
			Double_t edep1st = hitp    ->GetEnergyDeposit();
			Double_t edep2nd = nexthitp->GetEnergyDeposit();
			Double_t x = (pre1st.x() + post2nd.x())/2. ;
			Double_t y = (pre1st.y() + post2nd.y())/2. ;
			Double_t z = (pre1st.z() + post2nd.z())/2. ;
			TVector3 hitposition(x, y, z);
#if 0				
			std::cerr << " prehit x= " << pre1st.x()
				  << " y= " << pre1st.y() 
				  << " z= " << pre1st.z() 
				  << " posthit x= " << post2nd.x()
				  << " y= " << post2nd.y() 
				  << " z= " << post2nd.z() 
				  << " newhit x= " << x 
				  <<  " y= " << y 
				  << " z= " << z << std::endl;
#endif				
			hitp->SetPostPosition( post2nd );
			hitp->SetPosition( hitposition );
			hitp->SetEnergyDeposit( edep1st + edep2nd );
			nexthitp->SetFlag( kTRUE );	
		}
		hitp = (S4CDCExactHit *)next();
	}
	
}

#ifdef __KUBORIE__
void S4CDCMCTruthBuf::InitHists()
{
   TDirectory *last=gDirectory;
   gFile->cd("/");

   hLdef   = new THdef1[kNh];
   hLength = new TH1D*[kNh];
   for (Int_t i = 0; i<kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;

      sttag  << "S4CDCMCTruthBuf.Length"
             << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef  << "Length" << i << " 100 0. 10." << std::ends;
      std::stringstream stin;
      stin   << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
             << std::ends;
      stin   >> hLdef[i].title
	     >> hLdef[i].nxbin >> hLdef[i].xmin >> hLdef[i].xmax;

      hLength[i] = new TH1D(hLdef[i].title,hLdef[i].title,
	                    hLdef[i].nxbin,hLdef[i].xmin,hLdef[i].xmax);
#if 1
      std::cerr << " "      << hLdef[i].title
                << " nx "   << hLdef[i].nxbin
                << " xmin " << hLdef[i].xmin
                << " xmax " << hLdef[i].xmax
	        << std::endl;
#endif
   }

   hNEdef = new THdef2[kNh];
   hNHitE = new TH2D*[kNh];
   for (Int_t i = 0; i<kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag  << "S4CDCMCTruthBuf.NHitE" 
             << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef  << "NHitE" << i << " 100 0. 100. 100 0. 0.0003" << std::ends;
      std::stringstream stin;
      stin   << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
             << std::ends;
      stin   >> hNEdef[i].title 
	     >> hNEdef[i].nxbin >> hNEdef[i].xmin >> hNEdef[i].xmax
             >> hNEdef[i].nybin >> hNEdef[i].ymin >> hNEdef[i].ymax;

      hNHitE[i] = new TH2D(hNEdef[i].title,hNEdef[i].title,
                      hNEdef[i].nxbin,hNEdef[i].xmin,hNEdef[i].xmax,
		      hNEdef[i].nybin,hNEdef[i].ymin,hNEdef[i].ymax);
#if 1
      std::cerr << " "      << hNEdef[i].title
	        << " nx "   << hNEdef[i].nxbin
	        << " xmin " << hNEdef[i].xmin
	        << " xmax " << hNEdef[i].xmax
	        << " ny "   << hNEdef[i].nybin
	        << " ymin " << hNEdef[i].ymin
	        << " ymax " << hNEdef[i].ymax
	        << std::endl;
#endif
   }

   hNKdef = new THdef2[kNh];
   hNHitK = new TH2D*[kNh];
   for (Int_t i = 0; i<kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag  << "S4CDCMCTruthBuf.NHitK" 
             << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef  << "NHitK" << i << " 100 0. 100. 100 0. 0.002" << std::ends;
      std::stringstream stin;
      stin   << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
             << std::ends;
      stin   >> hNKdef[i].title 
	     >> hNKdef[i].nxbin >> hNKdef[i].xmin >> hNKdef[i].xmax
             >> hNKdef[i].nybin >> hNKdef[i].ymin >> hNKdef[i].ymax;

      hNHitK[i] = new TH2D(hNKdef[i].title,hNKdef[i].title,
                      hNKdef[i].nxbin,hNKdef[i].xmin,hNKdef[i].xmax,
		      hNKdef[i].nybin,hNKdef[i].ymin,hNKdef[i].ymax);
#if 1
      std::cerr << " "      << hNKdef[i].title
	        << " nx "   << hNKdef[i].nxbin
	        << " xmin " << hNKdef[i].xmin
	        << " xmax " << hNKdef[i].xmax
	        << " ny "   << hNKdef[i].nybin
	        << " ymin " << hNKdef[i].ymin
	        << " ymax " << hNKdef[i].ymax
	        << std::endl;
#endif
   }

   hXYdef = new THdef2[kNh];
   hXY    = new TH2D*[kNh];
   for (Int_t i = 0; i<kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag  << "S4CDCMCTruthBuf.XY" 
             << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef  << "XY"<< i <<" 100 -160. 160. 100 -160. 160." << std::ends;
      std::stringstream stin;
      stin   << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
             << std::ends;
      stin   >> hXYdef[i].title 
	     >> hXYdef[i].nxbin >> hXYdef[i].xmin >> hXYdef[i].xmax
             >> hXYdef[i].nybin >> hXYdef[i].ymin >> hXYdef[i].ymax;

      hXY[i] = new TH2D(hXYdef[i].title,hXYdef[i].title,
                        hXYdef[i].nxbin,hXYdef[i].xmin,hXYdef[i].xmax,
		        hXYdef[i].nybin,hXYdef[i].ymin,hXYdef[i].ymax);
#if 1
      std::cerr << " "      << hXYdef[i].title
	        << " nx "   << hXYdef[i].nxbin
	        << " xmin " << hXYdef[i].xmin
	        << " xmax " << hXYdef[i].xmax
	        << " ny "   << hXYdef[i].nybin
	        << " ymin " << hXYdef[i].ymin
	        << " ymax " << hXYdef[i].ymax
	        << std::endl;
#endif
   }
   last->cd();
}



#endif
