#ifndef __S4GENMCTRUTHBUF__
#define __S4GENMCTRUTHBUF__
//*************************************************************************
//* ---------------------
//* S4GENMCTruthBuf Class
//* ---------------------
//* (Description)
//*	A class to store generator information.
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4GENMCTruthBuf		: steering class
//* (Update Record)
//*	2005/04/01  K.Fujii	Original version.
//*************************************************************************

#include "S4GENMCTruth.h"
#include "S4VMCTruthBuf.h"
#include "TLorentzVector.h"

// -----------------------
// MonteCarlo Truth Buffer
// -----------------------

class S4GENMCTruthBuf : public S4VMCTruthBuf {

public:
  S4GENMCTruthBuf(const Char_t *name  = "S4GENMCTruthBuf",
                  const Char_t *title = "Generator Information Buf", 
                  S4GENMCTruth *mod   = 0,
                  Int_t         size  = 1000);

  S4GENMCTruthBuf(S4GENMCTruth *mod,
                  const Char_t *name  = "S4GENMCTruthBuf",
		  const Char_t *title = "Generator Information Buf",
                  Int_t         size  = 1000);

  virtual   ~S4GENMCTruthBuf();

  TObjArray *GetInputBufPtr()                { return fInputBufPtr; }
  TObjArray *GetOutputBufPtr()               { return fInputBufPtr; }
  void       SetInputBufPtr(TObjArray *bufp) { fDone = kFALSE; 
                                               fInputBufPtr = bufp; }

  Bool_t     ReadFile();

  TLorentzVector   GetPVcm()  { if (!fDone) Update(); return fPVcm; }

private:
  void       Update();

private:
  TObjArray      *fInputBufPtr;             //! 
  TObjArray      *fOutputBufPtr;            //! 
  Bool_t          fDone;                    //!
  TLorentzVector  fPVcm;                    //! Momentum of CM system
  
  ClassDef(S4GENMCTruthBuf, 1) // IT raw data buffer
};

#endif

