//*************************************************************************
//* --------------------
//* class S4BreakPoint
//* --------------------
//* (Description)
//*     Class to book keep break ponit information.
//* (Update Record)
//*     2006/01/29  S.Yamamoto     Original version.
//*************************************************************************

#ifndef __HasJSFFromLCIO__
#include "J4BreakPoint.hh"
#endif
#include "S4BreakPoint.h"
#include "S4HISTMCTruthBuf.h"
#include <iostream>

std::map<Int_t, S4BreakPoint*> S4BreakPoint::fgBreakPointMap; 

ClassImp(S4BreakPoint)

//=====================================================================
//* Constructors ------------------------------------------------------
S4BreakPoint::S4BreakPoint()
             :fTrackID(0),
	      fParentID(0),
	      fCharge(0),
	      fParticleID(0),
	      fGlobalTime(0),
	      fKineticEnergy(0),
	      fPFOPtr(0)
{
}

#ifndef __HasJSFFromLCIO__
S4BreakPoint::S4BreakPoint(const J4BreakPoint *j4bpp)
             :fTrackID(j4bpp->GetTrackID()),
	      fParentID(j4bpp->GetParentID()),
	      fCharge(j4bpp->GetCharge()),
	      fParticleID(j4bpp->GetParticleID()),
	      fGlobalTime(j4bpp->GetGlobalTime()/ns),
	      fPosition(j4bpp->GetPosition().x()/cm,
                        j4bpp->GetPosition().y()/cm,
                        j4bpp->GetPosition().z()/cm),
	      fFourMomentum(j4bpp->GetFourMomentum().x()/GeV,
                            j4bpp->GetFourMomentum().y()/GeV,
                            j4bpp->GetFourMomentum().z()/GeV,
                            j4bpp->GetFourMomentum().t()/GeV),
	      fKineticEnergy(j4bpp->GetKineticEnergy()/GeV),
	      fPFOPtr(0)
{
  fgBreakPointMap.insert(std::pair<Int_t, S4BreakPoint*>(fTrackID, this));
}
#endif

//=====================================================================
//* Destructor -------------------------------------------------------
S4BreakPoint::~S4BreakPoint()
{
}

//=====================================================================
//* GetBreakPointPtr --------------------------------------------------
S4BreakPoint * S4BreakPoint::GetBreakPointPtr(Int_t trackID)
{
  std::map<Int_t, S4BreakPoint *>::iterator iter = fgBreakPointMap.find(trackID);
  if (iter != fgBreakPointMap.end()) {
    return iter->second;
  } else {
    return 0;
  }
}

//=====================================================================
//* Clear -------------------------------------------------------------
void S4BreakPoint::Clear()
{
  fgBreakPointMap.clear();
}

//=====================================================================
//* GetBreakPointMap --------------------------------------------------
std::map<Int_t, S4BreakPoint *> & S4BreakPoint::GetBreakPointMap()
{
  return fgBreakPointMap;
}
#if 1
//=====================================================================
//* LockAllDescendants ------------------------------------------------
void S4BreakPoint::LockAllDescendants()
{

  TIter nextchild(this);
  S4BreakPoint *cbpp;
  while ((cbpp = static_cast<S4BreakPoint *>(nextchild()))) {
    cbpp->Lock();
    TAttLockable *cpfop = dynamic_cast<TAttLockable *>(cbpp->GetPFOPtr());
    if (cpfop)  cpfop->Lock();
    cbpp->LockAllDescendants();
  }   
}
#endif
//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4BreakPoint::DebugPrint(const Char_t *opt)
{
  using namespace std;

  Int_t    mid  = GetParentID();
  Int_t    pid  = GetParticleID();
  Double_t momx = GetFourMomentum().X();
  Double_t momy = GetFourMomentum().Y();
  Double_t momz = GetFourMomentum().Z();
  Double_t ene  = GetFourMomentum().T();
  Double_t kene = GetKineticEnergy();
  Double_t mass = GetFourMomentum().M();
  Double_t posx = GetPosition().X();
  Double_t posy = GetPosition().Y();
  Double_t posz = GetPosition().Z();
  Double_t perp = GetPosition().Perp();
  TObject *pfop = GetPFOPtr();

  cerr << " Lock = "       << (IsLocked() ? "Y" : "N")
       << " TrackID = "    << GetTrackID()
       << " ParentID = "   << mid  
       << " PID = "        << pid 
       << " mass = "       << mass
       << " E = "          << ene 
       << " Pv = ("        << momx  
       << ", "             << momy 
       << ", "             << momz 
       << ")"
       << " Ekin = "       << kene  
       << " Xv = ("        << posx  
       << ", "             << posy  
       << ", "             << posz  
       << ")"
       << " pfop = "        << pfop
       << " R = "          << perp << endl;
  TString sopt(opt);
  if (sopt.Contains("r")) {
    TIter next(this);
    S4BreakPoint *bpp;
#if 0
    Int_t no=0;
#endif
    while ((bpp = static_cast<S4BreakPoint *>(next()))) {
#if 1
      if (bpp) bpp->DebugPrint(opt);
#else
      //      if (no==0||(pid==311&&pid==3122)) {
      if (bpp && bpp->GetKineticEnergy() > 1.e-3) bpp->DebugPrint(opt);
      no++;
      //}
#endif
    }
  }
}

//=====================================================================
//* PrintAll ----------------------------------------------------------
void S4BreakPoint::PrintAll(const Char_t *opt)
{
  using namespace std;

  cerr <<"S4BreakPoint::PrintAll() Called" << endl;
  std::map<Int_t, S4BreakPoint*>::iterator iter;
  for (iter  = fgBreakPointMap.begin(); 
       iter != fgBreakPointMap.end(); iter++) {
    iter->second->DebugPrint(opt);
  }
}
