//*************************************************************************
//* ----------------
//* S4ITMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4ITMCTruthBuf		: MCTruth data buffer
//* (Update Record)
//*	2001/08/20  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>
#include "S4ITMCTruthBuf.h"
#include "S4ITExactHit.h"
#include "S4VMCTruthBuf.h"
using namespace std;

ClassImp(S4ITMCTruthBuf)


//===========================================================
// Constructor -----------------------------------------------

S4ITMCTruthBuf::S4ITMCTruthBuf(const Char_t *name,
                               const Char_t *title, 
                               S4ITMCTruth  *module,
                               Int_t         size) 
              : S4VMCTruthBuf(name, title, module, size),
                fInputBufPtr(0)
{
}

S4ITMCTruthBuf::S4ITMCTruthBuf(S4ITMCTruth *module,
                               const Char_t *name, 
                               const Char_t *title,
                               Int_t         size) 
              : S4VMCTruthBuf(name, title, module, size),
                fInputBufPtr(0)
{
}

//===========================================================
// Destructor -----------------------------------------------

S4ITMCTruthBuf::~S4ITMCTruthBuf()
{
}

//===========================================================
// ReadFile -----------------------------------------------

Bool_t S4ITMCTruthBuf::ReadFile()
{
   // Set Ownership Flag -------//
   GetExactHitsPtr()->SetOwner();
 
   ifstream & in = *S4VMCTruth::GetInputStreamPtr();

   // Seek IT header -------//
   Int_t eventno = Seek("*****_EXPName:IT_output_start_*****");
   if (eventno < 0)  return kTRUE;  // -ve means faiure to find one

   // read data -------//
   Char_t   dummy[1024];
   Int_t    layerID, trackID, mtrackID, pcode, charge;
   Double_t prex, prey, prez, postx, posty, postz; 
   Double_t mx, my, mz, totE, edep, tof;
   TVector3 pre, post, hit, momentum;
   	
   // unit converter -------//
   static const double MMtoCM   = 1.e-1;
   static const double MeVtoGeV = 1.e-3;
   
   Int_t hitID = 0;    // very temporary!!!!
 
   
   while (!in.eof()) {
      in >> dummy;
      if (! strncmp(dummy,"*****_EXPName:IT_output_end_*****" ,34) ) break;
      in >> layerID >> trackID >> mtrackID >> pcode >> charge  
         >> prex  >> prey  >> prez >> postx >> posty >> postz 
         >> mx    >> my    >> mz >> totE  >> edep  >> tof;
   
      prex  *= MMtoCM;
      prey  *= MMtoCM;
      prez  *= MMtoCM;
      postx *= MMtoCM;
      posty *= MMtoCM;
      postz *= MMtoCM;
      mx    *= MeVtoGeV;
      my    *= MeVtoGeV;
      mz    *= MeVtoGeV;
      totE  *= MeVtoGeV;
      edep  *= MeVtoGeV;
  
      pre.SetX(prex);
      pre.SetY(prey);
      pre.SetZ(prez);
   
      post.SetX(postx);
      post.SetY(posty);
      post.SetZ(postz);
   
      momentum.SetX(mx);
      momentum.SetY(my);
      momentum.SetZ(mz);

      S4ITExactHit *datum 
            = new S4ITExactHit(eventno, layerID, trackID, mtrackID, hitID,
                               pcode, charge, pre, post, momentum,
                               totE, edep, tof);
 
      GetExactHitsPtr()->Add(datum);	
      hitID++;  // very temporary!!!!
   }
 
   return kTRUE;
}
 
 
//===========================================================
// MergeHits  -----------------------------------------------

void S4ITMCTruthBuf::MergeHits()
{
   S4ITExactHit::SetSortBy(S4VExactHit::kTrackID);
   TObjArray *hitbufp = GetExactHitsPtr();
   if (!hitbufp->GetEntries()) return;
   hitbufp->UnSort();
   hitbufp->Sort(); 
   TObjArray track(hitbufp->GetEntries());

   TIter next(hitbufp); 
   S4ITExactHit *ehitp = static_cast<S4ITExactHit *>(next());
   Int_t last = ehitp->GetTrackID(); 
   next.Reset();
   while ((ehitp = static_cast<S4ITExactHit *>(next()))) {
      if (ehitp->GetTrackID() == last) {
         track.Add(ehitp);
      } else {
         last = ehitp->GetTrackID();
         CleanUp(track);
         track.Clear();
      }         
   }
   if (track.GetEntries()) CleanUp(track);
   hitbufp->Compress();
}


void S4ITMCTruthBuf::CleanUp(TObjArray &tracks)
{
   S4ITExactHit::SetSortBy(S4VExactHit::kTof);
   tracks.UnSort(); 
   tracks.Sort(); 
   TIter nexthit(&tracks);
   Int_t nhits = tracks.GetEntries();
   for (Int_t i = 1; i < nhits; i++) { 
      S4ITExactHit *lastp = static_cast<S4ITExactHit *>(tracks[i-1]);
      S4ITExactHit *curp  = static_cast<S4ITExactHit *>(tracks[i]);
      if (lastp->GetPostPosition() == curp->GetPrePosition()) { 
         curp->SetPrePosition(lastp->GetPrePosition());
         curp->SetPosition(lastp->GetPosition());
         curp->SetPhi(lastp->GetPhi());
         curp->SetLayerR(lastp->GetLayerR());
         GetExactHitsPtr()->Remove(lastp);
         if (!fInputBufPtr) delete lastp;
      }
   }
}
