//*****************************************************************************
//* ----------------
//* S4VMCTruth Class
//* ----------------
//*
//* (Description)
//*    This set of classes reads Monte Carlo ExactHit data 
//*    and put them into root objects.
//*    When a root file is opened in the read/write mode, the root
//*    objects are stored in it.  If the root file is read-only,
//*    the data are obtained from it.
//*    The package contains the following classes:
//*    The following parameters can be specified in "jsf.conf". 
//*
//* (Requires)
//*	TObject:
//*	TClonesArray:
//*
//* (Provides)
//*	S4VMCTruth		: MCTruth 
//*
//* (Update Record)
//*	1999/06/02  K.Fujii	Original version.
//*     2004/08/10  K.Fujii     Now handles input streams with static methods
//*                             so as to allow data with multiple detector
//*                             components.
//*
//*****************************************************************************

#include <iostream>
#include <fstream>
#include "S4VMCTruth.h"

// include "J4VHit.hh"
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif

ClassImp(S4VMCTruth)

std::string    S4VMCTruth::fgInputFileName  = std::string("hit.dat");
std::ifstream *S4VMCTruth::fgInputStreamPtr = 0;
Int_t          S4VMCTruth::fgEventToProcess = 0;

//_____________________________________________________________________________
// -----------------
// MCTruth analysis
// -----------------
//
//
//=============================================================================
//* Ctors and dtor    ---------------------------------------------------------
//
S4VMCTruth::S4VMCTruth(const Char_t *name, const Char_t *title) 
			: JSFModule(name,title)
{
}

S4VMCTruth::~S4VMCTruth()
{
}

//
//=============================================================================
//* Parent's methods to override ----------------------------------------------
//
Bool_t S4VMCTruth::BeginRun(Int_t nrun)
{
#ifdef __DEBUG__
    std::cout << " BeginRun is called for run " << nrun << std::endl;
#endif
        
    // Open MCTruth Root File
   
    if ( ! fFile->IsWritable() ) {
        return kTRUE;
    }

    // Open MCTruth File
    
    if (!S4VMCTruth::GetInputStreamPtr()) S4VMCTruth::OpenInputStream();
      
    // output standard information
    
    SetModuleStatus(kBeginRun);
    SetRunNumber(nrun);
    ConfDirectory();
    
    return kTRUE;
}

//
//=============================================================================
//* Utility functions    ------------------------------------------------------
//
Bool_t S4VMCTruth::Process(Int_t ev)
{
   fgEventToProcess = ev;
   return kTRUE;
}

//
//=============================================================================
//* Utility functions    ------------------------------------------------------
//
void S4VMCTruth::DebugPrint(const Char_t *)
{
}

//
//=============================================================================
//* Input stream hanndlers  ---------------------------------------------------
//

void S4VMCTruth::SetInputFileName(const Char_t *name)
{
   fgInputFileName = name;
}

std::ifstream * S4VMCTruth::GetInputStreamPtr()
{ 
   return fgInputStreamPtr;
}

void S4VMCTruth::OpenInputStream()
{
   if (fgInputStreamPtr) {
      std::cerr << ">>>>> S4VMCTruth::OpenInputStream >>>>>" << std::endl
                << " Can't open more than one input file!"   << std::endl
                << " Abort!" << std::endl;
      abort();
   }
   fgInputStreamPtr = new std::ifstream();
   std::cout << " Going to open " << fgInputFileName << std::endl;
   fgInputStreamPtr->open(fgInputFileName.data(),std::ios::in);
   std::cout << " Opened " << fgInputFileName << std::endl;
}

void S4VMCTruth::CloseInputStream()
{
   if (fgInputStreamPtr) {
      fgInputStreamPtr->close();
      delete fgInputStreamPtr;
      fgInputStreamPtr = 0;
   }
}

