//*************************************************************************
//* ----------------
//* S4VTXMCTruth Class
//* ----------------
//* (Description)
//* (Requires)
//*	S4VMCTruthBuf:
//* (Provides)
//*	S4VTXMCTruthBuf		: MCTruth data buffer
//* (Update Record)
//*	2001/08/20  A.Yamaguchi	Original version.
//*************************************************************************

#include <sstream>
#include <iomanip>
#include <iostream>
#include <fstream>
#include "S4VTXMCTruthBuf.h"
#include "S4VTXExactHit.h"

#define __MMTOCM__   0.100
#define __MEVTOGEV__ 0.001

using namespace std;

ClassImp(S4VTXMCTruthBuf)


//===========================================================
// Constructor -----------------------------------------------

S4VTXMCTruthBuf::S4VTXMCTruthBuf(const Char_t *name,
                                 const Char_t *title, 
                                 S4VTXMCTruth *module,
                                 Int_t         size) 
               : S4VMCTruthBuf(name, title, module, size),
                 fInputBufPtr(0)
{
}

S4VTXMCTruthBuf::S4VTXMCTruthBuf(S4VTXMCTruth *module,
                                 const Char_t *name, 
                                 const Char_t *title,
                                 Int_t         size) 
               : S4VMCTruthBuf(name, title, module, size),
                 fInputBufPtr(0)
{
}

//===========================================================
// Destructor -----------------------------------------------

S4VTXMCTruthBuf::~S4VTXMCTruthBuf()
{
}

//===========================================================
// ReadFile -----------------------------------------------

Bool_t S4VTXMCTruthBuf::ReadFile()
{
  // Set Ownership Flag -------//
  GetExactHitsPtr()->SetOwner();
 
  ifstream & in = *S4VMCTruth::GetInputStreamPtr();

  // Seek VTX header -------//
  Int_t eventno = Seek("*****_EXPName:VTX_output_start_*****");
  if (eventno < 0)  return kTRUE;  // -ve means faiure to find one


  // read data -------//
  Char_t   dummy[1024];
  Int_t    layerID, trackID, mothertrkID, pcode, charge;
  Double_t xin, yin, zin, xout, yout, zout; 
  Double_t mx, my, mz, totE, edep, tof;
  TVector3 prepos, postpos, hit, momentum;
 
  
  Int_t hitID = 0;    // very temporary!!!!

  
  while (!in.eof()) {  // now Layer Hit only
    in >> dummy;
    cout <<dummy;
    if (! strncmp(dummy,"*****_EXPName:VTX_output_end_*****" ,34) ) break;
      in >> dummy >> layerID >> trackID 
         >> mothertrkID >> pcode >> charge  
	 >> xin >> yin >> zin
	 >> xout >> yout >> zout 
         >> mx >> my >> mz 
         >> totE >> edep >> tof;
    
      xin    *= __MMTOCM__;
      yin    *= __MMTOCM__;
      zin    *= __MMTOCM__;
      xout   *= __MMTOCM__;
      yout   *= __MMTOCM__;
      zout   *= __MMTOCM__;
      mx     *= __MEVTOGEV__;
      my     *= __MEVTOGEV__;
      mz     *= __MEVTOGEV__;
      totE   *= __MEVTOGEV__;
      edep   *= __MEVTOGEV__;
    
      prepos.SetX(xin);
      prepos.SetY(yin);
      prepos.SetZ(zin);

      postpos.SetX(xout);
      postpos.SetY(yout);
      postpos.SetZ(zout);
    
      momentum.SetX(mx);
      momentum.SetY(my);
      momentum.SetZ(mz);

 
      S4VTXExactHit *datum 
            = new S4VTXExactHit(eventno, layerID, trackID, mothertrkID,
				hitID, pcode, charge, prepos, postpos,
                                momentum, totE, edep, tof);
  
      GetExactHitsPtr()->Add(datum);	
      hitID++;  // very temporary!!!!
  }

  return kTRUE;
}


//===========================================================
// MergeHits  -----------------------------------------------

void S4VTXMCTruthBuf::MergeHits()
{
   S4VTXExactHit::SetSortBy(S4VExactHit::kTrackID);
   TObjArray *hitbufp = GetExactHitsPtr();
   if (!hitbufp->GetEntries()) return;
   hitbufp->UnSort();
   hitbufp->Sort(); 
   TObjArray track(hitbufp->GetEntries());

   TIter next(hitbufp); 
   S4VTXExactHit *ehitp = static_cast<S4VTXExactHit *>(next());
   Int_t last = ehitp->GetTrackID(); 
   next.Reset();
   while ((ehitp = static_cast<S4VTXExactHit *>(next()))) {
      if (ehitp->GetTrackID() == last) {
         track.Add(ehitp);
      } else {
         last = ehitp->GetTrackID();
         CleanUp(track);
         track.Clear();
      }         
   }
   if (track.GetEntries()) CleanUp(track);
   hitbufp->Compress();
}


void S4VTXMCTruthBuf::CleanUp(TObjArray &tracks)
{
   S4VTXExactHit::SetSortBy(S4VExactHit::kTof);
   tracks.UnSort(); 
   tracks.Sort(); 
   TIter nexthit(&tracks);
   Int_t nhits = tracks.GetEntries();
   for (Int_t i = 1; i < nhits; i++) { 
      S4VTXExactHit *lastp = static_cast<S4VTXExactHit *>(tracks[i-1]);
      S4VTXExactHit *curp  = static_cast<S4VTXExactHit *>(tracks[i]);
      if (lastp->GetPostPosition() == curp->GetPrePosition()) { 
         curp->SetPrePosition(lastp->GetPrePosition());
         curp->SetPosition(lastp->GetPosition());
         curp->SetPhi(lastp->GetPhi());
         curp->SetZ(lastp->GetZ());
         curp->SetLayerR(lastp->GetLayerR());
         Double_t e=curp->GetEnergyDeposit() + lastp->GetEnergyDeposit();
         curp->SetEnergyDeposit(e);
         GetExactHitsPtr()->Remove(lastp);
         if (!fInputBufPtr) delete lastp;
      }
   }
}
