//*************************************************************************
//* -----------
//* S4BabyAnalysis
//* -----------
//* (Description)
//* 	Clasess to analyze Baby data.
//* (Update Record)
//*	2000/05/16  K.Fujii	Original version.
//*************************************************************************

#include "JSFSteer.h"
#include "S4BabyAnalysis.h"
#include <iostream>
#include <sstream>

#include "S4BabyHit.h"
#include "S4BabyHitWire.h"
#include "S4BabyHitMakerBuf.h"
#include "S4BabyAnalysis.h"

#include "S4BabyTrackHit.h"
#include "S4BabyTrackMakerBuf.h"

const Int_t S4BabyAnalysis::kZoneX   = 5;
const Int_t S4BabyAnalysis::kZoneY   = 3;
const Int_t S4BabyAnalysis::kNh = kZoneX*kZoneY;


ClassImp(TObjNum)

ClassImp(S4BabyAnalysis)


//___________________________________________________________________________
S4BabyAnalysis::S4BabyAnalysis(const Char_t *name, const Char_t *title)
	: JSFModule(name, title), fMode(0), fChi2Cut(36.), fNsigCut(5.),
          fSaturation(kTRUE)
{
   std::cerr << " constructor for S4BabyAnalysis is called..." << std::endl;
	
   hdefDXvsRes    = new THdef2[kNh];
   hdefDXvsSgt2   = new THdef2[kNh];
   hdefDXvsEff    = new THdef1[kNh];
   hdefDMultiScat = new THdef1[3];
   hdefXdiff      = new THdef2[3];
   hdefEff2tk     = new THdef1[5];
      
   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4BabyAnalysis.hDXvsRes" 
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hDXvsRes" << std::setw(2) << std::setfill('0') << i
      	    << " 50 0. 5. 200 -0.1 0.1" << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefDXvsRes[i].title
	    >> hdefDXvsRes[i].nxbin >> hdefDXvsRes[i].xmin >> hdefDXvsRes[i].xmax
	    >> hdefDXvsRes[i].nybin >> hdefDXvsRes[i].ymin >> hdefDXvsRes[i].ymax; 
   }

   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4BabyAnalysis.hDXvsSgt2" 
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hDXvsSgt2" << std::setw(2) << std::setfill('0') << i 
      	    << " 50 0. 5. 200 0.0 0.0004" << std::ends;      	  
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefDXvsSgt2[i].title
	    >> hdefDXvsSgt2[i].nxbin >> hdefDXvsSgt2[i].xmin >> hdefDXvsSgt2[i].xmax
	    >> hdefDXvsSgt2[i].nybin >> hdefDXvsSgt2[i].ymin >> hdefDXvsSgt2[i].ymax;
   }

   for (Int_t i = 0; i < kNh; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4BabyAnalysis.hDXvsEff" 
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hDXvsEff" << std::setw(2) << std::setfill('0') << i 
      	    << " 50 0. 5." << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefDXvsEff[i].title
	    >> hdefDXvsEff[i].nxbin >> hdefDXvsEff[i].xmin >> hdefDXvsEff[i].xmax;
   }

   for(Int_t i = 0; i < 3; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4BabyAnalysis.hDMultiScat"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hDMultiScat" << std::setw(2) << std::setfill('0') << i
#if 0
	    << " 10 0. 10. " << std::ends;
#else
            << " 100 -0.01 0.01 " << std::ends;
#endif
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefDMultiScat[i].title
	    >> hdefDMultiScat[i].nxbin >> hdefDMultiScat[i].xmin >> hdefDMultiScat[i].xmax;
   }
   
   for(Int_t i = 0; i < 3; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4BabyAnalysis.hXdiff"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hXdiff" << std::setw(2) << std::setfill('0') << i
	        << " 100 0. 5. 300 -0.015 0.015" << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefXdiff[i].title
	    >> hdefXdiff[i].nxbin >> hdefXdiff[i].xmin >> hdefXdiff[i].xmax
	    >> hdefXdiff[i].nybin >> hdefXdiff[i].ymin >> hdefXdiff[i].ymax;
   }
   
   for (Int_t i = 0; i < 5; i++) {
      std::stringstream sttag;
      std::stringstream stdef;
      sttag << "S4BabyAnalysis.hEff2tk"
	    << std::setw(2) << std::setfill('0') << i << std::ends;
      stdef << "hEff2tk" << std::setw(2) << std::setfill('0') << i
      	    << " 100 0. 0.5" << std::ends;
      std::stringstream stin;
      stin  << gJSF->Env()->GetValue(sttag.str().data(),stdef.str().data())
            << std::ends;
      stin  >> hdefEff2tk[i].title
	    >> hdefEff2tk[i].nxbin >> hdefEff2tk[i].xmin >> hdefEff2tk[i].xmax;
   }

   hDXvsRes    = new TH2D* [kNh];
   hDXvsSgt2   = new TH2D* [kNh];
   hDXvsEff    = new TH1E* [kNh];
   hDMultiScat = new TH1D* [3];
   hXdiff      = new TH2D* [3];
   hEff2tk     = new TH1E* [5];
}

//___________________________________________________________________________
S4BabyAnalysis::~S4BabyAnalysis()
{
#if 0
   delete [] hdefDXvsRes;
   delete [] hdefDXvsSgt2;
   delete [] hdefDXvsEff;
   for (Int_t i = 0; i < kNh; i++) {
  	delete hDXvsRes[i];
  	delete hDXvsSgt2[i];
   	delete hDXvsEff[i];
   }
   delete [] hDXvsRes;
   delete [] hDXvsSgt2;
   delete [] hDXvsEff;

   for (Int_t i = 0; i < 3; i++) delete hDMultiScat[i];
   delete [] hDMultiScat;
#endif
}

//___________________________________________________________________________
Bool_t S4BabyAnalysis::Initialize()
{
   std::cerr << " Initialize for S4BabyAnalysis is called..." << std::endl;

   TDirectory *last = gDirectory;
   gFile->cd("/");

   for (Int_t i = 0; i < kNh; i++) {
      hDXvsRes[i] = new TH2D(hdefDXvsRes[i].title,hdefDXvsRes[i].title,
      			 hdefDXvsRes[i].nxbin,hdefDXvsRes[i].xmin,hdefDXvsRes[i].xmax,
      			 hdefDXvsRes[i].nybin,hdefDXvsRes[i].ymin,hdefDXvsRes[i].ymax);
      std::cerr << i << " " << hdefDXvsRes[i].title 
      		<< " nx "   << hdefDXvsRes[i].nxbin
      		<< " xmin " << hdefDXvsRes[i].xmin
      		<< " xmax " << hdefDXvsRes[i].xmax
      		<< " ny "   << hdefDXvsRes[i].nybin
      		<< " ymin " << hdefDXvsRes[i].ymin
      		<< " ymax " << hdefDXvsRes[i].ymax
      		<< std::endl;
   }

   for (Int_t i = 0; i < kNh; i++) {
      hDXvsSgt2[i] = new TH2D(hdefDXvsSgt2[i].title,hdefDXvsSgt2[i].title,
      			 hdefDXvsSgt2[i].nxbin,hdefDXvsSgt2[i].xmin,hdefDXvsSgt2[i].xmax,
      			 hdefDXvsSgt2[i].nybin,hdefDXvsSgt2[i].ymin,hdefDXvsSgt2[i].ymax);
      std::cerr << i << " " << hdefDXvsSgt2[i].title 
      		<< " nx "   << hdefDXvsSgt2[i].nxbin
      		<< " xmin " << hdefDXvsSgt2[i].xmin
      		<< " xmax " << hdefDXvsSgt2[i].xmax
      		<< " ny "   << hdefDXvsSgt2[i].nybin
      		<< " ymin " << hdefDXvsSgt2[i].ymin
      		<< " ymax " << hdefDXvsSgt2[i].ymax
      		<< std::endl;
   }

   for (Int_t i = 0; i < kNh; i++) {
      hDXvsEff[i] = new TH1E(hdefDXvsEff[i].title,hdefDXvsEff[i].title,
      			 hdefDXvsEff[i].nxbin,hdefDXvsEff[i].xmin,hdefDXvsEff[i].xmax);
      std::cerr << i << " " << hdefDXvsEff[i].title 
      		<< " nx "   << hdefDXvsEff[i].nxbin
      		<< " xmin " << hdefDXvsEff[i].xmin
      		<< " xmax " << hdefDXvsEff[i].xmax
      		<< std::endl;
   }

   for (Int_t i = 0; i < 3; i++) {
      hDMultiScat[i] = new TH1D(hdefDMultiScat[i].title,hdefDMultiScat[i].title,
		            hdefDMultiScat[i].nxbin,hdefDMultiScat[i].xmin,hdefDMultiScat[i].xmax);
      std::cerr << i << " " << hdefDMultiScat[i].title
	        << " nx "   << hdefDMultiScat[i].nxbin
		<< " xmin " << hdefDMultiScat[i].xmin
		<< " xmax " << hdefDMultiScat[i].xmax
		<< std::endl;
   }

   for (Int_t i = 0; i < 3; i++) {
      hXdiff[i] = new TH2D(hdefXdiff[i].title,hdefXdiff[i].title,
      			 hdefXdiff[i].nxbin,hdefXdiff[i].xmin,hdefXdiff[i].xmax,
      			 hdefXdiff[i].nybin,hdefXdiff[i].ymin,hdefXdiff[i].ymax);
      std::cerr << i << " " << hdefXdiff[i].title 
      		<< " nx "   << hdefXdiff[i].nxbin
      		<< " xmin " << hdefXdiff[i].xmin
      		<< " xmax " << hdefXdiff[i].xmax
      		<< " ny "   << hdefXdiff[i].nybin
      		<< " ymin " << hdefXdiff[i].ymin
      		<< " ymax " << hdefXdiff[i].ymax
      		<< std::endl;
   }

   for (Int_t i = 0; i < 5; i++) {
      hEff2tk[i] = new TH1E(hdefEff2tk[i].title,hdefEff2tk[i].title,
      			 hdefEff2tk[i].nxbin,hdefEff2tk[i].xmin,hdefEff2tk[i].xmax);
      std::cerr << i << " " << hdefEff2tk[i].title 
      		<< " nx "   << hdefEff2tk[i].nxbin
      		<< " xmin " << hdefEff2tk[i].xmin
      		<< " xmax " << hdefEff2tk[i].xmax
      		<< std::endl;
   }

   hcDXvsRes = new TCanvas("hcDXvsRes","Canvas hcDXvsRes",0, 0, 200*kZoneX, 200*kZoneY);
   hcDXvsRes->Divide(kZoneX,kZoneY);
   hcDXvsSgt2 = new TCanvas("hcDXvsSgt2","Canvas hcDXvsSgt2",0, 0, 200*kZoneX, 200*kZoneY);
   hcDXvsSgt2->Divide(kZoneX,kZoneY);
   hcDXvsEff = new TCanvas("hcDXvsEff","Canvas hcDXvsEff",0, 0, 200*kZoneX, 200*kZoneY);
   hcDXvsEff->Divide(kZoneX,kZoneY);
   hcDMultiScat = new TCanvas("hcDMultiScat","Canvas hcMultiScat",0, 0, 300*3, 300*1);
   last->cd();
   return kTRUE;
}


//___________________________________________________________________________
Bool_t S4BabyAnalysis::Process(Int_t ev)
{
	MonitorHits(ev);
	MonitorTracks(ev);
std::cerr << "Analysis::Process evt" << ev << "is finished" << std::endl;	
	return kTRUE;	
}	

//___________________________________________________________________________
void S4BabyAnalysis::MonitorHits(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   S4BabyHitMaker *hm = (S4BabyHitMaker*)gJSF->FindModule("S4BabyHitMaker");
   S4BabyHitMakerBuf *hb = (S4BabyHitMakerBuf *)hm->EventBuf();
   TObjArray *outputbufp = hb->GetOutputBufPtr();

   Int_t wireno, cellno0 = -1;
   Double_t xpos0 = 0., ypos0 = 0., zpos0 = 0.;
   Double_t xpos4 = 0., ypos4 = 0., zpos4 = 0.;
   Int_t ecount = 0;

   TIter nextlayer(outputbufp);
   U4CDCHitLayer *layerp;
   while ((layerp = (U4CDCHitLayer *)nextlayer())) {
      TIter nextcell(layerp);
      U4CDCHitCell *cellp;
      while ((cellp = (U4CDCHitCell *)nextcell())) {
	 Int_t i = 0;
         TIter nextwire(cellp);
         S4BabyHitWire *wirep;
         while ((wirep = (S4BabyHitWire *)nextwire())) {         
	    wireno = wirep->GetWireNumber();
	    TIter nexthit(wirep);
	    S4BabyHit *hitp;
	    while ((hitp = (S4BabyHit *)nexthit())) {     		
	       Int_t cellno = hitp->GetExactHitPtr()->GetCellNumber();
	       Int_t pdgcode = hitp->GetExactHitPtr()->GetPDGCode();

	       if(pdgcode == 11 && cellno == 0 && wireno == 0) ecount++;
	       if(wireno == 0) {
	          xpos0   = (hitp->GetExactHitPtr()->GetPosition()).x();
	          ypos0   = (hitp->GetExactHitPtr()->GetPosition()).y();
	          zpos0   = (hitp->GetExactHitPtr()->GetPosition()).z();
		  cellno0 = hitp->GetExactHitPtr()->GetCellNumber();
		  i++;
	       }
	       if(cellno == cellno0 && wireno == 4) {
	          xpos4 = (hitp->GetExactHitPtr()->GetPosition()).x();
	          ypos4 = (hitp->GetExactHitPtr()->GetPosition()).y();
	          zpos4 = (hitp->GetExactHitPtr()->GetPosition()).z();
		  i++;
	       }
	       if(hitp->TestBit(S4BabyHitWire::kEModifiedByDeltaRay)){
	          Double_t exactX = hitp->GetExactHitPtr()->GetDriftLen();
	          Double_t X      = hitp->Xdrift();
		  Double_t absexactX = TMath::Abs(exactX);
		  Double_t absX = TMath::Abs(X);

	          //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	          if (cellno == 0) {
	             hXdiff[0]->Fill(absexactX,(absX-absexactX),1);
	          } else if(cellno == 1) {
	             hXdiff[1]->Fill(absexactX,(absX-absexactX),1);
	          } else {
	             hXdiff[2]->Fill(absexactX,(absX-absexactX),1);
	          }
	          //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	       }
	    }
	 }
	 
	 //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	 if (i==2){
#if 0
	 std::cerr << "S4BabyAnalysis::MonitorHits: DX = " << TMath::Abs(xpos0-xpos4) << std::endl;
	 std::cerr << "S4BabyAnalysis::MonitorHits: DY = " << TMath::Abs(ypos0-ypos4) << std::endl;
	 std::cerr << "S4BabyAnalysis::MonitorHits: DZ = " << TMath::Abs(zpos0-zpos4) << std::endl;
#endif
	    hDMultiScat[0]->Fill((xpos0-xpos4), 1.0);
	    hDMultiScat[1]->Fill((ypos0-ypos4), 1.0);
	    hDMultiScat[2]->Fill((zpos0-zpos4), 1.0);
	 }
	 //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      }
   }
   
   //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
//   hDMultiScat[1]->Fill(ecount, 1.0);
   //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
   
   last->cd();

}

//___________________________________________________________________________
void S4BabyAnalysis::MonitorTracks(Int_t ev)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");

   S4BabyTrackMaker *tm = (S4BabyTrackMaker*)gJSF->FindModule("S4BabyTrackMaker");
   S4BabyTrackMakerBuf *tb = (S4BabyTrackMakerBuf *)tm->EventBuf();
   TObjArray *outputbufp = tb->GetOutputBufPtr();


   Double_t firstx[5], secondx[5],  w[5];

   for (Int_t i = 0; i < 5; i++) {
      firstx [i] = 0.;
      secondx[i] = 0.;
      w [i]      = 0.;
   }
   TIter nexttrack(outputbufp);
   S4BabyTrack *trackp;
   
   while ((trackp = (S4BabyTrack *)nexttrack())) {
      Double_t trackno = trackp->GetTrackID();
         
      TIter nextvectorhit(trackp);
      S4BabyVectorHit *vectorhitp;
      while ((vectorhitp = (S4BabyVectorHit *)nextvectorhit())) {
      	 Int_t  cellno = vectorhitp->GetCellNumber();
         TIter  nexthit(vectorhitp);
         for (Int_t tolook = 0; tolook < 5; tolook++) {
            nexthit.Reset();
	    Int_t    nref         = 0;
	    Int_t    serialwireno = 5*cellno + tolook;
	    Double_t wat          = 0.;
	    S4BabyTrackHit *actualtrackhitp = 0;
	    S4BabyTrackHit *trackhitp;
            while ((trackhitp = (S4BabyTrackHit *)nexthit())) {
               S4BabyHit *hitp   = (S4BabyHit *)trackhitp->GetHitPtr();
#if 0
	       S4BabyExactHit *exacthitp = hitp->GetExactHitPtr();
#endif
	       Int_t      wireno = hitp->GetWireNumber();
	       if (tolook == wireno) actualtrackhitp = trackhitp;
	       else                  nref++;
            }

	    Double_t xoff = (cellno == 0 ? 0.0 : (cellno == 1 ? -5.0 : 5.0));
	    Double_t yoff = (cellno == 0 ? 7.0 : -3.0);
	    Double_t y    = yoff - tolook;
	    Double_t x    = vectorhitp->Xat(y) - xoff;

	    if (trackno == 1 && cellno == 0) firstx [tolook] = x;
	    if (trackno == 2 && cellno == 0) secondx[tolook] = x;
	    if (trackno == 2 && cellno == 0 && 
	        actualtrackhitp && !actualtrackhitp->GetHitPtr()->TestBit(S4BabyHitWire::kECannotSeparate)) {
	        w[tolook]       = 1.;
#if 0
	   	std::cerr << " trackno  = " << trackno << std::endl;
            	std::cerr << " w[" << tolook << "] = " << w[tolook] << std::endl;
#endif
            }

	    if (nref < 4) continue;
	    if (actualtrackhitp && fMode == 0) {
	       actualtrackhitp->Lock();
	       vectorhitp->Update();   
	       if (vectorhitp->GetChi2() > fChi2Cut) {
                  actualtrackhitp->Unlock();
		  continue;
	       }
            }
	    if (actualtrackhitp) {
               S4BabyHit *hitp  = (S4BabyHit *)actualtrackhitp->GetHitPtr();
#if 0
               Double_t y    = hitp->Ywire();
#endif
#if 0
	       Double_t x    = vectorhitp.Xat(y) - hitp->Xwire();
#else
	       Double_t x    = hitp->Xdrift();
#endif
	       Double_t dx   = actualtrackhitp->GetResidual();
	       Double_t ddx  = hitp->DXdrift();
	       Double_t sgx2 = vectorhitp->SGX2at(hitp->Ywire());

               //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	       hDXvsRes[serialwireno]->Fill(x,dx,1.0);
	       hDXvsSgt2[serialwireno]->Fill(x,sgx2,1.0);
               //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	       
               if (TMath::Abs(dx) < fNsigCut*TMath::Sqrt(sgx2+ddx*ddx)) wat = 1.;
	    }

   //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	    hDXvsEff[serialwireno]->Fill(TMath::Abs(x), wat);
   //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	    
	    if (actualtrackhitp && fMode == 0) actualtrackhitp->Unlock();
  	 }
      }
   }

   for(Int_t tolook = 0; tolook < 5; tolook++) {
      Double_t sep = secondx[tolook]-firstx[tolook];
#if 0
      std::cerr << " sep = " << sep << std::endl;
      std::cerr << " w[" << tolook << "] = " << w[tolook] << std::endl;
#endif
      //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      if(sep > 0) hEff2tk[tolook]->Fill(sep,w[tolook]);
      //>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
   }

   last->cd();
}

//___________________________________________________________________________
void S4BabyAnalysis::Draw(Option_t *opt)
{
   TDirectory *last = gDirectory;
   gFile->cd("/");
   Int_t i;
   for (i = 0; i < S4BabyAnalysis::kNh; i++) {
      hcDXvsRes->cd(i+1); hDXvsRes[i]->Draw(); hcDXvsRes->Update();
      hcDXvsSgt2->cd(i+1); hDXvsSgt2[i]->Draw(); hcDXvsSgt2->Update();
      hcDXvsEff->cd(i+1); hDXvsEff[i]->Draw(); hcDXvsEff->Update();
   }
   last->cd();
}
