//*************************************************************************
//* --------------------
//* S4BabyHitMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad hits.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2002/08/27  R.Kuboshima
//*************************************************************************

#include <iostream>
#include "S4BabyHitMakerBuf.h"
#include "S4BabyExactHit.h"
#include "S4BabyHit.h"

#define __TMP__
//#define __DEBUG__

ClassImp(S4BabyHitMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4BabyHitMakerBuf::S4BabyHitMakerBuf(U4CDCTimeToXRelation *calib, const char *name,
	                           const char *title, S4BabyHitMaker *module,
	                           Int_t bufsize)
		 :U4CDCHitMakerBuf(calib, name, title, module, bufsize)
{
}	       
	       
S4BabyHitMakerBuf::S4BabyHitMakerBuf(S4BabyHitMaker *module,
			           U4CDCTimeToXRelation *calib, 
			           const char *name,
	                           const char *title, 
	                           Int_t bufsize)
		 :U4CDCHitMakerBuf(module, calib, name, title, bufsize)
{
}	       

//=====================================================================
//* destructor --------------------------------------------------------

S4BabyHitMakerBuf::~S4BabyHitMakerBuf()
{
}


//=====================================================================
//* MakeHits ----------------------------------------------------------

void S4BabyHitMakerBuf::MakeHits()
{
	if (!IsInitialized()) {
		std::cerr << "ERROR  S4BabyHitMakerBuf::MakeHits" << std::endl
		     << "...... cluster data not yet initialized." << std::endl;
		return;
	} else if (fDone) {
		return;
	}
	
	fDone = kTRUE;
	
	TObjArray* inputbufp 
		= ((S4BabyMCTruthBuf*)fInputDataBufPtr)->GetExactHitsPtr();
		
	S4BabyExactHit::SetSortBy("CellNumber");
	inputbufp->UnSort();
	inputbufp->Sort();
	
	TObjArray layertmp(2000);
		
	Int_t layerlast = -9999;
		
	TIter next(inputbufp);
	S4BabyExactHit *exacthitp;
	while ((exacthitp = (S4BabyExactHit *)next())) {
		Int_t layercur = exacthitp->GetLayerNumber();
		if ( layercur != layerlast ) {			
#ifdef __DEBUG__
			std::cerr << " S4BabyHitMakerBuf::MakeHits: LayerNumber = "
			     << layercur << std::endl;
#endif
			if ( layertmp.GetEntries() > 0 ) {
#ifdef __DEBUG__
			     std::cerr << " S4BabyHitMakerBuf::MakeHits: LayerNumber"
			          << layerlast << " hits end"   << std::endl;
#endif
				AppendLayer( &layertmp, &fOutputBuf );
				layertmp.Clear();
			}
			layerlast = layercur;
		} 
		layertmp.Add ( exacthitp );
	}
	
	if ( layertmp.GetEntries() > 0 ) {
		AppendLayer( &layertmp, &fOutputBuf);
		layertmp.Clear();
	}
	
}

//=====================================================================
//* AppendLayer -------------------------------------------------------

void S4BabyHitMakerBuf::AppendLayer(TObjArray* layertmpp, TObjArray* outputbufp)
{
	
	U4CDCHitLayer  *layerp = new U4CDCHitLayer();
	TObjArray       celltmp(1000);
	
//	S4BabyExactHit::SetSortBy("CellNumber");
//	layertmpp->Sort();
	
	Int_t celllast = -99999;
		
	TIter next(layertmpp);
	S4BabyExactHit *exacthitp;
	while ((exacthitp = (S4BabyExactHit *)next())) {
		Int_t cellcur = exacthitp->GetCellNumber();
		if ( cellcur != celllast ) {
#ifdef __DEBUG__
			std::cerr << " S4BabyHitMakerBuf::AppendLayer: CellNumber = "
			     << cellcur << std::endl;
#endif
			if ( celltmp.GetEntries() > 0 ) {
#ifdef __DEBUG__
			        std::cerr << " S4BabyHitMakerBuf::AppendLayer: CellNumber"
			             << celllast << " hits end"<< std::endl;
#endif
				AppendCell( &celltmp, layerp );
				celltmp.Clear();
			}
			celllast = cellcur;
		} 
		celltmp.Add ( exacthitp );
	}
	
	if ( celltmp.GetEntries() > 0 ) {
		AppendCell( &celltmp, layerp );
		celltmp.Clear();
	}
	
	if ( layerp->GetEntries() > 0 ) {
		outputbufp->Add(layerp);
	} else {
		delete layerp;
	}	
	
}

//=====================================================================
//* AppendCell --------------------------------------------------------

void S4BabyHitMakerBuf::AppendCell(TObjArray* celltmpp, U4CDCHitLayer* layercurp)
{

	U4CDCHitCell  *cellp  = 0;
	S4BabyHitWire  *wirep  = 0;
	
	S4BabyExactHit::SetSortBy("WireNumber");
	celltmpp->Sort();
	
	TIter next(celltmpp);
	S4BabyExactHit *exacthitp;
	while ((exacthitp = (S4BabyExactHit *)next())) {
		if (!cellp) {
			cellp  = new U4CDCHitCell();
			wirep  = new S4BabyHitWire();
		}
		S4BabyHit *ht = new S4BabyHit(exacthitp);
		Int_t r1 = wirep->Append(ht);
		switch (r1) {
			case -1: {		// 1st hit
				cellp->Append(wirep);
				layercurp->Append(cellp);
				break;
			}
			case  0: break; 	// appended
			case  3: {		// new layer
				std::cerr << " S4BabyHitMakerBuf::AppendCell:LogicError!!"
				     << " new layer not expected." << std::endl;
				break;
			}
			case  2: {		// new cell
				std::cerr << " S4BabyHitMakerBuf::AppendCell:LogicError!!"
				     << " new cell not expected." << std::endl;
				break;
			}
			case  1: {		// new wire
#ifdef __DEBUG__
			        std::cerr << " S4BabyHitMakerBuf::AppendCell: WireNumber = "
			             << wirep->GetWireNumber() << " "
			             << wirep->GetEntries() << " hits" << std::endl;	 
#endif       
			        wirep->KillHits();
				wirep = new S4BabyHitWire();
				wirep->Append(ht);
				cellp->Append(wirep);
				break;
			}
		}
	}
	if ( wirep && !wirep->IsKilled() ){
	       	wirep->KillHits();
         }
#ifdef __DEBUG__
        std::cerr << " S4BabyHitMakerBuf::AppendCell: WireNumber = "
	     << wirep->GetWireNumber() << " "
             << wirep->GetEntries() << " hits" << std::endl;	        
#endif       
}













