///////////////////////////////////////////////////////////////////
//
//  S4CALCalibration
//
//  Sample program
//
//$Id: S4CALCalibration.cxx,v 1.1 2004/08/18 01:17:12 miyamoto Exp $ 
//  
//////////////////////////////////////////////////////////////////

#include <iostream>

#include "JSFSteer.h"
#include "JSFJ4.h"
#include "S4CALCalibration.h"
#include "S4CALExactHit.h"

#include "TNtuple.h"
TNtuple *fCalib;

ClassImp(S4CALCalibration)

//_____________________________________________________________________________
  S4CALCalibration::S4CALCalibration(const char *name, const char *title)
       : JSFModule(name,title)
{

/*
  //  Example to get parameter  
  fFlag = gJSF->Env()->GetValue("S4CALCalibration.Flag",3);
  sscanf(gJSF->Env()->GetValue("S4CALCalibration.Parameter","3.14"),"%g",&fParameter);
*/

}

//_____________________________________________________________________________
S4CALCalibration::~S4CALCalibration()
{
}

//_____________________________________________________________________________
Bool_t S4CALCalibration::Initialize()
{
  // Initialize

  fCalib=new TNtuple("fCalib","CAL Calibration","etot:eem:ehd");

  return kTRUE;
}

//_____________________________________________________________________________
Bool_t S4CALCalibration::Process(Int_t nev)
{
// 

  JSFJ4 *jsfj4=(JSFJ4*)gJSF->FindModule("JSFJ4");
  JSFJ4Buf  *jsfj4buf=(JSFJ4Buf*)jsfj4->EventBuf();
  TObjArray *calhits=jsfj4buf->GetComponent("CALHits");

  std::cout << " There are " << calhits->GetEntries() << " hits" << std::endl; 
  Double_t etot=0;
  Double_t eem=0;
  Double_t ehd=0;

  TIter next(calhits);
  S4CALExactHit *calh=0;
  while( (calh=(S4CALExactHit*)next() ) ) {
    calh->Print();
    etot+=calh->GetEnergyDeposit();
    if ( calh->IsEM() ) {
      eem+=calh->GetEnergyDeposit();
    }
    else {
      ehd+=calh->GetEnergyDeposit();
    }      

  }

  fCalib->Fill(etot, eem, ehd);

  return kTRUE;
}

//_____________________________________________________________________________
Bool_t S4CALCalibration::BeginRun(Int_t nrun)
{
  return kTRUE;
}

//_____________________________________________________________________________
Bool_t S4CALCalibration::EndRun()
{
  return kTRUE;
}

//_____________________________________________________________________________
Bool_t S4CALCalibration::Terminate()
{
  return kTRUE;
}
