#ifndef __S4CALREALCLUSTERMAKERBUF__
#define __S4CALREALCLUSTERMAKERBUF__

//*************************************************************************
//* ---------------------
//* S4CALRealClusterMakerBuf Class
//* ---------------------
//* (Description)
//*	This set of classes makes CALClusters. 
//* (Requires)
//*	JSFEventBuf
//* (Provides)
//*	S4CALRealClusterMakerBuf		: buffering class
//* (Update Record)
//*	2004/09/04  S.Yamamoto	Original version.
//*************************************************************************

#include "TObjArray.h"
#include "S4CALRealClusterMaker.h"
#include "S4CALRealCluster.h"
#include "S4CALSmallCluster.h"
#include "S4HYBTrack.h"
#include <utility>
#include <map>
#include <vector>


class S4CALRealClusterMakerBuf : public JSFEventBuf {

friend class S4CALRealClusterMaker;

 public:

 S4CALRealClusterMakerBuf(S4CALRealClusterMaker *mod     = 0,
			  const Char_t *name  = "S4CALRealClusterMakerBuf",
			  const Char_t *title = "CAL Real Cluster Buffer", 
			  Int_t       bufsize = 500);
 virtual ~S4CALRealClusterMakerBuf();
    
 virtual JSFEventBuf *GetInputDataBufPtr();
 virtual TObjArray   *GetOutputBufPtr();

 virtual Int_t	GetEntries(); // return number of small clusters
 virtual void   SetInputDataBufPtr(JSFEventBuf *buf); // set ptr to input buf

 virtual void 	DebugPrint(const Char_t *opt = "");  


 private:  
 // this function make "Real Clusters"
 virtual void MakeRealClusters();

 // cheated real cluster maker (kink etc. are not considered)
 virtual void MakeCheatedRealClusters();

 //---------------------------------------------------------
 // Map maker
 //---------------------------------------------------------

 private:

 std::map<Int_t, S4CALSmallCluster*> MakeSmallClusterMap(Int_t nhitscut = 0);
 std::map<Int_t, S4HYBTrack*> MakeTrackMap();

 //=== data member ==============================

 private:
  Bool_t	fDone;	 	   // (1,0) = (small clustering is done, not)
  JSFEventBuf  *fInputDataBufPtr;  // Input data buffer ptr
  TObjArray    *fOutputBufPtr;     // Output buffer ptr
  Int_t         fBufSize;          // output buffer size




 //=============================================================
 // Cheated clustering/efficiency checking method
 //-------------------------------------------------------------

 private:

 //---------------------------------------------------------
 // "Cheated" k/pi finder in EC region
 //---------------------------------------------------------
 void MakeKPIClusters(std::map<Int_t, S4CALRealCluster*> &clmap, std::map<Int_t, S4CALSmallCluster*> &sclmap, std::map<Int_t,S4HYBTrack*> &trkmap);

 //---------------------------------------------------------
 // "Cheated" Gamma finder
 //---------------------------------------------------------
 void MakeGammaCheatedClusters(std::map<Int_t, S4CALRealCluster*> &clmap, std::map<Int_t, S4CALSmallCluster*> &sclmap);

 //---------------------------------------------------------
 // "Cheated"cluster maker for endcap region
 //---------------------------------------------------------
 void MakeECClusters(std::map<Int_t, S4CALRealCluster*> &clmap, std::map<Int_t, S4CALSmallCluster*> &sclmap, std::map<Int_t,S4HYBTrack*> &trkmap);

 //---------------------------------------------------------
 // Unlock all S4CALHit/S4HYBTrack objects for Cheated PFA
 //---------------------------------------------------------
 void UnlockAll();


  ClassDef(S4CALRealClusterMakerBuf, 1) // Cluster data buffer

};

#endif
