//*************************************************************************   
//* ---------------------                      
//* S4CALSmallCluster Class
//* ---------------------   
//* (Description) 
//*     represent calorimeter cluster
//* (Requires)
//*     S4CALHit
//* (Provides)  
//*     S4CALSmallCluster            : made object class
//* (Update Record)
//*     2004/09/03  S.Yamamoto  Original version.
//*     2004/11/15  S.Yamamoto  Add IsOK
//*************************************************************************

#include "S4CALSmallCluster.h"

#include <iostream>

//#define __DEBUG__

Int_t    S4CALSmallCluster::fgMaxNHits    = 20;
Double_t S4CALSmallCluster::fgEMHitEmin   = 50.e-6; // GeV
//Double_t S4CALSmallCluster::fgEMHitTOFmax = 500.;   // nsec
Double_t S4CALSmallCluster::fgEMHitTOFmax = 100.;   // nsec
Double_t S4CALSmallCluster::fgHDHitEmin   = 50.e-6;
Double_t S4CALSmallCluster::fgHDHitTOFmax = 1000.;
Double_t S4CALSmallCluster::fgMaxEnergy   = 2.;     // GeV


ClassImp(S4CALSmallCluster)

//=====================================================================
//* constructor -------------------------------------------------------
S4CALSmallCluster::S4CALSmallCluster()
              :	fIsMIP(kFALSE),
		fIsEM(kTRUE),
		fILPos(0.,0.,0.),
		fILTrackPtr(0)
{
  fPFOMap.clear();
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CALSmallCluster::~S4CALSmallCluster()
{
}

//=====================================================================
//* Add ---------------------------------------------------------------

void S4CALSmallCluster::Add(const S4CALHit *hitp)
{
  S4VCluster::Add(hitp);
  fIsEM = hitp->IsEM();
}

//=====================================================================
//* Add ---------------------------------------------------------------

void S4CALSmallCluster::Add(S4CALSmallCluster* scl)
{

  //if(IsEM() != scl->IsEM()) return; // !!! not necessary??

  std::vector<S4CALHit*> shitv = scl->GetCALHitsVector();

  for(Int_t l = 0; l < (Int_t) shitv.size(); l++)  Add(shitv[l]); // add hits
  
  if(!scl->IsOwner())  delete scl;
  
}

//=====================================================================
//* DebugPrint --------------------------------------------------------  

void S4CALSmallCluster::DebugPrint(const Char_t *opt)
{
}


////////////////////////////////////////////////////////////////////////
// Cheating method                                                    //

//=====================================================================
//* IsMuon ------------------------------------------------------------  

Bool_t S4CALSmallCluster::IsMuon(Double_t cut)
{

  if(!fDone) Update();
 
  Double_t emu = 0.;
 
  std::map<Int_t,Double_t>::iterator piditr;

  for(piditr = fPIDMap.begin(); piditr != fPIDMap.end(); piditr++){
    if(TMath::Abs(piditr->first) == 13)  emu += piditr->second;
  }

  Double_t ratio = emu/GetEnergyDeposit();

  if(ratio > cut) return kTRUE;
  else            return kFALSE;

}

//=====================================================================
//* IsKPI -------------------------------------------------------------  

Bool_t S4CALSmallCluster::IsKPI(Double_t cut)
{

  if(!fDone) Update();
 
  Double_t epi = 0.;
  Double_t eck = 0.;
  Double_t ep  = 0.;

  std::map<Int_t,Double_t>::iterator piditr;

  for(piditr = fPIDMap.begin(); piditr != fPIDMap.end(); piditr++){
    if(TMath::Abs(piditr->first) == 211)       epi += piditr->second;
    else if(TMath::Abs(piditr->first) == 321)  eck += piditr->second;
    else if(TMath::Abs(piditr->first) == 2212) ep  += piditr->second;
  }

  Double_t ratio = (epi+eck+ep)/GetEnergyDeposit();

  if(ratio > cut) return kTRUE;
  else            return kFALSE;

}
