//*************************************************************************
//* ---------------------
//* S4CALSmallClusterUtils Class
//* ---------------------
//* (Description)
//*
//* (Requires)
//*
//* (Provides)
//*
//* (Update Record)
//*
//*************************************************************************
#include "S4CALSmallClusterUtils.h"
#include "JSFHelicalTrack.h"
#include "S4CALPreHit.h"
#include "S4CALHit.h"
#include "S4CALExactHit.h"
#include "S4CALAddress.h"

#include "TNtuple.h"

#include <iostream>

// for MakeClustersLDC()
Double_t S4CALSmallClusterUtils::fgEMRCutMin = 2.;  // cm
Double_t S4CALSmallClusterUtils::fgEMRCutMax = 10.; // cm
Double_t S4CALSmallClusterUtils::fgHDRCutMin = 10.; // cm
Double_t S4CALSmallClusterUtils::fgHDRCutMax = 50.; // cm
Double_t S4CALSmallClusterUtils::fgEMDCut    = 2.;  // cm
Double_t S4CALSmallClusterUtils::fgHDDCut    = 5.;  // cm
Int_t    S4CALSmallClusterUtils::fgEMNSteps  = 5;
Int_t    S4CALSmallClusterUtils::fgHDNSteps  = 5;

// for MergeSmallClustersLDC()
Double_t S4CALSmallClusterUtils::fgEMDistCutBack  = 10.;
Double_t S4CALSmallClusterUtils::fgEMDistCutFwd   = 5.;
Double_t S4CALSmallClusterUtils::fgHDDistCutBack  = 50.;
Double_t S4CALSmallClusterUtils::fgHDDistCutFwd   = 10.;
Int_t    S4CALSmallClusterUtils::fgNHitsCut       = 15;
Int_t    S4CALSmallClusterUtils::fgMinLayCut      = -1;

// for MergeSmallClustersdXdT()
Double_t S4CALSmallClusterUtils::fgEMdX = 10.; // cm
Double_t S4CALSmallClusterUtils::fgHDdX = 20.; // cm
Double_t S4CALSmallClusterUtils::fgEMdT = 20.; // nsec
Double_t S4CALSmallClusterUtils::fgHDdT = 40.; // nsec

// for ILFinder()
Double_t S4CALSmallClusterUtils::fgILEMTubeR      = 6.;
Double_t S4CALSmallClusterUtils::fgILHDTubeR      = 20.;
Double_t S4CALSmallClusterUtils::fgILMIPCutEnergy = 700.e-6;

ClassImp(S4CALSmallClusterUtils)


//=====================================================================
//* constructor -------------------------------------------------------
S4CALSmallClusterUtils::S4CALSmallClusterUtils()
{
}


//=====================================================================
//* destructor --------------------------------------------------------

S4CALSmallClusterUtils::~S4CALSmallClusterUtils()
{
}

//===============================================================
// MakeClustersLDC
std::vector<S4CALSmallCluster*> S4CALSmallClusterUtils::MakeClustersLDC(std::map<Int_t,S4CALHit*> &hitmap, Bool_t isem)
{

   //----- cut values for Step 2. -------
   // LDC's parameter
  const Double_t emrcutmin = fgEMRCutMin;
  const Double_t emrcutmax = fgEMRCutMax;
  const Double_t hdrcutmin = fgHDRCutMin;
  const Double_t hdrcutmax = fgHDRCutMax;
  const Double_t emdcut    = fgEMDCut;
  const Double_t hddcut    = fgHDDCut;
  const Int_t    emnsteps  = fgEMNSteps;
  const Int_t    hdnsteps  = fgHDNSteps;
  //------------------------------------
  
  std::vector<S4CALSmallCluster*> sclvec;
  
  //===============================================================
  // Step 1. Sort hits
  
#if 1
  std::vector<S4CALHit*> hitv = SortHitMap(hitmap);
#else
  TObjArray hitsarray(30000);
  
  std::map<Int_t,S4CALHit*>::iterator hitr;
  for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++){
    hitsarray.Add(hitr->second);
  }
  S4VHit::SetSortBy(4); // sort by position
  hitsarray.UnSort();
  hitsarray.Sort();
#endif
  
  //===============================================================
  // Step 2. Small(?) clustering
  
  Double_t rcutmin = 0.;
  Double_t rcutmax = 0.;
  Double_t dcut    = 0.;
  Int_t    nsteps  = 0;
  
  for(Int_t i = 0; i < (Int_t) hitv.size(); i++){ // remaining hits loop

    if(hitv[i]->IsLocked()) continue;

    hitv[i]->Lock();
    
    if(hitv[i]->IsEM()){
      rcutmin = emrcutmin;
      rcutmax = emrcutmax;
      dcut    = emdcut;
      nsteps  = emnsteps;
    }else{
      rcutmin = hdrcutmin;
      rcutmax = hdrcutmax;
      dcut    = hddcut;
      nsteps  = hdnsteps;
    }
    
    Double_t dmin  = dcut; // minimum arc which satisfy "dmin < dcut" (cm)
    Double_t rstep = (Double_t) ((rcutmax-rcutmin)/nsteps);
     
    S4CALSmallCluster* matchscl = 0; // matched small cluster
    
    TVector3 hitpos = hitv[i]->GetCellPosition();

    //----------- set rcut value bigger and bigger ---------------
    for(Double_t rcut = rcutmin; rcut <= rcutmax; rcut += rstep){// rstep loop
      
      //----- find the candidate of matched small cluster ------
      for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){ // scls loop
	
	if(isem && (hitv[i]->IsEM() != sclvec[k]->IsEM())) continue;
	
	S4CALHit* calhit = dynamic_cast<S4CALHit*>(sclvec[k]->First());
	TVector3 seedpos = calhit->GetCellPosition();
	
	std::vector<S4CALHit*> shitv = sclvec[k]->GetCALHitsVector();
	for(Int_t l = 0; l < (Int_t) shitv.size(); l++){ // scl cal hits loop
	  
	  TVector3 pos = shitv[l]->GetCellPosition();
	  
	  Double_t distance = (hitpos-pos).Mag();
	  Double_t angle    = (pos-seedpos).Angle(hitpos-pos);
	  Double_t arc      = distance*angle;
	  
	  //if( (distance < rcut) && (angle < dmin) ){
	  if( (distance < rcut) && (arc < dmin) ){
	    dmin     = arc;
	    matchscl = sclvec[k];
	  }
	  
	} // end of l-loop (small cluster cal hits loop)
	
       } // end of k-loop (small clusters loop)
      //--------------------------------------------------------
      
       if(matchscl){
	 matchscl->Add(hitv[i]);
	 matchscl->Update();
	 break;
       } // goto next hit
       
    } // end of rcut loop
    //------------------------------------------------------------
    
    if(!matchscl){
      S4CALSmallCluster* scl = new S4CALSmallCluster();
      scl->Add(hitv[i]);
      scl->Update();
      sclvec.push_back(scl);
    }
    
  } // end of i-loop (remaining hits loop)
  
  return sclvec;
  
}

//===============================================================
// MergeSmallClustersdXdT
std::vector<S4CALSmallCluster*> S4CALSmallClusterUtils::MergeSmallClustersdXdT(std::vector<S4CALSmallCluster*> invec, Bool_t isem, Char_t* opt)
{

  Double_t emdx = fgEMdX;
  Double_t hddx = fgHDdX;
  Double_t emdt = fgEMdT;
  Double_t hddt = fgHDdT;

  Double_t dx = 0.;
  Double_t dt = 0.;

  Bool_t emonly = kFALSE;
  Bool_t hdonly = kFALSE;

  // temporary
  if(opt[0] == 'E') emonly = kTRUE;
  if(opt[0] == 'H') hdonly = kTRUE;

  std::vector<S4CALSmallCluster*> sclvec;
  std::vector<S4CALSmallCluster*> tmpvec;

  std::multimap<Double_t, S4CALSmallCluster*> tmpmap;
  std::multimap<Double_t,S4CALSmallCluster*>::reverse_iterator ritr;
  //---- sort by energy ----
  for(Int_t k = 0; k < (Int_t) invec.size(); k++){
    if(invec[k]->IsLocked() || invec[k]->IsMIP()) tmpvec.push_back(invec[k]);
    else tmpmap.insert(std::pair<Double_t,S4CALSmallCluster*>(invec[k]->GetEnergy(),invec[k]));
  }
  //------------------------

  //------ clustering ------
  for(ritr = tmpmap.rbegin(); ritr != tmpmap.rend(); ritr++){

    Bool_t match = kFALSE;

    (ritr->second->IsEM()) ? (dx = emdx, dt = emdt) : (dx = hddx, dt = hddt);

    for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){

      if( isem && (ritr->second->IsEM() != sclvec[k]->IsEM()) )      continue;
      if( emonly && !(ritr->second->IsEM() && sclvec[k]->IsEM()) )   continue;
      if( hdonly && !(!ritr->second->IsEM() && !sclvec[k]->IsEM()) ) continue;

      TVector3 center = sclvec[k]->GetPosition();
      TVector3 sclpos = ritr->second->GetPosition();
      Double_t ctof   = sclvec[k]->GetTOF();
      Double_t scltof = ritr->second->GetTOF();

      if( ((sclpos-center).Mag() < dx) && (TMath::Abs(scltof-ctof) < dt) ){
	sclvec[k]->Add(ritr->second);
	match = kTRUE;
	break;
      }

    }

    if(!match)  sclvec.push_back(ritr->second);

  }

  for(Int_t k = 0; k < (Int_t) tmpvec.size(); k++)
    sclvec.push_back(tmpvec[k]);

  return sclvec;

}

//===============================================================
// MergeSmallClustersLDCSCL
std::vector<S4CALSmallCluster*> S4CALSmallClusterUtils::MergeSmallClustersLDCSCL(std::vector<S4CALSmallCluster*> invec, Bool_t isem)
{

  std::vector<S4CALSmallCluster*> sclvec;

  //---------- sort small clusters/hits -----------
  std::vector<S4CALSmallCluster*> sclvtmp;
  std::vector<S4CALSmallCluster*> sclv;

  std::multimap<Double_t,S4CALSmallCluster*> tmppmap;
  std::multimap<Double_t,S4CALSmallCluster*>::iterator tmppitr;

  for(Int_t k = 0; k < (Int_t) invec.size(); k++){
    Double_t tmppos = (invec[k]->GetPosition()).Mag();
    tmppmap.insert(std::pair<Double_t,S4CALSmallCluster*>(tmppos,invec[k]));
  }

  for(tmppitr = tmppmap.begin(); tmppitr != tmppmap.end(); tmppitr++){
    sclvtmp.push_back(tmppitr->second);
  }
  //-----------------------------------------------

  //------ separate small clusters --------
  for(Int_t k = 0; k < (Int_t) sclvtmp.size(); k++){
    if(sclvtmp[k]->IsLocked() || sclvtmp[k]->IsMIP())
      sclvec.push_back(sclvtmp[k]);
    else
      sclv.push_back(sclvtmp[k]);
  }
  //---------------------------------------


  //--------------------------- merging -----------------------------------
  Double_t rcutmin = 0.;
  Double_t rcutmax = 0.;
  Double_t dcut    = 0.;
  Int_t    nsteps  = 0;
  
  for(Int_t i = 0; i < (Int_t) sclv.size(); i++){ // remaining scls loop
    
    if(sclv[i]->IsEM()){
      rcutmin = 6.;
      rcutmax = 15.;
      dcut    = 6.;
      nsteps  = 5;
    }else{
      rcutmin = 20.;
      rcutmax = 50.;
      dcut    = 10.;
      nsteps  = 5;
    }
    
    Double_t dmin  = dcut; // minimum arc which satisfy "dmin < dcut" (cm)
    Double_t rstep = (Double_t) ((rcutmax-rcutmin)/nsteps);
    
    S4CALSmallCluster* matchscl = 0; // matched small cluster

    TVector3 sclpos = sclv[i]->GetPosition();
    
    //----------- set rcut value bigger and bigger ---------------
    for(Double_t rcut = rcutmin; rcut <= rcutmax; rcut += rstep){// rstep loop
      
      //----- find the candidate of matched small cluster ------
      for(Int_t k = 0; k < (Int_t) sclvec.size(); k++){ // scls loop
	
	//!!!! not necessary, but... !!!!
	if(isem)
	  if(sclv[i]->IsEM() != sclvec[k]->IsEM()) continue;

	TVector3 minpos = sclvec[k]->GetMinLayerPos();
	
	std::vector<S4CALHit*> shitv = sclvec[k]->GetCALHitsVector();
	for(Int_t l = 0; l < (Int_t) shitv.size(); l++){ // scl cal hits loop
	  
	  TVector3 pos = shitv[l]->GetCellPosition();
	  Double_t distance = (sclpos-pos).Mag();
	  Double_t angle    = (pos-minpos).Angle(sclpos-pos);
	  //Double_t distance = (sclpos-sclvec[k]->GetPosition()).Mag();
	  //Double_t angle    = (sclpos-sclvec[k]->GetPosition()).Angle(sclvec[k]->GetThrustVector());
	  Double_t arc      = distance*angle;
	  
	  //if( (distance < rcut) && (angle < dmin) ){
	  if( (distance < rcut) && (arc < dmin) ){
	    dmin     = arc;
	    matchscl = sclvec[k];
	  }
	  
	} // end of l-loop (small cluster cal hits loop)
	
      } // end of k-loop (small clusters loop)
      //--------------------------------------------------------
      
      if(matchscl){
	matchscl->Add(sclv[i]);
	matchscl->Update();
	break;
      } // goto next "small" small cluster
      
    } // end of rcut loop
    //------------------------------------------------------------
    
    if(!matchscl){
      sclvec.push_back(sclv[i]);
    }
    
  } // end of i-loop (remaining scls loop)
  //-----------------------------------------------------------------------

  return sclvec;

}

//=====================================================================
//* MergeSmallClustersLDC ---------------------------------------------

std::vector<S4CALSmallCluster*> S4CALSmallClusterUtils::MergeSmallClustersLDC(std::vector<S4CALSmallCluster*> &sclvec, Bool_t isem)
{
  std::vector<S4CALSmallCluster*> msclvec;

   //---- temporary container for "small" smallcluster ----
   std::vector<S4CALSmallCluster*> smalls;

   //------ separate small clusters by nhits -------
   S4VHit::SetSortBy(4); // sort by position
   for(Int_t i = 0; i < (Int_t) sclvec.size(); i++){
     sclvec[i]->UnSort();
     sclvec[i]->Sort();

     Int_t tmpnhits = sclvec[i]->GetNHits();
     Int_t minlayer = sclvec[i]->GetMinLayerID();

     if(!sclvec[i]->IsLocked() && !sclvec[i]->IsMIP()
	&& tmpnhits < fgNHitsCut && minlayer > fgMinLayCut){
       smalls.push_back(sclvec[i]);
     }else{
       msclvec.push_back(sclvec[i]);
     }
   }
   //-----------------------------------------------


   //----- merge "small" smallclusters into msclvec ------
   for(Int_t i = 0; i < (Int_t) smalls.size(); i++){ // "small" scls loop

     Double_t dcutback = 0.;
     Double_t dcutfwd  = 0.;
     Double_t dcutmin  = 100000;
     S4CALSmallCluster* mergedscl = 0; // small scl will be added into this

     //------- check the nearest/farthest hit -------
     S4CALHit* nearhit  = dynamic_cast<S4CALHit*>(smalls[i]->First());
     S4CALHit* farhit   = dynamic_cast<S4CALHit*>(smalls[i]->Last());
     TVector3  nearpos  = nearhit->GetCellPosition();
     TVector3  farpos   = farhit->GetCellPosition();
     Double_t  neardist = nearpos.Mag();
     Double_t  fardist  = farpos.Mag();
     nearhit->IsEM() ? dcutback = fgEMDistCutBack : dcutback = fgHDDistCutBack;
     //farhit->IsEM()  ? dcutfwd  = fgEMDistCutFwd  : dcutfwd  = fgHDDistCutFwd;
     //----------------------------------------------

     //------ find the nearest "large" small cluster -------
     for(Int_t k = 0; k < (Int_t) msclvec.size(); k++){ // "large" scls loop

       // Double_t anglecut = TMath::Pi()/6.;
       // TVector3 relpos = 0.;
       // if(smalls[i]->GetPosition().Mag() > msclvec[k]->GetPosition().Mag())
       //   relpos = smalls[i]->GetPosition() - msclvec[k]->GetPosition();
       // else
       //   relpos = msclvec[k]->GetPosition() - smalls[i]->GetPosition();
       // Double_t angle  = (msclvec[k]->GetThrustVector()).Angle(relpos);
       // if(angle > anglecut) continue;
       

       //!!!!! separate ECAL/HCAL for GLD small clustering
       if(isem)
	 if(msclvec[k]->IsEM() != smalls[i]->IsEM()) continue;

       std::vector<S4CALHit*> shitv = msclvec[k]->GetCALHitsVector();

       S4CALHit* nearhit2 = dynamic_cast<S4CALHit*>(msclvec[k]->First());
       nearhit2->IsEM() ? dcutfwd = fgEMDistCutFwd : dcutfwd = fgHDDistCutFwd;

       for(Int_t l = 0; l < (Int_t) shitv.size(); l++){ // hits loop

	 TVector3 hpos = shitv[l]->GetCellPosition();

	 if(neardist > hpos.Mag() && (nearpos-hpos).Mag() < dcutback && (nearpos-hpos).Mag() < dcutmin){

	   dcutmin = (nearpos-hpos).Mag();
	   mergedscl = msclvec[k];
	 }
	 if(fardist < hpos.Mag() && (farpos-hpos).Mag() < dcutfwd && (farpos-hpos).Mag() < dcutmin){
	   dcutmin = (farpos-hpos).Mag();
	   mergedscl = msclvec[k];
	 }

       } // end of hits loop

     } // end of "large" scls loop
     //-----------------------------------------------------

     //----------- merge -------------
     if(mergedscl){ // merge "small" small cluster into larger one

       // Problem: how to treat IsEM()??
       mergedscl->Add(smalls[i]);
       mergedscl->Update();

     }else{
       msclvec.push_back(smalls[i]);
     }
     //-------------------------------

   } // end of "small" scls loop
   //-----------------------------------------------------


  return msclvec;

}

//===============================================================
// ILFinder
std::pair<TVector3,std::vector<S4CALHit*> > S4CALSmallClusterUtils::ILFinder(std::map<Int_t,S4CALHit*> &hitmap, S4HYBTrack* trk)
{

//#define __THRUSTCUT__
//#define __LAYER__

  static Int_t ilnevt = 0;
  ilnevt++;

  std::pair<TVector3,std::vector<S4CALHit*> > ilpair;



#ifdef __THRUSTCUT__
  const Double_t thrustcut  = 0.5;  // thrust cut value for all collected hits
#endif

#ifdef __LAYER__

  //static const Double_t emactthick = gJSF->Env()->GetValue("J4CAL.EM.ActiveLayerThickness",0.1);
  //static const Double_t hdactthick = gJSF->Env()->GetValue("J4CAL.HD.ActiveLayerThickness",0.2);
  //  static const Int_t emnlayers = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);
  //  static const Int_t hdnlayers = gJSF->Env()->GetValue("J4CAL.HD.NLayers",130);
  static const Int_t emnlayers = S4CALAddress::GetEMNLayers();
  static const Int_t hdnlayers = S4CALAddress::GetHDNLayers();

  const Int_t    nsteps      = emnlayers+hdnlayers;
  const Double_t range       = (Double_t) nsteps;
  const Int_t    countcut    = 1;
  const Int_t    mipcountcut = 3;
  const Int_t    sumcut      = 3;
#else

  //  static const Double_t emactthick = gJSF->Env()->GetValue("J4CAL.EM.ActiveLayerThickness",0.1);
  static const Double_t emactthick = S4CALAddress::GetEMActiveLayerThickness();
  //static const Double_t hdactthick = gJSF->Env()->GetValue("J4CAL.HD.ActiveLayerThickness",0.2);
  //  static const Double_t emabsthick = gJSF->Env()->GetValue("J4CAL.EM.AbsLayerThickness",0.1);
  static const Double_t emabsthick = S4CALAddress::GetEMAbsLayerThickness();
  static const Double_t emlthick   = emactthick+emabsthick + 0.1;
  const Int_t    nsteps      = 200;
  const Double_t range       = 100.;
  const Int_t    countcut    = 2;
  const Int_t    mipcountcut = 5;
  const Int_t    sumcut      = 7;

#endif

  //  static Double_t calinR = gJSF->Env()->GetValue("J4CAL.BarrelTower.FronRadius",210.);
  //  static Double_t calinZ = gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270.);
  static Double_t calinR = S4CALAddress::GetBarrelInnerR();
  static Double_t calinZ = S4CALAddress::GetBarrelHalfZ();

  std::map<Int_t,S4CALHit*>::iterator hitr;

  std::vector<S4CALHit*> ilinvec;
  ilpair = std::pair<TVector3,std::vector<S4CALHit*> >(TVector3(0.,0.,0.),ilinvec);

  Int_t nhits = 0;
  Int_t nhits1 = 0;
  //--------- collect hits within a tube -------------
  for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++){
    nhits1++;
    if(hitr->second->IsLocked()) continue;

    Double_t dis = S4CALSmallClusterUtils::CalcTrackDistance(trk,hitr->second->GetCellPosition());
    if( hitr->second->IsEM() && dis < fgILEMTubeR){
      ilinvec.push_back(hitr->second);
      nhits++;
    }
    if(!hitr->second->IsEM() && dis < fgILHDTubeR){
      ilinvec.push_back(hitr->second);
      nhits++;
    }
  }
  //--------------------------------------------------

  //------------- calculate IL position -------------
  Double_t x[nsteps];
  Double_t etot[nsteps];
  for(Int_t n = 0; n < nsteps; n++){
    x[n]    = n*range/nsteps;
    etot[n] = 0.;
  }

  std::multimap<Int_t,S4CALHit*> samplemap;

  std::vector<TVector3> posmom = S4CALSmallClusterUtils::GetInTrack(trk,calinR,calinZ);
  TVector3 inpos = posmom[0];
  TVector3 inmom = posmom[1];

  TVector3 ilpos(0.,0.,0.);

  //------ sampling -------
  for(Int_t i = 0; i < (Int_t) ilinvec.size(); i++){

#ifdef __LAYER__
    Int_t layerID = S4CALAddress::GetLayerID(ilinvec[i]->GetCellID(), ilinvec[i]->IsEM());
    Double_t etmp = ilinvec[i]->GetEnergyDeposit();
    if(!ilinvec[i]->IsEM()){
      layerID += emnlayers;
      //etmp *= emactthick/hdactthick;
    }
    etot[layerID] += etmp;
    samplemap.insert(std::pair<Int_t,S4CALHit*>(layerID,ilinvec[i]));
#else
    TVector3 hitpos = ilinvec[i]->GetCellPosition();
    Double_t xtmp = (hitpos-inpos).Mag();
    Double_t etmp = ilinvec[i]->GetEnergyDeposit();
    //if(!ilinvec[i]->IsEM())  etmp *= emactthick/hdactthick;
    for(Int_t n = 0; n < nsteps; n++){
      if(xtmp < x[n]+range/nsteps && xtmp > x[n]){
	etot[n] += etmp;
	samplemap.insert(std::pair<Int_t,S4CALHit*>(n,ilinvec[i]));
	break;
      }
    }
#endif
  }
  //------------------------

  //------ find & set IL position -------
  Int_t mipcount  = 0;
  Int_t count     = 0;
  Int_t countsum  = 0;
  Int_t nIL       = 0;
  Int_t nnosignal = 0;
  Int_t nsignal   = 0;

  for(Int_t n = 0; n < nsteps; n++){

    if(etot[n] < 1.e-6){
      nnosignal++;
      continue;
    }else{
      nsignal++;
    }
    
    //--- check if track is trustable ---
#ifdef __LAYER__
    if(n > 2 && (Double_t) n/nsignal > 3.)  break;
#else
    if( x[n]/emlthick > 3. && (Double_t) (nsignal+nnosignal)/nsignal > 3.*emlthick/emactthick )
      break;
#endif
    //-----------------------------------    
    
    (etot[n] > fgILMIPCutEnergy) ? (count++, countsum++) : (count = 0, mipcount++);
    
    if(countsum > sumcut) break;
    if(count > countcut && mipcount > mipcountcut){
      Int_t istep = 0;
      for(Int_t i = n; i >= 0; i--){
	if(etot[i] > 0.) istep++;
	if(istep > countcut){
	  nIL = i;
	  break;
	}
      }
      break;
    }

  }

  if(nIL < 1) return ilpair; // IL is not found

  typedef std::multimap<Int_t,S4CALHit*>::iterator MULTIITR;
  MULTIITR sampitr;
  std::pair<MULTIITR,MULTIITR> itr_range = samplemap.equal_range(nIL);

  for(sampitr = itr_range.first; sampitr != itr_range.second; sampitr++){
    S4CALHit* calhit = sampitr->second;
    ilpos += (calhit->GetCellPosition())*(calhit->GetEnergyDeposit());
  }

  ilpos *= (1./etot[nIL]);
  //-------------------------------------

  //-------------------------------------------------

#ifdef __THRUSTCUT__
  //---------------- check thrust -------------------
  S4CALSmallCluster scltmp;
  for(Int_t k = 0; k < (Int_t) ilinvec.size(); k++) scltmp.Add(ilinvec[k]);
  scltmp.Update();
  Double_t thrust = scltmp.GetThrust();
  std::cerr << "########### thrust = " << thrust << std::endl;
  if(thrust < thrustcut) continue;
  //-------------------------------------------------
#endif

  //------------ Lock hits ------------
  //for(Int_t k = 0; k < (Int_t) ilinvec.size(); k++){
  //ilinvec[k]->Lock();
  //}
  //-----------------------------------

#if 1 
  std::vector<S4CALHit*> ilinvec2;
  for(Int_t i = 0; i <= nIL; i++){
    itr_range = samplemap.equal_range(i);
    for(sampitr = itr_range.first; sampitr != itr_range.second; sampitr++){
      ilinvec2.push_back(sampitr->second);
    }
  }
  ilpair = std::pair<TVector3,std::vector<S4CALHit*> >(ilpos,ilinvec2);
#else
  ilpair = std::pair<TVector3,std::vector<S4CALHit*> >(ilpos,ilinvec);
#endif

  return ilpair;

}

//==========================================================================
//* CheatedILFinder --------------------------------------------------------

std::map<Int_t,std::pair<Int_t,TVector3> > S4CALSmallClusterUtils::CheatedILFinder(std::map<Int_t,S4CALHit*> hitmap)
{

  std::map<Int_t,std::pair<Int_t,TVector3> > ilmap;  // (post/pre hit track ID, IL position)
  std::map<Int_t,Double_t> tofmap;
  std::map<Int_t,std::pair<Int_t,TVector3> > checker;

  std::map<Int_t,S4CALHit*>::iterator hitr;

  for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++){

    TObjArray *ehits = hitr->second->GetExactHitsPtr();
    TIter next(ehits);
    S4CALExactHit* ehit = 0;

    while( (ehit = (S4CALExactHit*)next()) ){

      Int_t ptrkID = 0;
      Int_t pid    = 0;
      S4CALPreHit *prehitp = ehit->GetPreHitPtr();

#if 1
      if(prehitp->GetPostHitPtr()){
	ptrkID = prehitp->GetPostHitPtr()->GetTrackID();
	pid    = prehitp->GetPostHitPtr()->GetPDGCode();
      }else{
	ptrkID = prehitp->GetTrackID();
	pid    = prehitp->GetPDGCode();
      }
#else
      ptrkID = prehitp->GetTrackID();
      pid    = prehitp->GetPDGCode();
#endif

      if(checker.count(ptrkID) > 0){
	Double_t mindis  = ((checker.find(ptrkID)->second).second).Mag();
	Double_t thisdis = (ehit->GetPosition()).Mag();
	if(thisdis < mindis){
	  TVector3 &minpos = (checker.find(ptrkID)->second).second;
	  minpos = ehit->GetPosition();
	}
      }else{
	checker[ptrkID] = std::pair<Int_t,TVector3>(pid,ehit->GetPosition());
      }

      if( ehit->GetTrackID() != ptrkID ) continue; // choose primary particles

      if(ilmap.count(ptrkID) > 0){
	Double_t &tof = tofmap.find(ptrkID)->second;
	if(ehit->GetTOF() > tof){ // IL position = the latest hit position
	  TVector3 &ilpos = (ilmap.find(ptrkID)->second).second;
	  tof   = ehit->GetTOF();
	  ilpos = ehit->GetPosition();
	}
      }else{
	ilmap[ptrkID] = std::pair<Int_t,TVector3>(pid,ehit->GetPosition());
	tofmap[ptrkID] = ehit->GetTOF();
      }

    } // end of while loop

  }

  std::map<Int_t,std::pair<Int_t,TVector3> >::iterator citr;

  for(citr = checker.begin(); citr != checker.end(); citr++){
    if(ilmap.count(citr->first) < 1) // make shower before reaching to Sci
      ilmap[citr->first] = citr->second;
  }

  return ilmap;

}

//==========================================================================
//* NearestTrackIs ---------------------------------------------------------

std::pair<Int_t,Double_t> S4CALSmallClusterUtils::NearestTrackIs(std::map<Int_t, S4HYBTrack*> &trkmap, TVector3 pos, Double_t pcut)
{

  // Calculate the distance from the nearest track.
  // Return value is (track ID, distance). If there are no track
  // which is close to pos, then return value is (-1, 1.e+5).

  std::map<Int_t,S4HYBTrack*>::iterator titr;

  Int_t    trkID = -1;
  Double_t dist  = 1.e+5;

  for(titr = trkmap.begin(); titr != trkmap.end(); titr++){

    if(titr->second->IsLocked()) continue;

    if(pcut > (titr->second->GetHelicalTrackPtr()->GetMomentum(0.)).Abs()) continue;


    Double_t tmpdist = CalcTrackDistance(titr->second, pos);

    if(tmpdist < dist){
      trkID = titr->second->GetTrackID();
      dist  = tmpdist;
    }

  }

  return std::pair<Int_t,Double_t>(trkID, dist);

}

//==========================================================================
//* GetTrackDistanceList ---------------------------------------------------

std::multimap<Double_t,S4HYBTrack*> S4CALSmallClusterUtils::GetTrackDistanceList(std::map<Int_t,S4HYBTrack*> &trkmap, TVector3 pos, Double_t pcut)
{

  std::multimap<Double_t,S4HYBTrack*> distmap;

  std::map<Int_t,S4HYBTrack*>::iterator titr;

  for(titr = trkmap.begin(); titr != trkmap.end(); titr++){

    if(titr->second->IsLocked()) continue;
    if(pcut > (titr->second->GetHelicalTrackPtr()->GetMomentum(0.)).Abs()) continue;

    Double_t dist = CalcTrackDistance(titr->second, pos);

    if(dist < 1.e+3)
      distmap.insert(std::pair<Double_t,S4HYBTrack*>(dist,titr->second));
    
  } // end of titr loop

  return distmap;

}
#if 0
// new ver.
//==========================================================================
//* CalcTrackDistance ------------------------------------------------------

Double_t S4CALSmallClusterUtils::CalcTrackDistance(S4HYBTrack *trk, TVector3 pos)
{

  //----------------------- CAL parameters -------------------------------
  //  static const Double_t calinR     = gJSF->Env()->GetValue("J4CAL.BarrelTower.FrontRadius",210.);
  //  static const Double_t calinZ     = gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270.);
  //  static const Double_t emabsthick = gJSF->Env()->GetValue("J4CAL.EM.AbsLayerThickness",0.4);
  //  static const Double_t emactthick = gJSF->Env()->GetValue("J4CAL.EM.ActiveLayerThickness",0.1);
  //  static const Double_t hdabsthick = gJSF->Env()->GetValue("J4CAL.HD.AbsLayerThickness",0.8);
  //  static const Double_t hdactthick = gJSF->Env()->GetValue("J4CAL.HD.ActiveLayerThickness",0.2);
  //  static const Int_t    emnlayers  = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);
  //  static const Int_t    hdnlayers  = gJSF->Env()->GetValue("J4CAL.HD.NLayers",130);
  //  static const Double_t magfield   = gJSF->Env()->GetValue("J4SOL.BField",3.);
  static const Double_t calinR     = S4CALAddress::GetBarrelInnerR();
  static const Double_t calinZ     = S4CALAddress::GetBarrelHalfZ();
  static const Double_t emabsthick = S4CALAddress::GetEMAbsLayerThickness(); 
  static const Double_t emactthick = S4CALAddress::GetEMActiveLayerThickness();
  static const Double_t hdabsthick = S4CALAddress::GetHDAbsLayerThickness();
  static const Double_t hdactthick = S4CALAddress::GetHDActiveLayerThickness();
  static const Int_t    emnlayers  = S4CALAddress::GetEMNLayers();
  static const Int_t    hdnlayers  = S4CALAddress::GetHDNLayers();
  static const Double_t magfield   = gJSF->Env()->GetValue("J4SOL.BField",3.);

  static const Double_t caloutR    = calinR+emnlayers*(emabsthick+emactthick+0.1)+hdnlayers*(hdabsthick+hdactthick+0.1);
  static const Double_t caloutZ    = calinZ+emnlayers*(emabsthick+emactthick+0.1)+hdnlayers*(hdabsthick+hdactthick+0.1);
  //----------------------------------------------------------------------

  static const Int_t step = 300; // number of sampling

  Double_t distance = 1.e+10;

  //---------------- check trk, pos ------------------
  if( pos.Perp() < calinR-2. && TMath::Abs(pos.Z()) < calinZ-2. ){
    return distance;
  }
  //if(trk->IsLocked()|| (GetInTrack(trk,calinR,calinZ)).size() < 1 ){
  if(!trk->GetHelicalTrackPtr() || (GetInTrack(trk,calinR,calinZ)).size() < 1 ){
    return distance;
  }
  //--------------------------------------------------

  //-------- check trk at CAL inner surface ----------
  JSFHelicalTrack* heltrackp = trk->GetHelicalTrackPtr();

  Double_t phi  = 0.;
  Double_t phi1 = 0.;
  Double_t startx = (heltrackp->GetCoordinate(0.)).X();
  Double_t starty = (heltrackp->GetCoordinate(0.)).Y();

  Int_t ir = heltrackp->OriginToCylinder(calinR, calinZ, phi,
					 phi1, 1000, startx, starty);

  if( ir != 0 && ir != 1)  return distance;

  JSF3DV tmppos = heltrackp->GetCoordinate(phi1);
  JSF3DV tmpmom = heltrackp->GetMomentum(phi1);
  TVector3 trkpos(tmppos.X(), tmppos.Y(), tmppos.Z());
  TVector3 trkmom(tmpmom.X(),tmpmom.Y(),tmpmom.Z());

#if 0
  if((pos-trkpos).Angle(trkmom) > TMath::Pi()/2.){
#if 0
    distance = (trkpos-pos).Mag();
#else
    Double_t backcut = ((10./trkmom.Mag() > 5.) ?  10./trkmom.Mag() : 5.);

    //if((trkpos-pos).Mag() < 5.) distance = (trkpos-pos).Mag();
    if((trkpos-pos).Mag() < backcut) distance = (trkpos-pos).Mag();
#endif
    return distance;
  }
#endif

  Double_t defz = pos.Z()-trkpos.Z();
  if(TMath::Abs(defz) > 60. && defz*(trkmom.Z()) < 0.)  return distance;
  //--------------------------------------------------


  Double_t pt   = TMath::Sqrt(trkmom.X()*trkmom.X()+trkmom.Y()*trkmom.Y());
  Double_t phi2 = 0.;

  if(pt < TMath::Abs(trkmom.Z())){ // in case Pt < Pz

    heltrackp->OriginToCylinder(caloutR, TMath::Abs(pos.Z()), phi,
				phi2, 1000, startx, starty);

    // track info. is not reliable if it is rotated too many time in the CAL
    if(TMath::Abs(phi2-phi1) > TMath::Pi()/2.) return distance;
    //if(TMath::Abs(phi2-phi1) > TMath::Pi()) return distance;

    tmppos = heltrackp->GetCoordinate(phi2);
    tmpmom = heltrackp->GetMomentum(phi2);
    trkpos = TVector3(tmppos.X(),tmppos.Y(),tmppos.Z());
    trkmom = TVector3(tmpmom.X(),tmpmom.Y(),tmpmom.Z());
      
    Double_t k = (pos - trkpos)*(trkmom*(1/trkmom.Mag()));
    distance = (pos - (k*(trkmom*(1/trkmom.Mag())) + trkpos)).Mag();
    
  }else{  // Pt > Pz

    if(2.*pt/(3.*magfield) > pos.Perp()-5.){

      Int_t ir2 = heltrackp->OriginToCylinder(pos.Perp()-5.,
					      TMath::Abs(pos.Z())+20., phi,
					      phi2, 1000, startx, starty);

      if(ir2 != 0)  return distance;  // not matched

      if(TMath::Abs(phi2-phi1) > TMath::Pi()/2.) return distance;
      //if(TMath::Abs(phi2-phi1) > TMath::Pi()) return distance;

      tmppos = heltrackp->GetCoordinate(phi2);
      tmpmom = heltrackp->GetMomentum(phi2);
      trkpos = TVector3(tmppos.X(),tmppos.Y(),tmppos.Z());
      trkmom = TVector3(tmpmom.X(),tmpmom.Y(),tmpmom.Z());
      
      if((trkpos - pos).Mag() > calinR)    return distance; // enough far away
      
      Double_t k = (pos - trkpos)*(trkmom*(1/trkmom.Mag()));
      distance = (pos - (k*(trkmom*(1/trkmom.Mag())) + trkpos)).Mag();
      
    }else{

      Double_t dphi = (heltrackp->GetCharge() < 0.) ? TMath::Pi()/2. : -1.*TMath::Pi()/2.;
      heltrackp->OriginToCylinder(caloutR,caloutZ,phi,phi2,1000,startx,starty);
      if(TMath::Abs(phi2-phi1) < TMath::Abs(dphi)) dphi = phi2-phi1;
      
      for(Int_t k = 0; k < step; k++){
	tmppos = heltrackp->GetCoordinate(phi1+k*dphi/step);
	tmpmom = heltrackp->GetMomentum(phi1+k*dphi/step);
	trkpos = TVector3(tmppos.X(),tmppos.Y(),tmppos.Z());
	trkmom = TVector3(tmpmom.X(),tmpmom.Y(),tmpmom.Z());
	if((pos-trkpos).Mag() < distance){
	  distance = (pos-trkpos).Mag();
	}else{
	  break; // previous one is minimum
	}
      }      

    }

  } // end of if(pt/TMath::Abs(trkmom.Z()) > 1.)
  
  return distance;
  //---------------------------------------------------

}
#else
//==========================================================================
//* CalcTrackDistance ------------------------------------------------------

Double_t S4CALSmallClusterUtils::CalcTrackDistance(S4HYBTrack *trk, TVector3 pos)
{

  //  static const Double_t calinR = gJSF->Env()->GetValue("J4CAL.BarrelTower.FrontRadius",210.);
  //  static const Double_t calinZ = gJSF->Env()->GetValue("J4CAL.EndcapTower.FrontZ",270.);
  //  static const Double_t emabsthick = gJSF->Env()->GetValue("J4CAL.EM.AbsLayerThickness",0.4);
  //  static const Double_t emactthick = gJSF->Env()->GetValue("J4CAL.EM.ActiveLayerThickness",0.1);
  //  static const Double_t hdabsthick = gJSF->Env()->GetValue("J4CAL.HD.AbsLayerThickness",0.8);
  //  static const Double_t hdactthick = gJSF->Env()->GetValue("J4CAL.HD.ActiveLayerThickness",0.2);
  //  static const Int_t    emnlayers  = gJSF->Env()->GetValue("J4CAL.EM.NLayers",38);
  //  static const Int_t    hdnlayers  = gJSF->Env()->GetValue("J4CAL.HD.NLayers",130);
  //static const Double_t magfield   = gJSF->Env()->GetValue("J4SOL.BField",3.);
  static const Double_t calinR     = S4CALAddress::GetBarrelInnerR();
  static const Double_t calinZ     = S4CALAddress::GetBarrelHalfZ();
  static const Double_t emabsthick = S4CALAddress::GetEMAbsLayerThickness(); 
  static const Double_t emactthick = S4CALAddress::GetEMActiveLayerThickness();
  static const Double_t hdabsthick = S4CALAddress::GetHDAbsLayerThickness();
  static const Double_t hdactthick = S4CALAddress::GetHDActiveLayerThickness();
  static const Int_t    emnlayers  = S4CALAddress::GetEMNLayers();
  static const Int_t    hdnlayers  = S4CALAddress::GetHDNLayers();
  static const Double_t magfield   = gJSF->Env()->GetValue("J4SOL.BField",3.);

  static const Double_t caloutR    = calinR+emnlayers*(emabsthick+emactthick+0.1)+hdnlayers*(hdabsthick+hdactthick+0.1);
  //static const Double_t caloutZ    = calinZ+emnlayers*(emabsthick+emactthick)+hdnlayers*(hdabsthick+hdactthick);
  
  Double_t distance = 1.e+10;

  if( pos.Perp()<calinR && TMath::Abs(pos.Z())<calinZ ){
    return distance;
  }

  //if(trk->IsLocked()|| (GetInTrack(trk,calinR,calinZ)).size() < 1 ){
  if(!trk->GetHelicalTrackPtr() || (GetInTrack(trk,calinR,calinZ)).size() < 1 ){
    return distance;
  }

  JSFHelicalTrack* heltrackp = trk->GetHelicalTrackPtr();

  Double_t phi  = 0.;
  Double_t phi1 = 0.;
  Double_t startx = (heltrackp->GetCoordinate(0.)).X();
  Double_t starty = (heltrackp->GetCoordinate(0.)).Y();

  Int_t ir = heltrackp->OriginToCylinder(calinR, calinZ, phi,
					 phi1, 1000, startx, starty);

  ///////////////////////////////////////////////////////////////////////
  //! def.
#if 0
  JSF3DV checktmpp = heltrackp->GetCoordinate(phi1);
  JSF3DV checktmpm = heltrackp->GetMomentum(phi1);
  TVector3 checkpos(checktmpp.X(), checktmpp.Y(), checktmpp.Z());
  TVector3 checkmom(checktmpm.X(),checktmpm.Y(),checktmpm.Z());

  if((pos-checkpos).Angle(checkmom) > TMath::Pi()/2.){
#if 0
    distance = (checkpos-pos).Mag();
#else
    Double_t backcut = ((10./checkmom.Mag() > 5.) ?  10./checkmom.Mag() : 5.);

    //if((trkpos-pos).Mag() < 5.) distance = (trkpos-pos).Mag();
    if((checkpos-pos).Mag() < backcut)  distance = (checkpos-pos).Mag();
#endif
    return distance;
  }
#endif
  ////////////////////////////////////////////////////////////////////

  if(ir == 0){ // barrel

    //////////////////////////////////////////////////////////////////
    //! def.
#if 1
    Double_t radius = pos.Perp()-10. < calinR ? calinR : pos.Perp()-10.;
#else
    Double_t radius = pos.Perp()-10.;
#endif
    //////////////////////////////////////////////////////////////////

    Int_t ir2 = heltrackp->OriginToCylinder(radius, calinZ+20., phi,
					    phi1, 1000, startx, starty);

    if(ir2 != 0)  return distance;

    JSF3DV tmppos = heltrackp->GetCoordinate(phi1);
    JSF3DV tmpmom = heltrackp->GetMomentum(phi1);

    TVector3 trkpos = TVector3(tmppos.X(), tmppos.Y(), tmppos.Z());
    TVector3 trkmom(tmpmom.X(),tmpmom.Y(),tmpmom.Z());

    if((trkpos - pos).Mag() > calinR)    return distance; // enough far away

    Double_t k = (pos - trkpos)*(trkmom*(1/trkmom.Mag()));
    distance = (pos - (k*(trkmom*(1/trkmom.Mag())) + trkpos)).Mag();

  }else if(ir == 1){ // endcap

    if(TMath::Abs(pos.Z()) < calinZ-10.) return distance; // enough far away

    //heltrackp->OriginToCylinder(caloutR, TMath::Abs(pos.Z()), phi,
    //			phi1, 1000, startx, starty);

    ///////////////////////////////////////////////////////////////////
    //! def.
#if 1
    heltrackp->OriginToCylinder(caloutR, TMath::Abs(pos.Z()), phi,
				phi1, 1000, startx, starty);
#else
    //* default
    heltrackp->OriginToCylinder(calinR, TMath::Abs(pos.Z()), phi,
				phi1, 1000, startx, starty);
#endif
    ///////////////////////////////////////////////////////////////////

    JSF3DV tmppos = heltrackp->GetCoordinate(phi1);
    JSF3DV tmpmom = heltrackp->GetMomentum(phi1);

    TVector3 trkpos(tmppos.X(),tmppos.Y(),tmppos.Z());
    TVector3 trkmom(tmpmom.X(),tmpmom.Y(),tmpmom.Z());

    if( (trkmom.Z())*(pos.Z()) < 0. )  return distance;  // oposit side

    Double_t k = (pos - trkpos)*(trkmom*(1/trkmom.Mag()));
    distance = (pos - (k*(trkmom*(1/trkmom.Mag())) + trkpos)).Mag();

  }

  return distance;
  //---------------------------------------------------

}
#endif
//==========================================================================
//* GetInTrack -------------------------------------------------------------

std::vector<TVector3> S4CALSmallClusterUtils::GetInTrack(S4HYBTrack *track, Double_t tubeR, Double_t tubehalfH)
{

  //
  // the vector of return value is:
  // vector[0]: track intersection point at cylinder surface
  // vector[1]: track momentum which is calculated at intersection point
  //
  // If track cannot intersect with cylinder, return empty vector
  //

  std::vector<TVector3> posmom;
  TVector3 inposv = TVector3(0.,0.,0.);
  TVector3 inmomv = TVector3(0.,0.,0.);

  //if(track->IsLocked() || tubeR < 1.e-9 || tubehalfH < 1.e-9)  return posmom;
  if(!track->GetHelicalTrackPtr() || tubeR < 1.e-9 || tubehalfH < 1.e-9)  return posmom;

  if(tubeR < 0.)     tubeR     *= -1.;
  if(tubehalfH < 0.) tubehalfH *= -1.;

  JSFHelicalTrack   *heltrackp = track->GetHelicalTrackPtr();

  Double_t phi  = 0.;
  Double_t phi1 = 0.;
  Double_t startx = (heltrackp->GetCoordinate(0.)).X();
  Double_t starty = (heltrackp->GetCoordinate(0.)).Y();

  Int_t ir = heltrackp->OriginToCylinder(tubeR, tubehalfH, phi, phi1, 1000,
					 startx, starty);

  if(ir != 0 && ir != 1)   return posmom;

  JSF3DV tmppos = heltrackp->GetCoordinate(phi1);
  JSF3DV tmpmom = heltrackp->GetMomentum(phi1);

  inposv = TVector3(tmppos.X(),tmppos.Y(),tmppos.Z());
  inmomv = TVector3(tmpmom.X(),tmpmom.Y(),tmpmom.Z());

  posmom.push_back(inposv);
  posmom.push_back(inmomv);
  return posmom;

}

//==========================================================================
//* CalcEnergyDensity ------------------------------------------------------

Double_t S4CALSmallClusterUtils::CalcEnergyDensity(std::map<Int_t,S4CALHit*> hitmap, TVector3 pos, Double_t radius)
{

  std::map<Int_t,S4CALHit*>::iterator hitr;
  Double_t energy = 0.;

  for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++){
    //if(hitr->second->IsLocked()) continue;
    TVector3 hitpos = hitr->second->GetCellPosition();
    if( (hitpos-pos).Mag() < radius ){
      energy += hitr->second->GetEnergyDeposit();
    }
  }

  return 3.*energy/(4.*TMath::Pi()*radius*radius*radius);

}

//=====================================================================
//* SortHitMap --------------------------------------------------------

std::vector<S4CALHit*> S4CALSmallClusterUtils::SortHitMap(std::map<Int_t,S4CALHit*> &hitmap)
{

  std::vector<S4CALHit*> hitv;

  std::multimap<Double_t,S4CALHit*> tmpmap;
  std::multimap<Double_t,S4CALHit*>::iterator tmpitr;
  std::map<Int_t,S4CALHit*>::iterator hitr;


  for(hitr = hitmap.begin(); hitr != hitmap.end(); hitr++){
    Double_t dist = (hitr->second->GetCellPosition()).Mag();
    tmpmap.insert(std::pair<Double_t,S4CALHit*>(dist,hitr->second));
  }

  for(tmpitr = tmpmap.begin(); tmpitr != tmpmap.end(); tmpitr++){
    hitv.push_back(tmpitr->second);
  }

  return hitv;

}
