//*************************************************************************
//* ---------------------
//* S4CDCVectorHit Class
//* ---------------------
//* (Description)
//*	Class to store "S4CDCHitWire"s for a single cell.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	S4CDCVectorHit		: Array containing "S4CDCHitWire"s
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

#include <iostream>
#include <iomanip>
#include "S4CDCVectorHit.h"

ClassImp(S4CDCVectorHit)

//=====================================================================
//* constructor -------------------------------------------------------

S4CDCVectorHit::S4CDCVectorHit()
              : fKilled(kFALSE), fTrackNumber(0), fNExpectedCellHitsCut(8)
{
	SetOwner();
}

//=====================================================================
//* destructor --------------------------------------------------------

S4CDCVectorHit::~S4CDCVectorHit()
{
}

//=====================================================================
//* IsSortable --------------------------------------------------------

Bool_t S4CDCVectorHit::IsSortable() const
{
	return kTRUE;
}

//=====================================================================
//* Compare -----------------------------------------------------------

Int_t S4CDCVectorHit::Compare(const TObject *obj) const
{
	Int_t cell  = GetCellID();
	Int_t cello = ((S4CDCVectorHit *)obj)->GetCellID();
   	if (cell < cello) return -1;
   	else if (cell > cello) return 1;
   	else return 0;
}

//=====================================================================
//* Append ------------------------------------------------------------

Int_t S4CDCVectorHit::Append(S4CDCTrackHit *trackhit)
{	
	Int_t track  = trackhit->GetTrackID();
	Int_t cellid = trackhit->GetExactHitPtr()->GetCellID(); 
	
	// Return value = -1:	new trackhit.
	//		=  0:	successfully appended.
	//		=  1:	new cell.
	//		=  2:	new track.
	
	if (GetEntries() == 0) {
		fTrackNumber = track;
		SetCellID(cellid);
		trackhit->fVectorHitPtr = this;
                Add(trackhit);
                return 0;
	} else {
		if (track == fTrackNumber) {
			if (cellid == GetCellID()) {
				trackhit->fVectorHitPtr = this;
                        	Add(trackhit);
                        	return 0;
                        } else {
                        	return 1;
                        }
                } else {
                        return 2;
                }
        }
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4CDCVectorHit::DebugPrint(const Char_t *opt)
{
	std::cerr 	<< "(track,cell) = " 
		<< "(" << fTrackNumber << "," << GetCellID() << ")" 
		<< " NEntries = " << GetEntries() << std::endl;

	if (opt[1] == 'e') { 			// Detailed
		TIter next(this);
		S4CDCTrackHit *trackhit;
		Int_t n = 0;
		while ((trackhit = (S4CDCTrackHit *)next())) {
			std::cerr << std::setw(3) << ++n << " ";
			trackhit->DebugPrint(opt);
		}
	}
}

//=====================================================================
//* KillHits ----------------------------------------------------------

void S4CDCVectorHit::KillHits() 
{
	if (GetEntries() > fNExpectedCellHitsCut) {
		SetBit(kELooping);
	}
		
	fKilled = kTRUE;
}	
