//*************************************************************************
//* --------------------
//* S4TPCTrackMakerBuf
//* --------------------
//* (Description)
//*	This set of classes makes pad Tracks.
//* (Requires)
//* (Provides)
//* (Update Record)
//*	2004/08/28  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>
#include "S4TPCTrackMakerBuf.h"
#include "S4TPCExactHit.h"
#include "S4TPCTrack.h"
#include "S4TPCTrackHit.h"
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif
using namespace std;

//#define __DEBUG__

ClassImp(S4TPCTrackMakerBuf)

//=====================================================================
//* constructor -------------------------------------------------------
S4TPCTrackMakerBuf::S4TPCTrackMakerBuf(const Char_t    *name,
                                       const Char_t    *title, 
                                       S4TPCTrackMaker *module,
                                       Int_t            bufsize)
                  : U4TPCTrackMakerBuf(name, title, module, bufsize)
{
}
       
S4TPCTrackMakerBuf::S4TPCTrackMakerBuf(S4TPCTrackMaker *module,
                                       const Char_t    *name,
                                       const Char_t    *title, 
                                       Int_t            bufsize)
                  : U4TPCTrackMakerBuf(module, name, title, bufsize)
{
#ifdef __DEBUG__
   cerr << "constructor for S4TPCTrackMakerBuf is called..." << endl;
#endif
}

//=====================================================================
//* destructor --------------------------------------------------------

S4TPCTrackMakerBuf::~S4TPCTrackMakerBuf()
{
}

//=====================================================================
//* MakeTracks ----------------------------------------------------------

void S4TPCTrackMakerBuf::MakeTracks()
{
   if (!IsInitialized()) {
      cerr << "ERROR  S4TPCTrackMakerBuf::MakeTracks"    << endl
           << "...... pad hit data not yet initialized." << endl;
      abort();
   } else if (fDone) return;
   
   fDone = kTRUE;

   TObjArray *inputbufp = ((S4TPCHitMakerBuf *)GetInputDataBufPtr())->GetOutputBufPtr();
#ifdef __DEBUG__
   cerr << " S4TPCTrackMakerBuf::MakeTracks: NHits of inputbuf = " 
        << inputbufp->GetEntries() << endl;
#endif
   S4TPCHit::SetSortBy("TrackID");
   inputbufp->UnSort();
   inputbufp->Sort();


   TIter next(inputbufp);

   S4TPCHit *hitp;
   S4TPCTrack  *trackp = new S4TPCTrack(static_cast<S4TPCTrackMaker *>(Module()));
   while ((hitp = static_cast<S4TPCHit *>(next()))) {
      if (!trackp->Append(hitp)) {
         if (!trackp->Verify()) {
            delete trackp;
         } else {
            GetOutputBufPtr()->Add(trackp);
         }
         trackp = new S4TPCTrack(static_cast<S4TPCTrackMaker *>(Module()));
         trackp->Append(hitp);
       }
   }
   if (trackp->GetEntries()) {
      if (!trackp->Verify()) {
         delete trackp;
      } else {
         GetOutputBufPtr()->Add(trackp);
      }
   }   
}


