#ifndef EHEHSPRING_H
#define EHEHSPRING_H
//*****************************************************************************
//* =====================
//*  EHEHSpring
//* =====================
//*  
//* (Description)
//*    e+e- -> eH+ eH- generator
//*    Based on example/FFbarSpring directory.
//*
//* (Update Record)
//*    2010 06/26 E.Kato	Original version.
//*
//*****************************************************************************

#include "TNamed.h"
#include "JSFModule.h"
#include "JSFBases.h"
#include "JSFSpring.h"
#include "JSFBeamGeneration.h"

#include "HELLib.h"
#include "GENLib.h"

class ZHBoson;
class WHBoson;
class AHBoson;
class EHFermion;
//_______________________________________________________________________
// =====================
//  class EHEHBases
// =====================
//-----------------------------------------------------------------------
class EHEHBases : public JSFBases {
friend class EHEHSpringBuf;
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  EHEHBases(const char *name  = "EHEHBases", 
            const char *title = "EHEH Bases");
  virtual ~EHEHBases();

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetF         ()           const { return fF;          }
  Double_t GetKappaL    ()           const { return fKappaL;     }
  Double_t GetMass      ()           const { return fMass;       }
  Double_t GetMassZH    ()           const { return fMassZH;     }
  Double_t GetMassDM    ()           const { return fMassDM;     }
  Double_t GetMassHiggs ()           const { return fMassHiggs;  }
  Double_t GetEcmInit   ()           const { return fEcmInit;    }

  Double_t GetZBoost    ()           const { return fZBoost;     }
  Double_t GetEcmIP     ()           const { return fEcmIP;      }
  Double_t GetQ2XX      ()           const { return fQ2XX;       }
  Double_t GetQ2X1      ()           const { return fQ2X1;       }
  Double_t GetQ2X2      ()           const { return fQ2X2;       }
  Double_t GetQ2ZH1     ()           const { return fQ2ZH1;      }
  Double_t GetQ2ZH2     ()           const { return fQ2ZH2;      }

  Double_t GetCosTheta  ()           const { return fCosTheta;   }   //EH production angle costheta
  Double_t GetPhi       ()           const { return fPhi;        } //EH production angle phi

  Double_t GetCosThetaF1()           const { return fCosThetaF1; }  //EH decay angle costheta e- 
  Double_t GetCosThetaF2()           const { return fCosThetaF2; }  //EH decay angle costheta e+ 
  Double_t GetPhiF1     ()           const { return fPhiF1;      }
  Double_t GetPhiF2     ()           const { return fPhiF2;      }

  Double_t GetCosThetaH1()           const { return fCosThetaH1; }
  Double_t GetPhiH1     ()           const { return fPhiH1;      }
  Double_t GetCosThetaH2()           const { return fCosThetaH2; }
  Double_t GetPhiH2     ()           const { return fPhiH2;      }


  // ----------------------
  //   Base class methods
  // ----------------------
  virtual void     Userin();   // Bases user initialization
  virtual void     Userout();  // Bases user output 

  Double_t Func();     // Bases integration function.

  // ----------------------
  //   Utility methods
  // ----------------------
private:
  void      SelectHelicities(Double_t &weight);

  Double_t  DSigmaDX     (GENBranch &cmbranch);
  Double_t  AmpSquared   (GENBranch &cmbranch);
  Complex_t FullAmplitude();
  Complex_t AmpEEtoXX    (const HELFermion &em,
                          const HELFermion &ep,
                          const HELFermion &x1,
                          const HELFermion &x2);

private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Job parameters
  // ----------------
  Double_t fF;              // SU(5)->SO(5) breaking scale
  Double_t fKappaL;         // heavy lepton mass parameter
  Double_t fMass;           // m_x    : mass  of X
  Double_t fMassZH;         // m_zh   : mass  of ZH
  Double_t fMassDM;         // m_dm   : mass  of DM
  Double_t fMassHiggs;      // m_H   : mass  of Higgs
  Double_t fEcmInit;        // Initial Ecm

  Int_t    fISR;            // ISR on?
  Int_t    fIBType;         // Beam spread type (1: Uniform, 2: Gauss)
  Int_t    fBeamStr;        // Beamstrahlung on?
  Double_t fBeamWidth;      // Beam Spread relative to nominal Ebm
  Double_t fPole;           // electron polarization

  // ----------------
  //  Particle Data
  // ----------------
  EHFermion    *fXFermionPtr;     //! PD table entry of "eH"
  WHBoson      *fWHBosonPtr;      //! PD table entry of "WH"
  ZHBoson      *fZHBosonPtr;      //! PD table entry of "ZH"
  AHBoson      *fDMBosonPtr;      //! PD table entry of "DM"
  GENPDTWBoson *fWBosonPtr;       //! PD table entry of "W"
  GENPDTZBoson *fZBosonPtr;       //! PD table entry of "Z"
  GENPDTPhoton *fPhotonPtr;       //! PD table entry of "Gamma"
  GENPDTGluon  *fGluonPtr;        //! PD table entry of "Gluon"

  // ----------------
  //  Event info
  // ----------------
  Double_t       fZBoost;         // p_z(cm)      in lab frame
  Double_t       fEcmIP;          // Ecm after B-strahlung
  Double_t       fQ2XX;           // q^2 of XX system
  Double_t       fQ2X1;           // q^2 of final state X1
  Double_t       fQ2X2;           // q^2 of final state X2
  Double_t       fQ2ZH1;          // q^2 of final state ZH1
  Double_t       fQ2ZH2;          // q^2 of final state ZH2

  Int_t          fHelInitial[2];  // initial state helicities
  Int_t          fHelFinal  [6];  // final   state helicities
  Int_t          fJCombI;         // intial helicity combination
  Int_t          fJCombF;         // final  helicity combination
  ANL4DVector    fK[2];           // [0,1] = [e-, e+]
  ANL4DVector    fP[6];           // [0,1,2,3,4,5] = [e-, ah1, h1, e+, ah2 ,h2]
  Double_t       fM[6];           // [0,1,2,3,4,5] = [me, mdm, mh, me, mdm ,mh]

  // ----------------
  //  Integ. vars
  // ----------------
  Double_t       fHelCombInitial; // initial state helicity combination
  Double_t       fHelCombFinal;   // final   state helicity combination
  Double_t       fCosTheta;       // cos(theta_x1) in cm frame
  Double_t       fPhi;            // phi_x1        in cm frame
  Double_t       fXQ2X1;          // q^2 of final state X1
  Double_t       fXQ2X2;          // q^2 of final state X2
  Double_t       fXQ2ZH1;         // q^2 of final state ZH1
  Double_t       fXQ2ZH2;         // q^2 of final state ZH2

  Double_t       fCosThetaF1;     //cos(theta_f1) in X1 frame
  Double_t       fPhiF1;          //Phif1 in X1 frame
  Double_t       fCosThetaF2;     //cos(theta_f2) in X2 frame
  Double_t       fPhiF2;          //Phif2 in X2 frame
  Double_t       fCosThetaH1;     // cos(theta_H1) in ZH1 frame
  Double_t       fPhiH1;          // phi_H1        in ZH1 frame
  Double_t       fCosThetaH2;     // cos(theta_H2) in ZH2 frame
  Double_t       fPhiH2;          // phi_H2        in ZH2 frame

  Double_t       fR_BW_m;         //! naturl Ebeam spread for e-
  Double_t       fR_BW_p;         //!                     for e+
  Double_t       fR_BS_m;         //! beamstrahlung       for e-
  Double_t       fR_BS_p;         //!                     for e+

  Double_t       fR_ISR_var;      //! initial state radiation
  Double_t       fR_ISR_side;     //! from which side ? (e-/e+) 

  Double_t       fXU[10];          //! [0,1,2,3,4,5] = (cosx, phix,
  Double_t       fXL[10];          //!    cosh1, phih1, cosh2, phih2)

  TFile   *fBeamFile;         //! Beamstrahlung data file
  JSFBeamGenerationCain *fBM; //! Beamstrahlung data

  ClassDef(EHEHBases, 1) // Bases for e+e- -> XX process
};

class EHEHSpring;

//_______________________________________________________________________
// =====================
//  class EHEHSpringBuf
// =====================
//-----------------------------------------------------------------------
class EHEHSpringBuf : public JSFSpringBuf {
public:
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
  EHEHSpringBuf(const char   *name   = "EHEHSpringBuf", 
                const char   *title  = "EHEH Spring test event buffer",
	        EHEHSpring   *spring = 0)
	     : JSFSpringBuf(name,title,(JSFSpring*)spring) {} 
  virtual ~EHEHSpringBuf() {}

  // ----------------------
  //  Getters and Setters
  // ----------------------
  Double_t GetQ2XX       ()           const { return fQ2XX;        }
  Double_t GetCosTheta   ()           const { return fCosTheta;    }
  Double_t GetPhi        ()           const { return fPhi;         }
  Double_t GetQ2X1       ()           const { return fQ2X1;        }
  Double_t GetCosThetaF1 ()           const { return fCosThetaF1;  }
  Double_t GetPhiF1      ()           const { return fPhiF1;       }
  Double_t GetQ2X2       ()           const { return fQ2X2;        }
  Double_t GetCosThetaF2 ()           const { return fCosThetaF2;  }
  Double_t GetPhiF2      ()           const { return fPhiF2;       }
  Double_t GetQ2ZH1      ()           const { return fQ2ZH1;       }
  Double_t GetCosThetaH1 ()           const { return fCosThetaH1;  }
  Double_t GetPhiH1      ()           const { return fPhiH1;       }
  Double_t GetQ2ZH2      ()           const { return fQ2ZH2;       }
  Double_t GetCosThetaH2 ()           const { return fCosThetaH2;  }
  Double_t GetPhiH2      ()           const { return fPhiH2;       }
  Double_t GetZBoost     ()           const { return fZBoost;      }
  Double_t GetEcmIP      ()           const { return fEcmIP;       }

  // ----------------------
  //   Base class methods   
  // ----------------------
  virtual Bool_t SetPartons();

private:
  // --------------------------------------------------------------------
  //  Data Members
  // --------------------------------------------------------------------
  // ----------------
  //  Event info
  // ----------------

  Double_t fQ2XX;            // q^2 of XX system
  Double_t fCosTheta;        // cos(theta_x1) in cm  frame
  Double_t fPhi;             // phi_x1        in cm  frame
  Double_t fQ2X1;            // q^2 of final state X1
  Double_t fCosThetaF1;      // cos(theta_F1) in X1 frame
  Double_t fPhiF1;           // phi_F1        in X1 frame
  Double_t fQ2X2;            // q^2 of final state X2
  Double_t fCosThetaF2;      // cos(theta_F2) in X2 frame
  Double_t fPhiF2;           // phi_F2        in X2 frame 
  Double_t fQ2ZH1;           // q^2 of final state ZH1
  Double_t fCosThetaH1;      // cos(theta_H1) in ZH1 frame
  Double_t fPhiH1;           // phi_H1        in ZH1 frame
  Double_t fQ2ZH2;           // q^2 of final state ZH2
  Double_t fCosThetaH2;      // cos(theta_H2) in ZH2 frame
  Double_t fPhiH2;           // phi_H2        in ZH2 frame
  Double_t fZBoost;          // p_z(cm)       in lab frame
  Double_t fEcmIP;           // Ecm after B-strahlung

  ClassDef(EHEHSpringBuf, 1)  // EHEHSpring event buffer
};

//_______________________________________________________________________
// =====================
//  class EHEHSpring
// =====================
//-----------------------------------------------------------------------
class EHEHSpring : public JSFSpring {
  // --------------------------------------------------------------------
  //  Member Functions
  // --------------------------------------------------------------------
  // ----------------------
  //  C-tor and D-tor
  // ----------------------
public:
   EHEHSpring(const char   *name  = "EHEHSpring", 
              const char   *title = "EHEH Spring test",
              EHEHBases    *bases = 0);
   virtual ~EHEHSpring();

   virtual Bool_t Initialize(); // overwrite the base class method

   ClassDef(EHEHSpring, 1)  // EHEHSpring class
};

//_______________________________________________________________________
// =====================
//  class ZHBoson
// =====================
//-----------------------------------------------------------------------

class ZHBoson: public GENPDTEntry {
public:
   ZHBoson(Double_t fF = 580.,
           Double_t mh = 134.);
   virtual ~ZHBoson() {}

   Double_t  GetF     () const { return fF;      }
   Double_t  GetSh    () const { return fSh;     }
   Double_t  GetCh    () const { return fCh;     }
   Double_t  GetSf    () const { return fSf;     }
   Double_t  GetCf    () const { return fCf;     }
   Double_t  GetMassDM() const { return fMassDM; }

private:
   void     Initialize();
   Double_t GamToSV(Double_t m1, Double_t m2, Double_t a);

private:
   Double_t fF;         // SU(5)->SO(5) breaking scale
   Double_t fSh;        // Sh parameter
   Double_t fCh;        // Ch parameter
   Double_t fSf;        // Sf parameter
   Double_t fCf;        // Cf parameter
   Double_t fMassDM;    // daughter DM mass
   Double_t fMassHiggs; // Higgs mass

   ClassDef(ZHBoson, 1)  // X boson class
};

//_______________________________________________________________________
// =====================
//  class AHBoson
// =====================
//-----------------------------------------------------------------------

class AHBoson: public GENPDTEntry {
public:
   AHBoson(Double_t m      = 81.9);
   virtual ~AHBoson() {}

   ClassDef(AHBoson, 1)  // X boson class
};


//_______________________________________________________________________
// =====================
//  class WHBoson
// =====================
//-----------------------------------------------------------------------

class WHBoson: public GENPDTEntry {
public:
   WHBoson(Double_t fF = 580.);
   virtual ~WHBoson() {}

   Double_t  GetF     () const { return fF;      }
   Double_t  GetSh    () const { return fSh;     }
   Double_t  GetMassDM() const { return fMassDM; }

private:
   void     Initialize();
   Double_t GamToVV(Double_t m1, Double_t m2, Double_t a);

private:
   Double_t fF;      // SU(5)->SO(5) breaking scale
   Double_t fSh;     // sh parameter
   Double_t fMassDM; // daughter DM mass

   ClassDef(WHBoson, 1)  // X boson class
};

//_______________________________________________________________________
// =====================
//  class EHFermion
// =====================
//-----------------------------------------------------------------------

class EHFermion: public GENPDTEntry {
public:
   EHFermion(Double_t fF = 580., Double_t fKappaL = 0.5, Double_t mh = 134.);
   virtual ~EHFermion() {}

   Double_t  GetF     () const { return fF;      }
   Double_t  GetKappa () const { return fKappa;  }
   Double_t  GetMassDM() const { return fMassDM; }

private:
   void     Initialize();
   Double_t GamToFV(Double_t m1, Double_t m2, Double_t gl,Double_t gr);  //added

private:
   Double_t fSh;        // mixing angle sin
   Double_t fCh;        // mixing angle cos
   Double_t fF;         // SU(5)->SO(5) breaking scale
   Double_t fKappa;     // kappa l parameter
   Double_t fMassDM;    // daughter DM mass
   Double_t fMassHiggs; // Higgs mass

   ClassDef(EHFermion, 1)  // X boson class
};
#endif
