// This macro gets in memory an histogram from a root file
// and fits a user defined function.
// Note that a user defined function must always be defined
// as in this example:
//  - first parameter: array of variables (in this example only 1-dimension)
//  - second parameter: array of parameters
// Note also that in case of user defined functions, one must set
// an initial value for each parameter.

Double_t fitf(Double_t *x, Double_t *par)
{
   Double_t fitval = par[0]*TMath::Exp(-par[1]*(*x))
   			+ par[2]*(*x) + par[3];
   return fitval;
}

void Fit(TH1D *h)
{
   Axis_t xmin = h->GetXaxis()->GetXmin();
   Axis_t xmax = h->GetXaxis()->GetXmax();
   xmin = 0.00;
   // xmax = 3.75;
   xmax = 4.5;
   
// Creates a Root function based on function fitf above
   TF1 *func = new TF1("fitf",fitf,xmin,xmax,4);

// Sets initial values and parameter names
   func->SetParameters(0.0002,2.0,0.00005,0.0);
   func->SetParNames("Exp_mag","Exp_slope","Diffusion","Constant");

// Fit histogram in range defined by function
   h->Fit("fitf","r");

// Gets integral of function between fit limits
   cout << "xmin = " << xmin << " xmax = " << xmax << endl;
   Float_t integ = func->Integral(xmin,xmax);
   cout << "Integral of function = " << integ << endl;
   cout << "Average = " << (sqrt(integ/(xmax-xmin))) << endl;
}
