#ifndef __U4CDCCLUSTER__
#define __U4CDCCLUSTER__

//************************************************************************
//* -------------------------------
//* Cluster Datum
//* -------------------------------
//************************************************************************

#include <iostream>
#include <iomanip>
#include "TClass.h"
#include "TObjArray.h"

#include "JSFSteer.h"
#include "JSFModule.h"
#include "U4CDCRawData.h"
#include "U4CDCFADCCalibration.h"

class U4CDCCluster : public TObjArray {
public:
  U4CDCCluster(U4CDCFADCCalibration *calib = 0, Bool_t varped = kTRUE);
  U4CDCCluster(U4CDCCluster &cl);
  virtual ~U4CDCCluster();

  void  SetNpeds(Int_t n);

  inline Int_t  GetStationNo()     const  { return fStationNo;     }
  inline Int_t  GetChannelNo()     const  { return fChannelNo;     }
  inline Int_t  GetStartTimeSlot() const  { return fStartTimeSlot; }
  inline Int_t  GetLastTimeSlot()  const  { return fLastTimeSlot;  }
  Double_t      GetSum()        { if (!fDone) Update(); return fSum;       }
  Double_t      GetPeak()       { if (!fDone) Update(); return fPeak;      }
  Double_t      GetVarPedestal(){ if (!fDone) Update(); return fPedestal;  }
  Double_t      GetStartTime()  { if (!fDone) Update(); return fStartTime; }
  Double_t      GetWidth()      { if (!fDone) Update(); return fWidth;     }
  Double_t      GetPedestal();
  
  static void   SetNpres      (Int_t    n) { fNpres  = n; }
  static void   SetNposts     (Int_t    n) { fNposts = n; }
  static void   SetMinNslots  (Int_t    n) { fMinNslots   = n; }
  static void   SetMaxNslots  (Int_t    n) { fMaxNslots   = n; }
  static void   SetSoftThresh (Double_t x) { fSoftThresh  = x; }
  static void   SetSplitThresh(Double_t x) { fSplitThresh = x; }
  static void   SetIntegrateNslots (Int_t n) {fIntegrateNslots = n; }

  void          UseVarPed();
  void          UseConstPed();
  Bool_t        Append(U4CDCRawADCDatum *adc);
  void          Integrate();
  void          SplitInto(TObjArray &list);

  Bool_t        IsSortable() const;
  Int_t         Compare(const TObject *obj) const;
  static void   SetSortBy(Char_t *opt = "Time");

  void          DebugPrint(const Char_t *opt = "Brief");

private:
  void          Update();

private:
  Bool_t        fDone;                  // (1,0) = (cluster done, not yet)
  Bool_t        fUseVarPed;             // (1,0) = (var ped, const ped)
  Int_t         fStationNo;             // Station number
  Int_t         fChannelNo;             // Channel number
  Int_t         fStartTimeSlot;         // start time slot
  Int_t         fLastTimeSlot;          // last  time slot
  Double_t      fSum;                   // ADC sum
  Double_t      fPeak;                  // ADC peak
  Double_t      fPedestal;              // ADC pedestal
  Double_t      fStartTime;             // start time (ns)
  Double_t      fWidth;                 // width (ns)
  Int_t         fNpeds;                 // No. slots for pedestal calc.
  static Int_t  fNpres;                 // No. pre-slots
  static Int_t  fNposts;                // No. post-slots
  static Int_t  fSortBy;                // (0,1,2) = by (Time, Peak, Charge)
  static Int_t  fMinNslots;             // Min No. of time slots for splitting 
  static Int_t  fMaxNslots;             // Max No. of time slots for PH sum
  static Int_t  fIntegrateNslots;       // No. of slots to integrate 
  static Double_t fSoftThresh;          // Software threshold 
  static Double_t fSplitThresh;         // Threshold for peak splitting
 
  U4CDCFADCCalibration *fCalib;           // pointer to calibration data object
  U4CDCCluster         *fParent;          // pointer to parent

  ClassDef(U4CDCCluster, 1)       // U4CDC Cluster
};

#endif
