//**************************************************************************
//* ------------------------------
//* Cluster Maker
//* ------------------------------
//**************************************************************************

#include "U4CDCCluster.h"
#include "U4CDCClusterBuf.h"
#include "U4CDCClusterMaker.h"
#include "U4CDCFADCCalibration.h"
#include <fstream>

//--------------------------------------------------------------------------

ClassImp(U4CDCClusterMaker)

U4CDCClusterMaker::U4CDCClusterMaker(U4CDCFADCCalibration *calib,
                                 const char *name, const char *title)
       : JSFModule(name,title),
         fToDeleteCalib(calib ? kFALSE : kTRUE),
         fCalib(calib) {}

U4CDCClusterMaker::~U4CDCClusterMaker()
{
        if (fToDeleteCalib && fCalib) delete fCalib;
}

void U4CDCClusterMaker::SetCalibrationData(U4CDCFADCCalibration *calib)
{
        fCalib = calib;
        fToDeleteCalib = kFALSE;
        ((U4CDCClusterBuf *)EventBuf())->SetCalibrationData(calib);
}

void U4CDCClusterMaker::UseVarPed()
{
        fUseVarPed = kTRUE;
        if (EventBuf()) ((U4CDCClusterBuf *)EventBuf())->UseVarPed();
}

void U4CDCClusterMaker::UseConstPed()
{
        fUseVarPed = kFALSE;
        if (EventBuf()) ((U4CDCClusterBuf *)EventBuf())->UseConstPed();
}

Bool_t U4CDCClusterMaker::Initialize()
{
        if (!fCalib) {
                fCalib = new U4CDCFADCCalibration();
                fToDeleteCalib = kTRUE;
        }
        fEventBuf = new U4CDCClusterBuf(this,fCalib);
        if (fUseVarPed) ((U4CDCClusterBuf *)fEventBuf)->UseVarPed();
        else            ((U4CDCClusterBuf *)fEventBuf)->UseConstPed();

        return kTRUE;
}

Bool_t U4CDCClusterMaker::BeginRun(Int_t run)
{
        fCalib->Update();

        return kTRUE;
}

Bool_t U4CDCClusterMaker::Process(Int_t ev)
{
        U4CDCRawData    *raw    = (U4CDCRawData*)gJSF->FindModule("U4CDCRawData");
        U4CDCRawDataBuf *evt    = (U4CDCRawDataBuf *)raw->EventBuf();
        Int_t         nadc    = evt->GetNADCData();
        TClonesArray *adcdata = evt->GetADCData();

        U4CDCClusterBuf *clbufp = (U4CDCClusterBuf *)EventBuf();
        clbufp->SetADCData(nadc,adcdata);
        clbufp->FindClusters();

        return kTRUE;
}
