//****************************************************************************
//* ---------------------
//* FADC Calibration Data
//* ---------------------
//****************************************************************************

#include "U4CDCCluster.h"
#include "U4CDCClusterBuf.h"
#include "U4CDCClusterMaker.h"
#include "U4CDCFADCCalibration.h"
#include <fstream>

using namespace std;

//----------------------------------------------------------------------------

ClassImp(U4CDCFADCCalibration)

U4CDCFADCCalibration::U4CDCFADCCalibration(Double_t *ped, Double_t *gain,
                                       Double_t *tped, Double_t *tscale, Int_t last)
{
        CalibDatum *datum = new CalibDatum(last,ped,gain,tped,tscale);
        Add(datum);
        fCurr = datum;
}

U4CDCFADCCalibration::U4CDCFADCCalibration(Char_t * calibfile)
{
        std::cout << " Going to open a calib file " << calibfile << std::endl;
        ifstream in;
        in.open(calibfile,std::ios::in);
        std::cout << " Opened " << calibfile << std::endl;
        Int_t last = 99999999;
        Double_t *ped    = new Double_t [U4CDCRawADCDatum::GetNchMax()];
        Double_t *gain   = new Double_t [U4CDCRawADCDatum::GetNchMax()];
        Double_t *tped   = new Double_t [U4CDCRawADCDatum::GetNchMax()];
        Double_t *tscale = new Double_t [U4CDCRawADCDatum::GetNchMax()];
        while ((in >> last
                   >> ped   [0] >> ped   [1] >> ped   [2]  >> ped   [3]
                   >> ped   [4] >> ped   [5] >> ped   [6]  >> ped   [7]
                   >> ped   [8] >> ped   [9] >> ped   [10] >> ped   [11]) &&
               (in >> gain  [0] >> gain  [1] >> gain  [2]  >> gain  [3]
                   >> gain  [4] >> gain  [5] >> gain  [6]  >> gain  [7]
                   >> gain  [8] >> gain  [9] >> gain  [10] >> gain  [11]) &&
               (in >> tped  [0] >> tped  [1] >> tped  [2]  >> tped  [3]
                   >> tped  [4] >> tped  [5] >> tped  [6]  >> tped  [7]
                   >> tped  [8] >> tped  [9] >> tped  [10] >> tped  [11]) &&
               (in >> tscale[0] >> tscale[1] >> tscale[2]  >> tscale[3]
                   >> tscale[4] >> tscale[5] >> tscale[6]  >> tscale[7]
                   >> tscale[8] >> tscale[9] >> tscale[10] >> tscale[11])) {
                CalibDatum *datum = new CalibDatum(last,ped,gain,tped,tscale);
                Add(datum);
        }
        in.close();
        delete [] ped;
        delete [] gain;
        delete [] tped;
        delete [] tscale;
}

U4CDCFADCCalibration::~U4CDCFADCCalibration() { Delete(); }

Double_t U4CDCFADCCalibration::GetPedestal    (Int_t n, Int_t a)
                { return fCurr->GetPedestal(n,a);    }
Double_t U4CDCFADCCalibration::GetGain        (Int_t n, Int_t a)
                { return fCurr->GetGain(n,a);        }
Double_t U4CDCFADCCalibration::GetTimePedestal(Int_t n, Int_t a)
                { return fCurr->GetTimePedestal(n,a);}
Double_t U4CDCFADCCalibration::GetTimeScale   (Int_t n, Int_t a)
                { return fCurr->GetTimeScale(n,a);   }

void U4CDCFADCCalibration::Update()
{
        Int_t run = gJSF->GetRunNumber();
        TIter next(this);
        CalibDatum *datum;
        while ((datum = (CalibDatum *)next())) {
                fCurr = datum;
                if (run <= fCurr->GetLastRun()) break;
        }
}
