#ifndef __U4CDCHITWire__
#define __U4CDCHITWire__

//*************************************************************************
//* ---------------------
//* U4CDCHitWire Class
//* ---------------------
//* (Description)
//*	Class to store "U4CDCHit"s for a single wire.
//* (Requires)
//*	TObjArray
//* (Provides)
//*	U4CDCHitWire		: Array containing "U4CDCHit"s
//* (Update Record)
//*	1999/12/04  K.Fujii	Original version.
//*************************************************************************

class U4CDCHitCell;
class U4CDCHit;
class U4CDCTimeToXRelation;

#include "TVector3.h"
#include "U4CDCHit.h"
#include "U4CDCHitCell.h"
#include "U4CDCCluster.h"
#include "U4CDCRawData.h"

#define __CDCNWIRESPERCELL__  5
#define __CDCCELLIDORDER__    10000

class U4CDCHitWire : public TObjArray {
friend class U4CDCHit;
friend class U4CDCHitCell;

public:

  U4CDCHitWire(U4CDCTimeToXRelation *calib = 0);
  virtual ~U4CDCHitWire();

  inline virtual U4CDCHitCell *GetCellPtr() const { return fCellPtr;    }

  inline virtual Int_t	  GetLayerNumber() const { return fLayerNumber; }
  inline virtual Int_t	  GetCellNumber () const { return fCellNumber;  }
  inline virtual Int_t	  GetWireNumber () const { return fWireNumber;  }
  inline virtual Int_t	  GetCellID     () const 
  			   { return fCellNumber + fLayerNumber*__CDCCELLIDORDER__;}
  inline virtual Int_t	  GetDriftRegionNumber () const 
  			   { return fWireNumber + fLayerNumber*__CDCNWIRESPERCELL__;}
  			   
  inline virtual Int_t 	  WhatAmI() 
    { return ( fLayerNumber%3 == 0 ? 0 : (fLayerNumber%3 == 1 ? -1 : 1)) ; }

  inline virtual TVector3 GetWireTop    () const { return fWireTop   ; }
  inline virtual TVector3 GetWireBtm    () const { return fWireBtm   ; }
  inline virtual Double_t GetTan2Stereo () const { return fTan2Stereo; }
  inline virtual Double_t GetRwaist     () const { return fRwaist    ; }
  inline virtual Double_t GetRwaist2    () const { return fRwaist*fRwaist; }

  inline virtual Double_t X() const { return fXwire; }
  inline virtual Double_t Y() const { return fYwire; }

  virtual Int_t		Append(U4CDCHit *hit);

  virtual Bool_t	IsSortable() const;
  virtual Int_t		Compare(const TObject *obj) const;

  virtual void 		DebugPrint(const Char_t *opt = "Brief");

  static void ChannelToWire(Int_t dch,
  			    Int_t &l, Int_t &c, Int_t &w, 
  			    Double_t &x, Double_t &y);

private:
  static void ChannelToWire(U4CDCCluster *cluster,
  			    Int_t &l, Int_t &c, Int_t &w, 
  			    Double_t &x, Double_t &y);

protected:
  Int_t		fLayerNumber;		// layer Number.
  Int_t		fCellNumber;		// cell Number.
  Int_t		fWireNumber;		// wire Number.
  TVector3      fWireTop;               // top end point of wire
  TVector3      fWireBtm;               // bottom end point of wire
  Double_t	fTan2Stereo;		// tan^2(stereo angle)
  Double_t	fRwaist;		// waist radius of sense wire surface
  Double_t	fXwire;			// wire X position (cm)
  Double_t	fYwire;			// wire Y position (cm)
  U4CDCHitCell *fCellPtr;		// pointer to cell this belongs to

  U4CDCTimeToXRelation  *fCalibPtr;	//! pointer to Hit calibration data
 
  ClassDef(U4CDCHitWire, 1) 	// CDC Hit Wire
};

#endif
