#ifndef __U4CDCTRACKMAKERBUF__
#define __U4CDCTRACKMAKERBUF__

//*************************************************************************
//* ---------------------
//* U4CDCTrackMakerBuf Class
//* ---------------------
//* (Description)
//*
//* (Requires)
//*	JSFEventBuf
//* (Provides)
//*	U4CDCTrackMakerBuf		: buffering class
//* (Update Record)
//*	1999/12/04  R.Kuboshima	    Original version.
//*************************************************************************

#include "JSFSteer.h"
#include "U4VTrackMakerBuf.h"
#include "U4CDCTrackMaker.h"
#include "U4CDCHitCell.h"
#include "U4CDCHitMakerBuf.h"

class U4CDCTrack;

class U4CDCTrackMakerBuf : public U4VTrackMakerBuf {
friend class U4CDCTrackMaker;

public:

  U4CDCTrackMakerBuf(const Char_t          *name    = "U4CDCTrackMakerBuf",
                     const Char_t          *title   = "CDC Track data",
		           U4CDCTrackMaker *module  = 0,
                           Int_t            bufsize = 2000);
  U4CDCTrackMakerBuf(      U4CDCTrackMaker *module,
                     const Char_t          *name    = "U4CDCTrackMakerBuf",
                     const Char_t          *title   = "CDC Track data",
                           Int_t            bufsize = 2000);
		    
  virtual ~U4CDCTrackMakerBuf();
  
  virtual void 	DebugPrint(const Char_t *opt = "Brief");
  
  inline virtual void   SetChi2Cut(Double_t x)     { xChi2      = x; }
  inline virtual void   SetNsigCut(Double_t x)     { xNsigmas   = x; }
  inline virtual void   SetDXpivotCut(Double_t x)  { xDXpivot   = x; }
  inline virtual void   SetKCNsigCut(Double_t x)   { xKCNsigmas = x; }
  inline virtual void   SetSlopeCut(Double_t x)    { xSlope     = x; }
  inline virtual void   SetInterceptMin(Double_t x){ fBmin      = x; }
  inline virtual void   SetInterceptMax(Double_t x){ fBmax      = x; }
  inline virtual void   SetInterceptNbins(Int_t  n){ fNbins     = n; }

  inline virtual U4CDCHitMakerBuf *GetHitMakerBufPtr()
  {
      U4CDCHitMaker *hm = (U4CDCHitMaker*)gJSF->FindModule("U4CDCHitMaker");
      return (U4CDCHitMakerBuf *)hm->EventBuf();
  }

protected:  
  virtual void	MakeTracks() ;	// Make Baby Tracks
  void MakeVectorHits(U4CDCHitCell *celp, TObjArray *truevhittmpp);
  
protected:  
  Double_t            xNsigmas;         //  window for residual
  Double_t            xChi2;            //  chi^2 cut on a vector hit
  Double_t            xDXpivot;         //  dx cut for pivoits
  Double_t            xKCNsigmas;       //  window for hit poisoning
  Double_t            xSlope;           //  slope cut
  Double_t            fBmin;            //  minimum intercept
  Double_t            fBmax;            //  maximum intercept
  Int_t               fNbins;           //  parameter of depend on efficiency
  Int_t               fNvhitCut;        //  cut on No. of vector hits per track
  TObjArray           fJunkVHit;        //  array of unused vector hits

  ClassDef(U4CDCTrackMakerBuf, 1) // Cluster data buffer
};

#endif
