//*************************************************************************
//* --------------------
//* U4CDCRawADCDatum Class
//* --------------------
//* (Description)
//* 	Classes for a single FADC datum.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4CDCRawADCDatum
//* (Update Record)
//*	1999/06/02  K.Fujii	Original version.
//*
//*************************************************************************


#include "JSFSteer.h"
#include "U4CDCRawADCDatum.h"
#include <iostream.h>

ClassImp(U4CDCRawADCDatum)

// static variables are not declared as constant so as to allow
// different formats in future.

Int_t U4CDCRawADCDatum::gFormatID  =  0;	// Format ID
Int_t U4CDCRawADCDatum::gNstabits  =  0;	// No. station number bits
Int_t U4CDCRawADCDatum::gNchabits  =  4;	// No. channel number bits
Int_t U4CDCRawADCDatum::gNswperdt  =  2;	// No. short words per FADC datum
Int_t U4CDCRawADCDatum::gNchMax    = 12;	// Maximum FADC channel No

//___________________________________________________________________________
// ------------
// Constructors
// ------------
//
U4CDCRawADCDatum::U4CDCRawADCDatum() 
	: fStationNo(0), fChannelNo(0), fADC(0), fTime(0) {}
//___________________________________________________________________________
U4CDCRawADCDatum::U4CDCRawADCDatum(U4CDCRawADCDatum & rad) 
{
    fStationNo 	= rad.fStationNo;
    fChannelNo 	= rad.fChannelNo;
    fADC	= rad.fADC;
    fTime	= rad.fTime;
}

//___________________________________________________________________________
U4CDCRawADCDatum::U4CDCRawADCDatum(ADC_t adcword[])
{
    Unpack(adcword);
}

//___________________________________________________________________________
// ------------
// Destructors
// ------------
//
U4CDCRawADCDatum::~U4CDCRawADCDatum() {}

//___________________________________________________________________________
// -------
// Setters
// -------
//
// This methods should be modified according to your raw data format.
//
void U4CDCRawADCDatum::SetFormat(U4CDCUniversalRecordHeader & urh)
{ 
    gFormatID = urh.GetFormatID();
    // set gNstabits, etc., according to the format ID.
    switch (gFormatID) {
    	case 0: {
            gNstabits =  0;
            gNchabits =  4;
            gNchMax   = 12;
            break;
        }
        case 1: 
        case 2: {
            gNstabits =  5;
            gNchabits =  4;
            gNchMax   = 12;
            break;
        }
        default: { 
            std::cerr << "Format ID = " << gFormatID
                 << " is not supported" << std::endl;
        }
    }
}

//___________________________________________________________________________
// ---------------
// Service Methods
// ---------------
//
// This methods should be modified according to your raw data format.
//
void U4CDCRawADCDatum::Unpack(ADC_t adcword[])
{
    switch (gFormatID) {
    	case 0: {
#ifndef __LITTLE_ENDIAN__
           adcword[0] = (adcword[0] << 8) | (adcword[0] >> 8);
           adcword[1] = (adcword[1] << 8) | (adcword[1] >> 8);
#endif
  	   fStationNo = 0;
  	   fChannelNo = (adcword[0] >> (sizeof(ADC_t)*8 - gNchabits)) - 1;
  	   fTime      = adcword[0] & ( 0xffff >> gNchabits );
  	   fADC	      = adcword[1];

	   break;
  	}
  	case 1:
  	case 2: {
#ifndef __LITTLE_ENDIAN__
           adcword[0] = (adcword[0] << 8) | (adcword[0] >> 8);
           adcword[1] = (adcword[1] << 8) | (adcword[1] >> 8);
#endif
  	   fStationNo = adcword[0] >> (sizeof(ADC_t)*8 - gNstabits);
  	   fChannelNo = adcword[1] >> (sizeof(ADC_t)*8 - gNchabits);
  	   fADC	      = adcword[0] & ( 0xffff >> gNstabits );
  	   fTime      = adcword[1] & ( 0xffff >> gNchabits );

	   break;
  	}
        default: { 
            std::cerr << "Format ID = " << gFormatID
                 << " is not supported" << std::endl;
        }
    }
}

//___________________________________________________________________________
void U4CDCRawADCDatum::DebugPrint(const Char_t *)
{
    std::cerr << " n " << fStationNo
         << " a " << fChannelNo
         << " adc " << fADC
         << " tim " << fTime << std::endl;
}
