#ifndef __U4VTRACKHIT__
#define __U4VTRACKHIT__
//*************************************************************************
//* =================
//* U4VTrackHit Class
//* =================
//* (Description)
//*    Class for a single track hit.
//* (Requires)
//*    TObjArray
//* (Provides)
//*    U4VTrackHit : 
//* (Update Record)
//*	2004/09/05  K.Fujii    Original version.
//*************************************************************************

#include "TAttLockable.h"
#include "TObject.h"

class U4VTrackHit : public TObject, public TAttLockable {
friend class U4VTrack;
public:

  U4VTrackHit(TObject *ht = 0);
  U4VTrackHit(const U4VTrackHit &hit);
  virtual ~U4VTrackHit();

  static  void    SetSortBy(Char_t *opt = "Layer");
  static  void    SetSortBy(Int_t   key);
  virtual Bool_t  IsSortable()                 const;
  virtual Int_t   Compare(const TObject *obj)  const;
  virtual Int_t   GetSortBy()                  const { return fSortBy; }
       
  inline virtual Double_t  GetResidual() const { return fResidual;     }
  inline virtual Double_t  GetT0      () const { return fT0;           }
  inline virtual TObject  *GetHitPtr  ()       { return fHitPtr;       }

  inline virtual void  SetResidual(Double_t r)     { fResidual  = r;   }
  inline virtual void  SetT0      (Double_t t)     { fResidual  = t;   }

         virtual void DebugPrint(const Char_t *opt = "");

private:
  Double_t       fResidual;	// residual = hit - track (cm)
  Double_t       fT0;           // T0 (ns)
  TObject       *fHitPtr;	// pointer to CDC hit

  static Int_t   fSortBy;       // (0,1) = (chi2,t0)

  ClassDef(U4VTrackHit, 1) 	  // Base class for track hits
};


#endif
