//*************************************************************************
//* ===================
//*  U4TPCKalDetector Class
//* ===================
//*
//* (Description)
//* (Requires)
//* (Provides)
//* (Update Recored)
//*   2004/12/16  A.Yamaguchi       Original version.
//*
//*************************************************************************

#include <iostream>

#include "JSFSteer.h"
#include "U4TPCKalDetector.h"
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif

ClassImp(U4TPCKalDetector)

//=====================================================================
//* constructor -------------------------------------------------------

U4TPCKalDetector::U4TPCKalDetector(Int_t n)
                : U4VKalDetector(n)
{
   SetOwner();
}

U4TPCKalDetector::U4TPCKalDetector(const TString &flag, Int_t n)
                : U4VKalDetector(n)
{
   if (flag != "FillLayer") {
      std::cerr << "<<< ERROR!! >>> flag should be \"FillLayer\"" << std::endl;
      abort();
   }
   Initialize();
   SetOwner();
}

//=====================================================================
//* destructor --------------------------------------------------------

U4TPCKalDetector::~U4TPCKalDetector()
{
   Clear();
}

//=====================================================================
//* Initialize --------------------------------------------------------

void U4TPCKalDetector::Initialize()
{
   Double_t a, z, dens, radl;
   TMaterial &air  = *new TMaterial("TPCAir", "",
                                    a    = 14.00674*0.7+15.9994*0.3,
                                    z    = 7.3,
                                    dens = 1.205e-3,
                                    radl =  3.42e4, 0.);
   TMaterial &p10  = *new TMaterial("TPCP10", "",
                                    a    = 39.948*0.9+(12.011*0.2+1.00794*0.8)*0.1,
                                    z    = 16.4,
                                    dens = 0.749e-3,
                                    radl =  1.196e4*2, 0.);
   TMaterial &cfrp = *new TMaterial("TPCCFRP", "",
                                     a    = 12.0107,
                                     z    = 6.,
                                     dens = 0.1317,
                                     radl = 42.7/0.1317, 0.);

   JSFEnv &env = *gJSF->Env();
   Double_t ithick  = env.GetValue("J4TPC.InnerSupportTube.HalfThickness",2.1075) * 2;
   Double_t othick  = env.GetValue("J4TPC.OuterSupportTube.HalfThickness",4.1175) * 2;
   Int_t    nlayers = env.GetValue("J4TPC.NLayers", 200);
   Double_t rtub    = env.GetValue("J4.SupportTube.OuterRadius", 39.5);
   Double_t rmin    = rtub + ithick
                    + env.GetValue("J4TPC.FirstLayer.InnerRadius", 0.5);
   Double_t lhalf   = env.GetValue("J4.TPC.HalfZ", 260.0)
                    - env.GetValue("J4TPC.Endcap.HalfThickness", 1.0) * 2
                    - env.GetValue("J4TPC.PadPlane.HalfThickness", 1.5) * 2;
   Double_t outerr  = env.GetValue("J4.TPC.OuterRadius", 206.0);
   Double_t rstep   = (outerr - rmin - othick) / nlayers;

   Bool_t active = kTRUE;
   Bool_t dummy  = kFALSE;

   // create inner timing detector
   Add(new U4TPCMeasLayer(air, cfrp, rtub, lhalf, dummy));
   Add(new U4TPCMeasLayer(cfrp, p10, rtub+ithick, lhalf, dummy));

   // create measurement layers of central tracker
   Double_t r = rmin;
   for (Int_t layer=0; layer < nlayers; layer++) {
      Add(new U4TPCMeasLayer(p10, p10, r, lhalf, active));
      r += rstep;
   }
   Add(new U4TPCMeasLayer(p10, cfrp, outerr-othick, lhalf, dummy));
   Add(new U4TPCMeasLayer(cfrp, air, outerr, lhalf, dummy));
}

//=====================================================================
//* GetMeasLayerPtr ---------------------------------------------------

const U4TPCMeasLayer *U4TPCKalDetector::GetMeasLayerPtr(Int_t index) const
{
   return static_cast<U4TPCMeasLayer *>(At(index + 2));
}

