//*************************************************************************
//* ---------------------
//* U4VTXPadHit Class
//* ---------------------
//* (Description)
//*	Class for a single VTX hit.
//* (Requires)
//*	TObject
//* (Provides)
//*	U4VTXPadHit		: TObject containing single hit data
//* (Update Record)
//*	2004/08/07  A.Yamaguchi	Original version.
//*************************************************************************

#include <iostream>
#include "U4VTXHit.h"

ClassImp(U4VTXHit)

Int_t U4VTXHit::fgVTXHitSortBy = 0;

//=====================================================================
//* constructor -------------------------------------------------------

U4VTXHit::U4VTXHit(Int_t    layerID,
		   Double_t layerR,
		   Double_t phi,
		   Double_t z)
        : fDone(kFALSE), 
  	  fRPhi(layerR*phi),
          fDRPhi(0.),
          fZ(0.),
          fDZ(0.),
          fLayerID(layerID), 
          fLayerR(layerR ),
          fPhi(phi)
{
}

//=====================================================================
//* destructor -------------------------------------------------------

U4VTXHit::~U4VTXHit() 
{	
}


//=====================================================================
//* SetSortBy --------------------------------------------------------
                                                                                
void U4VTXHit::SetSortBy(Char_t *opt)
{
   switch (opt[2]) {
      case 'd':    // sort by Radius
      default:
         fgVTXHitSortBy = 0;
         break;
   }
}

//=====================================================================
//* IsSortable --------------------------------------------------------

Bool_t U4VTXHit::IsSortable() const
{
  return kTRUE;
}

//=====================================================================
//* Compare -----------------------------------------------------------

Int_t U4VTXHit::Compare(const TObject *obj) const
{
   switch (fgVTXHitSortBy) {
      case 0:
      default: {
         Int_t radius  = GetLayerID();
         Int_t radiuso = dynamic_cast<const U4VTXHit *>(obj)->GetLayerID();
         return (radius < radiuso ? -1 : (radius > radiuso ? 1 : 0));
      }
   }
}

//=====================================================================
//* Update ------------------------------------------------------------

void U4VTXHit::Update()
{
   if (fDone) return;
   fDone = kTRUE;
   CalculatePosition();
}

//=====================================================================
//* CalculatePosition -------------------------------------------------

void U4VTXHit::CalculatePosition()
{
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void U4VTXHit::DebugPrint(const Char_t *opt)
{
}
